Package: querychat
Title: Filter and Query Data Frames in 'shiny' Using an LLM Chat
        Interface
Version: 0.3.0
Authors@R: c(
    person("Garrick", "Aden-Buie", , "garrick@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-7111-0077")),
    person("Joe", "Cheng", , "joe@posit.co", role = c("aut", "ccp")),
    person("Carson", "Sievert", , "carson@posit.co", role = "aut",
           comment = c(ORCID = "0000-0002-4958-2844")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Adds an LLM-powered chatbot to your 'shiny' app, that can
    turn your users' natural language questions into 'SQL' queries that
    run against your data, and return the result as a reactive data frame.
    Use it to drive reactive calculations, visualizations, downloads, and
    more.
License: MIT + file LICENSE
URL: https://posit-dev.github.io/querychat/r/,
        https://posit-dev.github.io/querychat/,
        https://github.com/posit-dev/querychat
BugReports: https://github.com/posit-dev/querychat/issues
Depends: R (>= 4.1.0)
Imports: bsicons, bslib (>= 0.11.0), cli, DBI, ellmer (>= 0.4.1),
        htmltools, lifecycle, promises, R6, rlang (>= 1.1.0), S7,
        shiny, shinychat (>= 0.4.0), utils, whisker
Suggests: dbplyr, dplyr, DT, duckdb, ggsql, knitr, palmerpenguins,
        rmarkdown, RSQLite, rsvg, shinytest2, testthat (>= 3.0.0), V8,
        withr
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-06-01 20:54:39 UTC; cpsievert
Author: Garrick Aden-Buie [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7111-0077>),
  Joe Cheng [aut, ccp],
  Carson Sievert [aut] (ORCID: <https://orcid.org/0000-0002-4958-2844>),
  Posit Software, PBC [cph, fnd]
Maintainer: Garrick Aden-Buie <garrick@posit.co>
Repository: CRAN
Date/Publication: 2026-06-01 22:00:02 UTC
Built: R 4.5.2; ; 2026-06-02 00:59:41 UTC; unix

# Generated by roxygen2: do not edit by hand

export(DBISource)
export(DataFrameSource)
export(DataSource)
export(QueryChat)
export(TblSqlSource)
export(querychat)
export(querychat_app)
export(querychat_data_source)
export(querychat_greeting)
export(querychat_init)
export(querychat_server)
export(querychat_sidebar)
export(querychat_ui)
if (getRversion() < "4.3.0") importFrom("S7", "@")
import(rlang)
importFrom(R6,R6Class)
importFrom(bslib,sidebar)
importFrom(lifecycle,deprecated)

# QueryChat Modules Example

This example demonstrates how to use QueryChat within Shiny modules, following standard Shiny module patterns.

## Key Concepts

### Module UI Function

In a Shiny module UI function, you wrap the QueryChat ID with the namespace function `ns()`:

```r
module_ui <- function(id) {
  ns <- NS(id)
  card(
    qc$sidebar(id = ns("qc-ui"))  # Wrap ID with ns()
  )
}
```

### Module Server Function

In the corresponding server function, you pass the **unwrapped** ID to `qc$server()`:

```r
module_server <- function(id) {
  moduleServer(id, function(input, output, session) {
    qc_vals <- qc$server(id = "qc-ui")  # Use unwrapped ID
    # ... rest of server logic
  })
}
```

## Why This Pattern?

This follows the established Shiny module pattern where:

1. **UI functions** namespace all IDs using `ns()` to avoid conflicts when multiple instances exist
2. **Server functions** receive the unwrapped ID and use it to connect to the corresponding UI

This is the same pattern used for any Shiny component in a module, and QueryChat now supports it seamlessly.

## Benefits

- **Multiple instances**: You can have multiple QueryChat explorers in the same app
- **Familiar pattern**: Uses standard Shiny module conventions
- **Clean isolation**: Each module instance has its own reactive state

## Running the Example

From the R console:

```r
shiny::runApp(system.file("examples-shiny/03-module-app", package = "querychat"))
```

Or navigate to this directory and run:

```bash
Rscript app.R
```

# querychat <a href="https://posit-dev.github.io/querychat/r/"><img src="man/figures/logo.png" align="right" height="138" alt="querychat website" /></a>

<!-- badges: start -->
[![R-CMD-check](https://github.com/posit-dev/querychat/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/posit-dev/querychat/actions/workflows/R-CMD-check.yaml)
[![CRAN status](https://www.r-pkg.org/badges/version/querychat)](https://CRAN.R-project.org/package=querychat)
[![querychat status badge](https://posit-dev.r-universe.dev/querychat/badges/version)](https://posit-dev.r-universe.dev/querychat)
<!-- badges: end -->

querychat facilitates safe and reliable natural language exploration of tabular data, powered by SQL and large language models (LLMs). For users, it offers an intuitive web application where they can quickly ask questions of their data and receive verifiable data-driven answers. As a developer, you can access the chat UI component, generated SQL queries, and filtered data to build custom applications that integrate natural language querying into your data workflows.

## Installation

Install the stable release from CRAN:

```r
install.packages("querychat")
```

Or the development version from GitHub:

```r
# install.packages("pak")
pak::pak("posit-dev/querychat/pkg-r")
```

## Quick start

The quickest way to start chatting with your data is via `querychat_app()`, which provides a fully polished Shiny app. It requires a data [data source](https://posit-dev.github.io/querychat/r/articles/data-sources.html) (e.g., data.frame, database connection, etc.) and optionally other parameters (e.g. the LLM `client` [model](https://posit-dev.github.io/querychat/r/articles/models.html)).

```r
library(querychat)
library(palmerpenguins)

querychat_app(penguins, client = "openai/gpt-4.1")
```

Once running (which requires an API key[^api-key]), you'll notice 3 main views:

[^api-key]: By default, querychat uses OpenAI to power the chat experience. So, for this example to work, you'll need [an OpenAI API key](https://platform.openai.com/). See the [Models](https://posit-dev.github.io/querychat/r/articles/models.html) article for details on how to set up credentials for other model providers.

1. A sidebar chat with suggestions on where to start exploring.
2. A data table that updates to reflect filtering and sorting queries.
3. The SQL query behind the data table, for transparency and reproducibility.

![](man/figures/quickstart.png){alt="Screenshot of querychat's app with the penguins dataset." class="rounded shadow"}

Suppose we pick a suggestion like "Show me Adelie penguins". Since this is a filtering operation, both the data table and SQL query update accordingly.

![](man/figures/quickstart-filter.png){alt="Screenshot of the querychat's app with the penguins dataset filtered." class="rounded shadow"}

querychat can also handle more general questions about the data that require calculations and aggregations. For example, we can ask "What is the average bill length by species?". The LLM will generate the SQL query to perform the calculation, querychat will execute it, and return the result in the chat:

![](man/figures/quickstart-summary.png){alt="Screenshot of the querychat's app with a summary statistic inlined in the chat." class="rounded shadow"}

querychat can also create visualizations, powered by [ggsql](https://ggsql.org/).
With the [visualization tool](https://posit-dev.github.io/querychat/r/articles/visualize.html) enabled, ask for a chart and it appears inline in the conversation:

![](man/figures/viz-bar-chart.png){alt="Bar chart showing average body mass by penguin species." class="rounded shadow"}

## Custom apps

querychat is designed to be highly extensible -- it provides programmatic access to the chat interface, the filtered/sorted data frame, SQL queries, and more.
This makes it easy to build custom web apps that leverage natural language interaction with your data.
For example, [here](https://github.com/posit-conf-2025/llm/blob/main/_solutions/25_querychat/25_querychat_02-end-app.R)'s a bespoke app for exploring Airbnb listings in Ashville, NC:

![](man/figures/airbnb.png){alt="A custom app for exploring Airbnb listings, powered by querychat." class="shadow rounded mb-3"}

To learn more, see [Build an app](https://posit-dev.github.io/querychat/r/articles/build.html) for a step-by-step guide.

## How it works

querychat uses LLMs to translate natural language into SQL queries. Models of all sizes, from small ones you can run locally to large frontier models from major AI providers, are remarkably effective at this task. But even the best models need to understand your data's overall structure to perform well.

To address this, querychat includes schema metadata -- column names, types, ranges, categorical values -- in the LLM's [system prompt](https://posit-dev.github.io/querychat/r/articles/context.html). Importantly, querychat **does not** send raw data to the LLM; it shares only enough structural information for the model to generate accurate queries. When the LLM produces a query, querychat executes it in a SQL database (DuckDB[^duckdb], by default) to obtain precise results.

This design makes querychat reliable, safe, and reproducible:

- **Reliable**: query results come from a real database, not LLM-generated summaries -- so outputs are precise, verifiable, and less vulnerable to hallucination[^hallucination].
- **Safe**: querychat's tools are read-only by design, avoiding destructive actions on your data.[^permissions]
- **Reproducible**: generated SQL can be exported and re-run in other environments, so your analysis isn't locked into a single tool.

::: {.alert .alert-warning}
**Data privacy**

See the [Provide context](https://posit-dev.github.io/querychat/r/articles/context.html) and [Tools](https://posit-dev.github.io/querychat/r/articles/tools.html) articles for more details on exactly what information is provided to the LLM and how customize it.
:::

[^duckdb]: DuckDB is extremely fast and has a surprising number of [statistical functions](https://duckdb.org/docs/stable/sql/functions/aggregates.html#statistical-aggregates).

[^hallucination]: The [query tool](https://posit-dev.github.io/querychat/r/articles/tools.html) gives query results to the model for context and interpretation. Thus, there is *some* potential that the model to mis-interpret those results.

[^permissions]: To fully guarantee no destructive actions on your production database, ensure querychat's database permissions are read-only.

## Next steps

From here, you might want to learn more about:

- [Models](https://posit-dev.github.io/querychat/r/articles/models.html): customize the LLM behind querychat.
- [Data sources](https://posit-dev.github.io/querychat/r/articles/data-sources.html): different data sources you can use with querychat.
- [Provide context](https://posit-dev.github.io/querychat/r/articles/context.html): provide the LLM with the context it needs to work well.
- [Visualizations](https://posit-dev.github.io/querychat/r/articles/visualize.html): create inline charts powered by ggsql.
- [Build an app](https://posit-dev.github.io/querychat/r/articles/build.html): design a custom Shiny app around querychat.
- [Greet users](https://posit-dev.github.io/querychat/r/articles/greet.html): create welcoming onboarding experiences.
- [Tools](https://posit-dev.github.io/querychat/r/articles/tools.html): understand what querychat can do under the hood.

# Database Setup Examples for querychat

This document provides examples of how to set up querychat with various database types.

## SQLite

```r
library(DBI)
library(RSQLite)
library(querychat)

# Connect to SQLite database
conn <- dbConnect(RSQLite::SQLite(), "path/to/your/database.db")

# Create QueryChat instance
qc <- QueryChat$new(
  conn,
  "your_table_name",
  greeting = "Welcome! Ask me about your data.",
  data_description = "Description of your data..."
)

# Launch the app
qc$app()
```

## PostgreSQL

```r
library(DBI)
library(RPostgreSQL)  # or library(RPostgres)
library(querychat)

# Connect to PostgreSQL
conn <- dbConnect(
  RPostgreSQL::PostgreSQL(),  # or RPostgres::Postgres()
  dbname = "your_database",
  host = "localhost",
  port = 5432,
  user = "your_username",
  password = "your_password"
)

# Create QueryChat instance
qc <- QueryChat$new(conn, "your_table_name")

# Launch the app
qc$app()
```

## MySQL

```r
library(DBI)
library(RMySQL)
library(querychat)

# Connect to MySQL
conn <- dbConnect(
  RMySQL::MySQL(),
  dbname = "your_database",
  host = "localhost",
  user = "your_username",
  password = "your_password"
)

# Create QueryChat instance
qc <- QueryChat$new(conn, "your_table_name")

# Launch the app
qc$app()
```

## Connection Management

When using database sources in custom Shiny apps, make sure to properly manage connections:

```r
server <- function(input, output, session) {
  # Initialize QueryChat server
  qc$server()

  # Your custom outputs here
  output$table <- renderTable(qc$df())

  # Clean up connection when session ends
  session$onSessionEnded(function() {
    if (dbIsValid(conn)) {
      dbDisconnect(conn)
    }
  })
}
```

## Security Considerations

- Only SELECT queries are allowed - no INSERT, UPDATE, or DELETE operations
- All SQL queries are visible to users for transparency
- Use appropriate database user permissions (read-only recommended)
- Consider connection pooling for production applications
- Validate that users only have access to intended tables

## Error Handling

The database source implementation includes robust error handling:

- Validates table existence during creation
- Handles database connection issues gracefully
- Provides informative error messages for invalid queries
- Falls back gracefully when statistical queries fail

## Performance Tips

- Use appropriate database indexes for columns commonly used in queries
- Consider limiting row counts for very large tables
- Database connections are reused for better performance
- Schema information is cached to avoid repeated metadata queries

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## -----------------------------------------------------------------------------
# library(querychat)
# library(palmerpenguins)
# 
# qc <- querychat(mtcars)
# qc$app()  # Launch the app

## -----------------------------------------------------------------------------
# library(DBI)
# library(duckdb)
# library(querychat)
# 
# # Connect to a DuckDB database file
# con <- dbConnect(duckdb::duckdb(), dbdir = "my_database.duckdb")
# 
# qc <- querychat(con, "my_table")
# qc$app()  # Launch the app
# 
# # Don't forget to disconnect when done
# # dbDisconnect(con)

## -----------------------------------------------------------------------------
# library(DBI)
# library(RSQLite)
# library(querychat)
# 
# # Connect to a SQLite database file
# con <- dbConnect(RSQLite::SQLite(), "my_database.db")
# 
# qc <- querychat(con, "my_table")
# qc$app()  # Launch the app
# 
# # Don't forget to disconnect when done
# # dbDisconnect(con)

## -----------------------------------------------------------------------------
# library(DBI)
# library(RPostgres)
# library(querychat)
# 
# # Connect to PostgreSQL
# con <- dbConnect(
#   RPostgres::Postgres(),
#   host = "localhost",
#   port = 5432,
#   dbname = "mydatabase",
#   user = "myuser",
#   password = "mypassword"
# )
# 
# qc <- querychat(con, "my_table")
# qc$app()  # Launch the app
# 
# # Don't forget to disconnect when done
# # dbDisconnect(con)

## -----------------------------------------------------------------------------
# library(DBI)
# library(RMariaDB)
# library(querychat)
# 
# # Connect to MySQL
# con <- dbConnect(
#   RMariaDB::MariaDB(),
#   host = "localhost",
#   port = 3306,
#   dbname = "mydatabase",
#   user = "myuser",
#   password = "mypassword"
# )
# 
# qc <- querychat(con, "my_table")
# qc$app()  # Launch the app
# 
# # Don't forget to disconnect when done
# # dbDisconnect(con)

## -----------------------------------------------------------------------------
# library(DBI)
# library(duckdb)
# 
# con <- dbConnect(duckdb::duckdb(), dbdir = "my_database.duckdb")
# 
# # Write a data frame to the database
# dbWriteTable(con, "penguins", penguins)
# 
# # Or from CSV
# duckdb::duckdb_read_csv(con, "my_table", "path/to/your/file.csv")

library(querychat)
library(palmerpenguins)

# Create a QueryChat object and generate a complete app with $app()
qc <- QueryChat$new(penguins)
qc$app()

# That's it! The app includes:
# - A sidebar with the chat interface
# - SQL query display with syntax highlighting
# - Data table showing filtered results
# - Reset button to clear queries

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(querychat)
# library(DT)
# library(palmerpenguins)
# 
# # Step 1: Initialize QueryChat
# qc <- QueryChat$new(penguins)
# 
# # Step 2: Add UI component
# ui <- page_sidebar(
#   sidebar = qc$sidebar(),
#   card(
#     card_header("Data Table"),
#     dataTableOutput("table")
#   ),
#   card(
#     fill = FALSE,
#     card_header("SQL Query"),
#     verbatimTextOutput("sql")
#   )
# )
# 
# # Step 3: Use reactive values in server
# server <- function(input, output, session) {
#   qc_vals <- qc$server()
# 
#   output$table <- renderDataTable({
#     datatable(qc_vals$df(), fillContainer = TRUE)
#   })
# 
#   output$sql <- renderText({
#     qc_vals$sql() %||% "SELECT * FROM penguins"
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(querychat)
# 
# # Global scope - create QueryChat without data source
# qc <- QueryChat$new(NULL, "users")
# 
# ui <- page_sidebar(
#   sidebar = qc$sidebar(),
#   card(dataTableOutput("table"))
# )
# 
# server <- function(input, output, session) {
#   # Server scope - create connection with session credentials
#   conn <- get_user_connection(session)
#   qc_vals <- qc$server(data_source = conn)
# 
#   output$table <- renderDataTable({
#     qc_vals$df()
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# qc_vals <- qc$server()
# 
# output$table <- renderDataTable({
#   qc_vals$df()  # Returns filtered/sorted data
# })

## -----------------------------------------------------------------------------
# qc_vals <- qc$server()
# 
# output$current_query <- renderText({
#   qc_vals$sql() %||% "SELECT * FROM penguins"
# })

## -----------------------------------------------------------------------------
# qc_vals <- qc$server()
# 
# output$card_title <- renderText({
#   qc_vals$title() %||% "All Data"
# })

## -----------------------------------------------------------------------------
# library(querychat)
# library(palmerpenguins)
# 
# qc <- QueryChat$new(penguins)
# 
# ui <- page_sidebar(
#   sidebar = sidebar(
#     qc$ui(),  # Chat component
#     actionButton("reset", "Reset Filters", class = "w-100"),
#     fillable = TRUE,
#     width = 300
#   ),
#   # Main content here
# )

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(querychat)
# library(DT)
# library(plotly)
# library(palmerpenguins)
# 
# qc <- QueryChat$new(penguins, client = "claude/claude-sonnet-4-5")
# 
# ui <- page_sidebar(
#   sidebar = qc$sidebar(),
#   card(
#     card_header("Data Table"),
#     dataTableOutput("table")
#   ),
#   card(
#     card_header("Body Mass by Species"),
#     plotlyOutput("mass_plot")
#   )
# )
# 
# server <- function(input, output, session) {
#   qc_vals <- qc$server()
# 
#   output$table <- renderDataTable({
#     datatable(qc_vals$df(), fillContainer = TRUE)
#   })
# 
#   output$mass_plot <- renderPlotly({
#     ggplot(qc_vals$df(), aes(x = body_mass_g, fill = species)) +
#       geom_density(alpha = 0.4) +
#       theme_minimal()
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(DT)
# library(plotly)
# library(palmerpenguins)
# library(dplyr)
# library(bsicons)
# library(querychat)
# 
# 
# qc <- QueryChat$new(penguins)
# 
# ui <- page_sidebar(
#   title = "Palmer Penguins Analysis",
#   class = "bslib-page-dashboard",
#   sidebar = qc$sidebar(),
#   layout_column_wrap(
#     width = 1 / 3,
#     fill = FALSE,
#     value_box(
#       title = "Total Penguins",
#       value = textOutput("count"),
#       showcase = bs_icon("piggy-bank"),
#       theme = "primary"
#     ),
#     value_box(
#       title = "Species Count",
#       value = textOutput("species_count"),
#       showcase = bs_icon("bookmark-star"),
#       theme = "success"
#     ),
#     value_box(
#       title = "Avg Body Mass",
#       value = textOutput("avg_mass"),
#       showcase = bs_icon("speedometer"),
#       theme = "info"
#     )
#   ),
#   layout_columns(
#     card(
#       card_header(textOutput("table_title")),
#       DT::dataTableOutput("data_table")
#     ),
#     card(
#       card_header("Species Distribution"),
#       plotlyOutput("species_plot")
#     )
#   ),
#   layout_columns(
#     card(
#       card_header("Bill Length Distribution"),
#       plotlyOutput("bill_length_dist")
#     ),
#     card(
#       card_header("Body Mass by Species"),
#       plotlyOutput("mass_by_species")
#     )
#   )
# )
# 
# server <- function(input, output, session) {
#   qc_vals <- qc$server()
# 
#   output$count <- renderText({
#     nrow(qc_vals$df())
#   })
# 
#   output$species_count <- renderText({
#     length(unique(qc_vals$df()$species))
#   })
# 
#   output$avg_mass <- renderText({
#     avg <- mean(qc_vals$df()$body_mass_g, na.rm = TRUE)
#     paste0(round(avg, 0), "g")
#   })
# 
#   output$table_title <- renderText({
#     qc_vals$title() %||% "All Penguins"
#   })
# 
#   output$data_table <- DT::renderDataTable({
#     DT::datatable(
#       qc_vals$df(),
#       fillContainer = TRUE,
#       options = list(
#         scrollX = TRUE,
#         pageLength = 10,
#         dom = "ti"
#       )
#     )
#   })
# 
#   output$species_plot <- renderPlotly({
#     plot_ly(
#       count(qc_vals$df(), species),
#       x = ~species,
#       y = ~n,
#       type = "bar",
#       marker = list(color = c("#1f77b4", "#ff7f0e", "#2ca02c"))
#     )
#   })
# 
#   output$bill_length_dist <- renderPlotly({
#     plot_ly(
#       qc_vals$df(),
#       x = ~bill_length_mm,
#       type = "histogram",
#       nbinsx = 30,
#       marker = list(color = "#1f77b4", opacity = 0.7)
#     )
#   })
# 
#   output$mass_by_species <- renderPlotly({
#     plot_ly(
#       qc_vals$df(),
#       x = ~species,
#       y = ~body_mass_g,
#       color = ~sex,
#       type = "box",
#       colors = c("#1f77b4", "#ff7f0e")
#     )
#   })
# }
# 
# shinyApp(ui = ui, server = server)

## -----------------------------------------------------------------------------
# ui <- page_sidebar(
#   sidebar = sidebar(
#     qc$ui(),
#     hr(),
#     actionButton("reset", "Reset Filters")
#   ),
#   # Main content
#   card(dataTableOutput("table"))
# )
# 
# server <- function(input, output, session) {
#   qc_vals <- qc$server()
# 
#   output$table <- renderDataTable({
#     qc_vals$df()
#   })
# 
#   observeEvent(input$reset, {
#     qc_vals$sql("")
#     qc_vals$title(NULL)
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(palmerpenguins)
# library(titanic)
# library(querychat)
# 
# qc_penguins <- QueryChat$new(penguins)
# qc_titanic <- QueryChat$new(titanic_train)
# 
# ui <- page_navbar(
#   title = "Multiple Datasets",
#   sidebar = sidebar(
#     id = "sidebar",
#     conditionalPanel(
#       "input.navbar == 'Penguins'",
#       qc_penguins$ui()
#     ),
#     conditionalPanel(
#       "input.navbar == 'Titanic'",
#       qc_titanic$ui()
#     )
#   ),
#   nav_panel(
#     "Penguins",
#     card(dataTableOutput("penguins_table"))
#   ),
#   nav_panel(
#     "Titanic",
#     card(dataTableOutput("titanic_table"))
#   ),
#   id = "navbar"
# )
# 
# server <- function(input, output, session) {
#   qc_penguins_vals <- qc_penguins$server()
#   qc_titanic_vals <- qc_titanic$server()
# 
#   output$penguins_table <- renderDataTable({
#     qc_penguins_vals$df()
#   })
# 
#   output$titanic_table <- renderDataTable({
#     qc_titanic_vals$df()
#   })
# }
# 
# shinyApp(ui, server)

library(shiny)
library(bslib)
library(querychat)
library(palmerpenguins)

# Define a custom greeting for the penguins app
greeting <- r"(
# Welcome to the Palmer Penguins Explorer! 🐧

I can help you explore and analyze the Palmer Penguins dataset. Ask me questions
about the penguins, and I'll generate SQL queries to get the answers.

##### Explore the data
- <span class="suggestion">Show me the first 10 rows of the penguins dataset</span>
- <span class="suggestion">Which species has the largest body mass?</span>

##### Analyze statistics
- <span class="suggestion">What's the average bill length by species?</span>
- <span class="suggestion">Create a summary of measurements grouped by species and island</span>
)"

# Create QueryChat object with custom options
qc <- QueryChat$new(
  penguins,
  greeting = greeting,
  data_description = paste(
    "The Palmer Penguins dataset contains measurements of bill",
    "dimensions, flipper length, body mass, sex, and species",
    "(Adelie, Chinstrap, and Gentoo) collected from three islands in",
    "the Palmer Archipelago, Antarctica."
  ),
  extra_instructions = paste(
    "When showing results, always explain what the data represents",
    "and highlight any interesting patterns you observe."
  )
)

# Define custom UI with sidebar
ui <- page_sidebar(
  title = "Palmer Penguins Chat Explorer",
  sidebar = qc$sidebar(),

  card(
    fill = FALSE,
    card_header("Current SQL Query"),
    verbatimTextOutput("sql_query")
  ),

  card(
    full_screen = TRUE,
    card_header(
      "Current Data View",
      tooltip(
        bsicons::bs_icon("question-circle-fill", class = "mx-1"),
        "The table below shows the current filtered data based on your chat queries"
      ),
      tooltip(
        bsicons::bs_icon("info-circle-fill"),
        "The penguins dataset contains measurements on 344 penguins."
      )
    ),
    DT::DTOutput("data_table"),
    card_footer(
      markdown(
        "Data source: [palmerpenguins package](https://allisonhorst.github.io/palmerpenguins/)"
      )
    )
  )
)

# Define server logic
server <- function(input, output, session) {
  # Initialize QueryChat server
  qc_vals <- qc$server()

  # Render the data table
  output$data_table <- DT::renderDT(
    {
      qc_vals$df()
    },
    fillContainer = TRUE,
    options = list(pageLength = 25, scrollX = TRUE)
  )

  # Render the SQL query
  output$sql_query <- renderText({
    query <- qc_vals$sql()
    if (is.null(query) || !nzchar(query)) {
      "No filter applied - showing all data"
    } else {
      query
    }
  })
}

shinyApp(ui = ui, server = server)

library(shiny)
library(bslib)
library(querychat)
library(palmerpenguins)

qc <- QueryChat$new(
  penguins,
  tools = c("update", "query", "visualize"),
  data_description = paste(
    "The Palmer Penguins dataset contains measurements of bill",
    "dimensions, flipper length, body mass, sex, and species",
    "(Adelie, Chinstrap, and Gentoo) collected from three islands in",
    "the Palmer Archipelago, Antarctica."
  )
)

ui <- page_sidebar(
  title = "querychat viz demo",
  sidebar = qc$sidebar(width = 400, open = TRUE, position = "right"),
  card(
    full_screen = TRUE,
    card_header("Data"),
    DT::DTOutput("dt")
  )
)

server <- function(input, output, session) {
  qc_vals <- qc$server()
  output$dt <- DT::renderDT(qc_vals$df(), fillContainer = TRUE)
}

shinyApp(ui, server)

library(shiny)
library(bslib)
library(querychat)
library(DBI)
library(RSQLite)
library(palmerpenguins)

# Create a sample SQLite database for demonstration
# In a real app, you would connect to your existing database
temp_db <- tempfile(fileext = ".db")
onStop(function() {
  if (file.exists(temp_db)) {
    unlink(temp_db)
  }
})

conn <- dbConnect(RSQLite::SQLite(), temp_db)

# Create sample data in the database
dbWriteTable(conn, "penguins", palmerpenguins::penguins, overwrite = TRUE)

# Define a custom greeting for the database app
greeting <- "
# Welcome to the Database Query Assistant! 🐧

I can help you explore and analyze the Palmer Penguins dataset from the connected database.
Ask me questions about the penguins, and I'll generate SQL queries to get the answers.

##### Explore the data
- <span class=\"suggestion\">Show me the first 10 rows of the penguins dataset</span>
- <span class=\"suggestion\">Which species has the largest body mass?</span>

##### Analyze statistics
- <span class=\"suggestion\">What's the average bill length by species?</span>
- <span class=\"suggestion\">Create a summary of measurements grouped by species and island</span>
"

# Create QueryChat object with database connection
qc <- QueryChat$new(
  conn,
  "penguins",
  greeting = greeting,
  data_description = "This database contains the Palmer Penguins dataset with measurements of bill dimensions, flipper length, body mass, sex, and species (Adelie, Chinstrap, and Gentoo) collected from three islands in the Palmer Archipelago, Antarctica.",
  extra_instructions = "When showing results, always explain what the data represents and highlight any interesting patterns you observe."
)

ui <- page_sidebar(
  title = "Database Query Chat",
  sidebar = qc$sidebar(),

  card(
    fill = FALSE,
    card_header("Current SQL Query"),
    verbatimTextOutput("sql_query")
  ),

  card(
    full_screen = TRUE,
    card_header(
      "Current Data View",
      tooltip(
        bsicons::bs_icon("question-circle-fill", class = "mx-1"),
        "The table below shows the current filtered data based on your chat queries"
      ),
      tooltip(
        bsicons::bs_icon("info-circle-fill"),
        "The penguins dataset contains measurements on 344 penguins."
      )
    ),
    DT::DTOutput("data_table"),
    card_footer(
      markdown(
        "Data source: [palmerpenguins package](https://allisonhorst.github.io/palmerpenguins/)"
      )
    )
  )
)

server <- function(input, output, session) {
  qc_vals <- qc$server()

  output$data_table <- DT::renderDT(
    {
      qc_vals$df()
    },
    fillContainer = TRUE,
    options = list(pageLength = 10, scrollX = TRUE)
  )

  output$sql_query <- renderText({
    query <- qc_vals$sql()
    if (is.null(query) || !nzchar(query)) {
      "No filter applied - showing all data"
    } else {
      query
    }
  })
}

shinyApp(ui = ui, server = server)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## -----------------------------------------------------------------------------
# library(querychat)
# library(palmerpenguins)

## -----------------------------------------------------------------------------
# qc <- querychat(
#   penguins,
#   greeting = "greeting.md"
# )
# qc$app()  # Launch the app

## -----------------------------------------------------------------------------
# library(querychat)
# 
# # Create QueryChat object with your dataset
# qc <- querychat(penguins)
# 
# # Generate a greeting (this calls the LLM)
# greeting_text <- qc$generate_greeting(echo = "text")
# #> Hello! I'm here to help you explore and analyze the penguins dataset.
# #> Here are some example prompts you can try:
# #> ...
# 
# # Save it for reuse
# writeLines(greeting_text, "penguins_greeting.md")

## -----------------------------------------------------------------------------
# # Then use the saved greeting in your app
# querychat_app(
#   penguins,
#   greeting = "penguins_greeting.md"
# )

library(shiny)
library(bslib)
library(querychat)
library(palmerpenguins)

# Define a custom greeting for the penguins app
greeting <- r"(
# Welcome to the Palmer Penguins Explorer! 🐧

I can help you explore and analyze the Palmer Penguins dataset. Ask me questions
about the penguins, and I'll generate SQL queries to get the answers.

##### Explore the data
- <span class="suggestion">Show me the first 10 rows of the penguins dataset</span>
- <span class="suggestion">What's the average bill length by species?</span>
- <span class="suggestion">Which species has the largest body mass?</span>
)"

# Create QueryChat object with custom options
qc <- QueryChat$new(
  penguins,
  greeting = greeting,
  data_description = paste(
    "The Palmer Penguins dataset contains measurements of bill",
    "dimensions, flipper length, body mass, sex, and species",
    "(Adelie, Chinstrap, and Gentoo) collected from three islands in",
    "the Palmer Archipelago, Antarctica."
  )
)

# Module UI function
# This demonstrates the standard Shiny module pattern where:
# - The module ID is wrapped with ns() in the UI function
# - The same ID (unwrapped) is used in the corresponding server function
module_ui <- function(id) {
  ns <- NS(id)
  layout_sidebar(
    sidebar = qc$sidebar(id = ns("qc-ui")), # Pass namespaced ID to QueryChat
    padding = 0,
    navset_card_tab(
      title = "Data Explorer",
      nav_panel(
        "Data View",
        DT::DTOutput(ns("data_table"))
      ),
      nav_panel(
        "SQL Query",
        verbatimTextOutput(ns("sql_query"))
      )
    )
  )
}

# Module server function
module_server <- function(id) {
  moduleServer(id, function(input, output, session) {
    # Initialize QueryChat server with the same ID (unwrapped)
    # This connects to the UI initialized with id = ns("qc-ui")
    qc_vals <- qc$server(id = "qc-ui")

    # Render the data table
    output$data_table <- DT::renderDT(
      {
        qc_vals$df()
      },
      fillContainer = TRUE,
      options = list(pageLength = 25, scrollX = TRUE)
    )

    # Render the SQL query
    output$sql_query <- renderText({
      query <- qc_vals$sql()
      if (is.null(query) || !nzchar(query)) {
        "No filter applied - showing all data"
      } else {
        query
      }
    })
  })
}

# Define UI with multiple module instances
ui <- page_sidebar(
  title = "QueryChat Modules Example",
  sidebar = sidebar(
    "This example demonstrates using QueryChat within Shiny modules.",
    markdown(
      "Each module instance has its own QueryChat sidebar and data explorer.

      **UI:** `qc$sidebar(id = ns(\"qc-ui\"))` wraps the ID with the namespace function

      **Server:** `qc$server(id = \"qc-ui\")` uses the unwrapped ID"
    )
  ),
  class = "p-0",
  navset_card_underline(
    wrapper = \(...) card_body(..., padding = 0, border_radius = 0),
    nav_panel("Explorer 1", module_ui("module1")),
    nav_panel("Explorer 2", module_ui("module2"))
  )
)

# Define server logic
server <- function(input, output, session) {
  # Initialize both module instances
  module_server("module1")
  module_server("module2")
}

shinyApp(ui = ui, server = server)

#' Data Frame Source
#'
#' @description
#' A DataSource implementation that wraps a data frame using DuckDB or SQLite
#' for SQL query execution.
#'
#' @details
#' This class creates an in-memory database connection and registers the
#' provided data frame as a table. All SQL queries are executed against this
#' database table. See [DBISource] for the full description of available
#' methods.
#'
#' By default, DataFrameSource uses the first available engine from duckdb
#' (checked first) or RSQLite. You can explicitly set the `engine` parameter to
#' choose between `"duckdb"` or `"sqlite"`, or set the global option
#' `querychat.DataFrameSource.engine` to choose the default engine for all
#' DataFrameSource instances. At least one of these packages must be installed.
#'
#' @examplesIf rlang::is_installed("duckdb") || rlang::is_installed("RSQLite")
#' # Create a data frame source (uses first available: duckdb or sqlite)
#' df_source <- DataFrameSource$new(mtcars, "mtcars")
#'
#' # Get database type
#' df_source$get_db_type()  # Returns "DuckDB" or "SQLite"
#'
#' # Execute a query
#' result <- df_source$execute_query("SELECT * FROM mtcars WHERE mpg > 25")
#'
#' # Explicitly choose an engine
#' df_sqlite <- DataFrameSource$new(mtcars, "mtcars", engine = "sqlite")
#'
#' # Clean up when done
#' df_source$cleanup()
#' df_sqlite$cleanup()
#'
#' @export
DataFrameSource <- R6::R6Class(
  "DataFrameSource",
  inherit = DBISource,
  private = list(
    conn = NULL
  ),
  public = list(
    #' @description
    #' Create a new DataFrameSource
    #'
    #' @param df A data frame.
    #' @param table_name Name to use for the table in SQL queries. Must be a
    #'   valid table name (start with letter, contain only letters, numbers,
    #'   and underscores)
    #' @param engine Database engine to use: "duckdb" or "sqlite". Set the
    #'   global option `querychat.DataFrameSource.engine` to specify the default
    #'   engine for all instances. If NULL (default), uses the first available
    #'   engine from duckdb or RSQLite (in that order).
    #'
    #' @return A new DataFrameSource object
    initialize = function(
      df,
      table_name,
      engine = getOption("querychat.DataFrameSource.engine", NULL)
    ) {
      check_data_frame(df)
      check_sql_table_name(table_name)

      engine <- engine %||% get_default_dataframe_engine()
      engine <- tolower(engine)
      arg_match(engine, c("duckdb", "sqlite"))

      self$table_name <- table_name
      private$colnames <- colnames(df)

      # Create in-memory connection and register the data frame
      if (engine == "duckdb") {
        check_installed("duckdb")

        private$conn <- DBI::dbConnect(duckdb::duckdb(), dbdir = ":memory:")

        duckdb::duckdb_register(
          private$conn,
          table_name,
          df,
          experimental = FALSE
        )

        DBI::dbExecute(
          private$conn,
          r"(
-- extensions: lock down supply chain + auto behaviors
SET allow_community_extensions = false;
SET allow_unsigned_extensions = false;
SET autoinstall_known_extensions = false;
SET autoload_known_extensions = false;

-- external I/O: block file/database/network access from SQL
SET enable_external_access = false;
SET disabled_filesystems = 'LocalFileSystem';

-- freeze configuration so user SQL can't relax anything
SET lock_configuration = true;
        )"
        )
      } else if (engine == "sqlite") {
        check_installed("RSQLite")
        private$conn <- DBI::dbConnect(RSQLite::SQLite(), ":memory:")
        DBI::dbWriteTable(private$conn, table_name, df)
      }
    }
  )
)

get_default_dataframe_engine <- function() {
  if (is_installed("duckdb")) {
    return("duckdb")
  }
  if (is_installed("RSQLite")) {
    return("sqlite")
  }
  cli::cli_abort(c(
    "No compatible database engine installed for DataFrameSource",
    "i" = "Install either {.pkg duckdb} or {.pkg RSQLite}:",
    " " = "{.run install.packages(\"duckdb\")}",
    " " = "{.run install.packages(\"RSQLite\")}"
  ))
}

interpolate_package <- function(path, ..., .envir = parent.frame()) {
  # This helper replicates ellmer::interpolate_package() to work with load_all()
  stopifnot(
    "`path` must be a single string" = is.character(path),
    "`path` must be a single string" = length(path) == 1
  )

  path <- system.file("prompts", path, package = "querychat")
  stopifnot(
    "`path` does not exist" = nzchar(path),
    "`path` does not exist" = file.exists(path)
  )

  ellmer::interpolate_file(path, ..., .envir = .envir)
}


as_querychat_client <- function(client = NULL) {
  if (is.null(client)) {
    client <- querychat_client_option()
  }

  if (is.null(client)) {
    # Use OpenAI with ellmer's default model
    return(ellmer::chat_openai())
  }

  if (is_function(client)) {
    # `client` as a function was the first interface we supported and expected
    # `system_prompt` as an argument. This avoids breaking existing code.
    client <- client(system_prompt = NULL)
  }

  if (is_string(client)) {
    client <- ellmer::chat(client)
  }

  if (!inherits(client, "Chat")) {
    cli::cli_abort(
      "{.arg client} must be an {.pkg ellmer} {.cls Chat} object or a function that returns one"
    )
  }

  client
}

querychat_client_option <- function() {
  opt <- getOption("querychat.client", NULL)
  if (!is.null(opt)) {
    return(opt)
  }

  env <- Sys.getenv("QUERYCHAT_CLIENT", "")
  if (nzchar(env)) {
    return(env)
  }

  NULL
}

#' Data Source Base Class
#'
#' @description
#' An abstract R6 class defining the interface that custom QueryChat data
#' sources must implement. This class should not be instantiated directly;
#' instead, use one of its concrete implementations like [DataFrameSource] or
#' [DBISource].
#'
#' @examples
#' MyDataSource <- R6::R6Class(
#'   "MyDataSource",
#'   inherit = DataSource,
#'   public = list(
#'     initialize = function(table_name) {
#'       self$table_name <- table_name
#'     }
#'     # Implement abstract methods here...
#'   )
#' )
#'
#' @export
DataSource <- R6::R6Class(
  "DataSource",
  private = list(
    colnames = NULL
  ),
  public = list(
    #' @field table_name Name of the table to be used in SQL queries
    table_name = NULL,

    #' @description
    #' Get the database type
    #'
    #' @return A string describing the database type (e.g., "DuckDB", "SQLite")
    get_db_type = function() {
      cli::cli_abort(
        "{.fn get_db_type} must be implemented by subclass",
        class = "not_implemented_error"
      )
    },

    #' @description
    #' Get schema information about the table
    #'
    #' @param categorical_threshold Maximum number of unique values for a text
    #'   column to be considered categorical
    #' @return A string containing schema information formatted for LLM prompts
    get_schema = function(categorical_threshold = 20) {
      cli::cli_abort(
        "{.fn get_schema} must be implemented by subclass",
        class = "not_implemented_error"
      )
    },

    #' @description
    #' Execute a SQL query and return results
    #'
    #' @param query SQL query string to execute
    #' @return A data frame containing query results
    execute_query = function(query) {
      cli::cli_abort(
        "{.fn execute_query} must be implemented by subclass",
        class = "not_implemented_error"
      )
    },

    #' @description
    #' Test a SQL query by fetching only one row
    #'
    #' @param query SQL query string to test
    #' @param require_all_columns If `TRUE`, validates that the result includes
    #'   all original table columns (default: `FALSE`)
    #' @return A data frame containing one row of results (or empty if no
    #'   matches)
    test_query = function(query, require_all_columns = FALSE) {
      cli::cli_abort(
        "{.fn test_query} must be implemented by subclass",
        class = "not_implemented_error"
      )
    },

    #' @description
    #' Get the unfiltered data as a data frame
    #'
    #' @return A data frame containing all data from the table
    get_data = function() {
      cli::cli_abort(
        "{.fn get_data} must be implemented by subclass",
        class = "not_implemented_error"
      )
    },

    #' @description
    #' Clean up resources (close connections, etc.)
    #'
    #' @return NULL (invisibly)
    cleanup = function() {
      cli::cli_abort(
        "{.fn cleanup} must be implemented by subclass",
        class = "not_implemented_error"
      )
    }
  )
)

# Helper Functions -------------------------------------------------------------

is_data_source <- function(x) {
  inherits(x, "DataSource")
}

describe("TblSqlSource$new()", {
  it("creates proper R6 object for TblSqlSource", {
    source <- local_tbl_sql_source()

    expect_s3_class(source, "TblSqlSource")
    expect_s3_class(source, "DBISource")
    expect_s3_class(source, "DataSource")
    expect_equal(source$table_name, "test_table")
    expect_equal(source$get_db_type(), "DuckDB")
  })

  it("errors with non-tbl_sql input", {
    skip_if_not_installed("duckdb")
    skip_if_not_installed("dbplyr")

    expect_snapshot(error = TRUE, {
      TblSqlSource$new(data.frame(a = 1))
    })

    expect_snapshot(error = TRUE, {
      TblSqlSource$new(list(a = 1, b = 2))
    })
  })

  it("returns lazy tibble from execute_query()", {
    source <- local_tbl_sql_source()

    result <- source$execute_query("SELECT * FROM test_table WHERE value > 25")
    expect_s3_class(result, "tbl_sql")
    expect_s3_class(result, "tbl_lazy")

    # Collect to verify data
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 3)
    expect_equal(collected$value, c(30, 40, 50))
  })

  it("returns data frame from test_query()", {
    source <- local_tbl_sql_source()

    result <- source$test_query("SELECT * FROM test_table")
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 1)
  })

  it("returns lazy tibble from get_data()", {
    source <- local_tbl_sql_source()

    result <- source$get_data()
    expect_s3_class(result, "tbl_sql")
    expect_s3_class(result, "tbl_lazy")
  })
})

describe("TblSqlSource with transformed tbl (CTE mode)", {
  it("works with filtered tbl", {
    source <- local_tbl_sql_source(
      tbl_transform = function(tbl) dplyr::filter(tbl, value > 20)
    )

    # CTE should be used since tbl is transformed
    result <- source$execute_query("SELECT * FROM test_table")
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 3)
    expect_true(all(collected$value > 20))
  })

  it("works with selected columns tbl", {
    source <- local_tbl_sql_source(
      tbl_transform = function(tbl) dplyr::select(tbl, id, name)
    )

    result <- source$execute_query("SELECT * FROM test_table")
    collected <- dplyr::collect(result)
    expect_equal(names(collected), c("id", "name"))
  })
})

describe("TblSqlSource edge cases - Category A: Structural Violations", {
  # Note: TblSqlSource uses dplyr::tbl(conn, dplyr::sql(query)) which wraps

  # the user's query as a subquery. This means some SQL constructs that work
  # in standalone queries will fail when wrapped.

  it("errors on trailing semicolon (subquery wrapping)", {
    source <- local_tbl_sql_source()

    # Semicolons inside subqueries cause syntax errors in DuckDB
    # The query gets wrapped as: SELECT * FROM (SELECT * FROM test_table;) q01
    expect_error(
      dplyr::collect(source$execute_query("SELECT * FROM test_table;")),
      regexp = "syntax error"
    )
  })

  it("errors on trailing semicolon in CTE mode", {
    source <- local_tbl_sql_source(
      tbl_transform = function(tbl) dplyr::filter(tbl, value > 10)
    )

    # Same issue with CTE wrapping
    expect_error(
      dplyr::collect(source$execute_query("SELECT * FROM test_table;")),
      regexp = "syntax error"
    )
  })

  it("errors on multiple trailing semicolons", {
    source <- local_tbl_sql_source()

    expect_error(
      dplyr::collect(source$execute_query("SELECT * FROM test_table;;;")),
      regexp = "syntax error"
    )
  })

  it("errors on multiple statements", {
    source <- local_tbl_sql_source()

    # Multiple statements cause syntax errors when wrapped as subquery
    expect_error(
      dplyr::collect(source$execute_query("SELECT 1 AS a; SELECT 2 AS b")),
      regexp = "syntax error"
    )
  })

  it("errors on empty SELECT (syntax error)", {
    source <- local_tbl_sql_source()

    expect_error(
      dplyr::collect(source$execute_query("SELECT")),
      regexp = NULL
    )
  })

  it("errors on SELECT with no FROM when columns expected", {
    source <- local_tbl_sql_source()

    # SELECT without FROM is valid for literals but invalid for table columns
    expect_error(
      dplyr::collect(source$execute_query("SELECT id FROM")),
      regexp = NULL
    )
  })

  it("succeeds with query without trailing semicolon", {
    source <- local_tbl_sql_source()

    # Properly formed query without semicolon works
    result <- source$execute_query("SELECT * FROM test_table")
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 5)
  })
})

describe("TblSqlSource edge cases - Category B: Column Naming Issues", {
  it("handles unnamed expressions (auto-generated names)", {
    source <- local_tbl_sql_source()

    # DuckDB auto-generates names for unnamed expressions
    result <- source$execute_query(
      "SELECT id, 1+1, UPPER(name) FROM test_table"
    )
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 5)
    # Should have 3 columns (id + two computed)
    expect_equal(ncol(collected), 3)
  })

  it("handles unnamed expressions in CTE mode", {
    source <- local_tbl_sql_source(
      tbl_transform = function(tbl) dplyr::filter(tbl, value > 10)
    )

    result <- source$execute_query(
      "SELECT id, value * 2, UPPER(name) FROM test_table"
    )
    collected <- dplyr::collect(result)
    expect_equal(ncol(collected), 3)
  })

  it("errors on duplicate column names from JOIN (tibble requirement)", {
    skip_if_not_installed("duckdb")
    skip_if_not_installed("dbplyr")
    skip_if_not_installed("dplyr")

    conn <- local_duckdb_multi_table()
    tbl_a <- dplyr::tbl(conn, "table_a")
    source <- TblSqlSource$new(tbl_a, "table_a")

    # SELECT with explicit duplicate column names from JOIN
    # DuckDB allows duplicate names but tibble rejects them on collect
    result <- source$execute_query(
      "SELECT table_a.id, table_b.id FROM table_a JOIN table_b ON table_a.id = table_b.id"
    )
    expect_error(
      dplyr::collect(result),
      regexp = "must not be duplicated|must be unique"
    )
  })

  it("handles duplicate columns with aliases", {
    skip_if_not_installed("duckdb")
    skip_if_not_installed("dbplyr")
    skip_if_not_installed("dplyr")

    conn <- local_duckdb_multi_table()
    tbl_a <- dplyr::tbl(conn, "table_a")
    source <- TblSqlSource$new(tbl_a, "table_a")

    # Using aliases to avoid duplicate column names
    result <- source$execute_query(
      "SELECT table_a.id AS id_a, table_b.id AS id_b FROM table_a JOIN table_b ON table_a.id = table_b.id"
    )
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 3)
    expect_equal(ncol(collected), 2)
    expect_true("id_a" %in% names(collected))
    expect_true("id_b" %in% names(collected))
  })

  it("handles reserved word as alias", {
    source <- local_tbl_sql_source()

    # Using reserved word 'select' as column alias (quoted)
    result <- source$execute_query(
      "SELECT id AS \"select\", name AS \"from\" FROM test_table"
    )
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 5)
    expect_true("select" %in% names(collected))
    expect_true("from" %in% names(collected))
  })

  it("handles reserved word as unquoted alias (DuckDB permissive)", {
    source <- local_tbl_sql_source()

    # DuckDB is permissive with reserved words as aliases
    # This may work or fail depending on the specific word
    result <- source$execute_query(
      "SELECT id AS value_alias FROM test_table"
    )
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 5)
  })

  it("handles empty string alias (DB-dependent)", {
    source <- local_tbl_sql_source()

    # Empty string alias - DuckDB behavior
    # This typically creates a column with empty name or errors
    expect_error(
      {
        result <- source$execute_query(
          "SELECT id AS \"\" FROM test_table"
        )
        dplyr::collect(result)
      },
      regexp = NULL
    )
  })

  it("errors on wildcard with JOIN (duplicate columns)", {
    skip_if_not_installed("duckdb")
    skip_if_not_installed("dbplyr")
    skip_if_not_installed("dplyr")

    conn <- local_duckdb_multi_table()
    tbl_a <- dplyr::tbl(conn, "table_a")
    source <- TblSqlSource$new(tbl_a, "table_a")

    # SELECT * from JOIN produces duplicate 'id' columns
    # tibble rejects duplicate names on collect
    result <- source$execute_query(
      "SELECT * FROM table_a JOIN table_b ON table_a.id = table_b.id"
    )
    expect_error(
      dplyr::collect(result),
      regexp = "must not be duplicated|must be unique"
    )
  })

  it("handles wildcard with JOIN using USING clause (no duplicates)", {
    skip_if_not_installed("duckdb")
    skip_if_not_installed("dbplyr")
    skip_if_not_installed("dplyr")

    conn <- local_duckdb_multi_table()
    tbl_a <- dplyr::tbl(conn, "table_a")
    source <- TblSqlSource$new(tbl_a, "table_a")

    # USING clause produces single 'id' column
    result <- source$execute_query(
      "SELECT * FROM table_a JOIN table_b USING (id)"
    )
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 3)
    # id appears only once with USING
    expect_equal(sum(names(collected) == "id"), 1)
  })
})

describe("TblSqlSource edge cases - Category C: ORDER BY behavior", {
  it("handles ORDER BY without LIMIT", {
    source <- local_tbl_sql_source()

    # ORDER BY without LIMIT is valid SQL
    result <- source$execute_query(
      "SELECT * FROM test_table ORDER BY value DESC"
    )
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 5)
    # Verify order is applied
    expect_equal(collected$value[1], 50)
    expect_equal(collected$value[5], 10)
  })

  it("handles ORDER BY without LIMIT in CTE mode", {
    source <- local_tbl_sql_source(
      tbl_transform = function(tbl) dplyr::filter(tbl, value >= 10)
    )

    result <- source$execute_query(
      "SELECT * FROM test_table ORDER BY value DESC"
    )
    collected <- dplyr::collect(result)
    expect_true(nrow(collected) > 0)
    # Verify order is maintained through CTE
    expect_true(collected$value[1] >= collected$value[nrow(collected)])
  })

  it("handles LIMIT without ORDER BY (non-deterministic but valid)", {
    source <- local_tbl_sql_source()

    # LIMIT without ORDER BY is valid but non-deterministic
    result <- source$execute_query("SELECT * FROM test_table LIMIT 3")
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 3)
  })

  it("handles ORDER BY with LIMIT", {
    source <- local_tbl_sql_source()

    result <- source$execute_query(
      "SELECT * FROM test_table ORDER BY value DESC LIMIT 2"
    )
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 2)
    expect_equal(collected$value, c(50, 40))
  })

  it("handles ORDER BY with LIMIT and OFFSET", {
    source <- local_tbl_sql_source()

    result <- source$execute_query(
      "SELECT * FROM test_table ORDER BY value DESC LIMIT 2 OFFSET 1"
    )
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 2)
    expect_equal(collected$value, c(40, 30))
  })

  it("handles ORDER BY with column alias", {
    source <- local_tbl_sql_source()

    result <- source$execute_query(
      "SELECT id, value AS val FROM test_table ORDER BY val DESC"
    )
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 5)
    expect_equal(collected$val[1], 50)
  })

  it("handles ORDER BY with expression", {
    source <- local_tbl_sql_source()

    result <- source$execute_query(
      "SELECT id, value FROM test_table ORDER BY value * -1"
    )
    collected <- dplyr::collect(result)
    expect_equal(nrow(collected), 5)
    # ORDER BY value * -1 ascending means:
    # -50 < -40 < -30 < -20 < -10
    # So original values sorted: 50, 40, 30, 20, 10 (descending)
    expect_equal(collected$value[1], 50)
    expect_equal(collected$value[5], 10)
  })
})

describe("DataFrameSource$new()", {
  skip_if_no_dataframe_engine()

  it("creates proper R6 object for DataFrameSource", {
    test_df <- new_test_df()

    source <- DataFrameSource$new(test_df, "test_table")
    withr::defer(source$cleanup())

    expect_s3_class(source, "DataFrameSource")
    expect_s3_class(source, "DataSource")
    expect_equal(source$table_name, "test_table")
  })

  it("errors with non-data.frame input", {
    expect_snapshot(
      error = TRUE,
      DataFrameSource$new(list(a = 1, b = 2), "test_table")
    )
    expect_snapshot(error = TRUE, DataFrameSource$new(c(1, 2, 3), "test_table"))
    expect_snapshot(error = TRUE, DataFrameSource$new(NULL, "test_table"))
  })

  it("errors with invalid table names", {
    test_df <- new_test_df()

    expect_snapshot(error = TRUE, {
      DataFrameSource$new(test_df, "123_invalid")
      DataFrameSource$new(test_df, "table-name")
      DataFrameSource$new(test_df, "table name")
      DataFrameSource$new(test_df, "")
      DataFrameSource$new(test_df, NULL)
    })
  })
})

describe("DataFrameSource engine parameter", {
  describe("with duckdb engine", {
    skip_if_not_installed("duckdb")

    it("creates connection with duckdb backend", {
      df_source <- local_data_frame_source(new_test_df(), engine = "duckdb")

      expect_s3_class(df_source, "DataFrameSource")
      expect_s3_class(df_source, "DBISource")
      expect_equal(df_source$table_name, "test_table")
      expect_equal(df_source$get_db_type(), "DuckDB")
    })

    it("executes queries correctly", {
      test_df <- new_test_df()
      df_source <- local_data_frame_source(test_df, engine = "duckdb")

      # Test filtering
      result <- df_source$execute_query(
        "SELECT * FROM test_table WHERE value > 25"
      )
      expect_equal(nrow(result), 3)
      expect_equal(result$value, c(30, 40, 50))

      # Test get_data
      all_data <- df_source$get_data()
      expect_equal(all_data, test_df)

      # Test test_query
      one_row <- df_source$test_query("SELECT * FROM test_table")
      expect_equal(nrow(one_row), 1)
    })

    it("returns correct schema", {
      df_source <- local_data_frame_source(
        new_mixed_types_df(),
        engine = "duckdb"
      )
      schema <- df_source$get_schema()

      expect_type(schema, "character")
      expect_match(schema, "Table: test_table")
      expect_match(schema, "id \\(INTEGER\\)")
      expect_match(schema, "name \\(TEXT\\)")
      expect_match(schema, "active \\(BOOLEAN\\)")
    })
  })

  describe("with sqlite engine", {
    skip_if_not_installed("RSQLite")

    it("creates connection with sqlite backend", {
      df_source <- local_data_frame_source(new_test_df(), engine = "sqlite")

      expect_s3_class(df_source, "DataFrameSource")
      expect_s3_class(df_source, "DBISource")
      expect_equal(df_source$table_name, "test_table")
      expect_equal(df_source$get_db_type(), "SQLite")
    })

    it("executes queries correctly", {
      test_df <- new_test_df()
      df_source <- local_data_frame_source(test_df, engine = "sqlite")

      # Test filtering
      result <- df_source$execute_query(
        "SELECT * FROM test_table WHERE value > 25"
      )
      expect_equal(nrow(result), 3)
      expect_equal(result$value, c(30, 40, 50))

      # Test get_data
      all_data <- df_source$get_data()
      expect_equal(all_data, test_df)

      # Test test_query
      one_row <- df_source$test_query("SELECT * FROM test_table")
      expect_equal(nrow(one_row), 1)
    })

    it("returns correct schema", {
      df_source <- local_data_frame_source(
        new_mixed_types_df(),
        engine = "sqlite"
      )
      schema <- df_source$get_schema()

      expect_type(schema, "character")
      expect_match(schema, "Table:")
      expect_match(schema, "test_table")
      expect_match(schema, "id \\(INTEGER\\)")
      expect_match(schema, "name \\(TEXT\\)")
      # SQLite stores booleans as INTEGER (0/1)
      expect_match(schema, "active \\(INTEGER\\)")
    })
  })

  describe("engine parameter validation", {
    it("is case-insensitive", {
      skip_if_not_installed("duckdb")
      skip_if_not_installed("RSQLite")

      # Test various case combinations
      df1 <- local_data_frame_source(new_test_df(), engine = "DUCKDB")
      expect_equal(df1$get_db_type(), "DuckDB")

      df2 <- local_data_frame_source(new_test_df(), engine = "DuckDb")
      expect_equal(df2$get_db_type(), "DuckDB")

      df3 <- local_data_frame_source(new_test_df(), engine = "SQLite")
      expect_equal(df3$get_db_type(), "SQLite")

      df4 <- local_data_frame_source(new_test_df(), engine = "SQLITE")
      expect_equal(df4$get_db_type(), "SQLite")
    })

    it("errors on invalid engine name", {
      expect_snapshot(error = TRUE, {
        DataFrameSource$new(new_test_df(), "test_table", engine = "postgres")
      })

      expect_snapshot(error = TRUE, {
        DataFrameSource$new(new_test_df(), "test_table", engine = "invalid")
      })

      expect_snapshot(error = TRUE, {
        DataFrameSource$new(new_test_df(), "test_table", engine = "")
      })
    })

    it("respects getOption('querychat.DataFrameSource.engine')", {
      skip_if_not_installed("duckdb")
      skip_if_not_installed("RSQLite")

      # Test default (duckdb)
      withr::local_options(querychat.DataFrameSource.engine = NULL)
      df1 <- DataFrameSource$new(new_test_df(), "test_table")
      withr::defer(df1$cleanup())
      expect_equal(df1$get_db_type(), "DuckDB")

      # Test option set to sqlite
      withr::local_options(querychat.DataFrameSource.engine = "sqlite")
      df2 <- DataFrameSource$new(new_test_df(), "test_table")
      withr::defer(df2$cleanup())
      expect_equal(df2$get_db_type(), "SQLite")

      # Test explicit parameter overrides option
      withr::local_options(querychat.DataFrameSource.engine = "sqlite")
      df3 <- local_data_frame_source(new_test_df(), engine = "duckdb")
      expect_equal(df3$get_db_type(), "DuckDB")
    })
  })
})

describe("DataFrameSource$test_query()", {
  skip_if_no_dataframe_engine()

  test_df <- new_users_df()
  df_source <- local_data_frame_source(test_df, "test_table")

  it("correctly retrieves one row of data", {
    result <- df_source$test_query("SELECT * FROM test_table")
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 1)
    expect_equal(result$id, 1)

    result <- df_source$test_query("SELECT * FROM test_table WHERE age > 29")
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 1)
    expect_equal(result$age, 30)

    result <- df_source$test_query(
      "SELECT * FROM test_table ORDER BY age DESC"
    )
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 1)
    expect_equal(result$age, 35)

    result <- df_source$test_query(
      "SELECT * FROM test_table WHERE age > 100"
    )
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 0)
  })
})

# Modifies the data presented in the data dashboard, based on the given SQL
# query, and also updates the title.
# @param query A SQL query; must be a SELECT statement.
# @param title A title to display at the top of the data dashboard,
#   summarizing the intent of the SQL query.
tool_update_dashboard <- function(
  data_source,
  update_fn = function(query, title) {}
) {
  check_data_source(data_source)

  check_function(update_fn)
  has_args <- intersect(fn_fmls_names(update_fn), c("query", "title"))
  if (length(has_args) != 2) {
    missing_args <- setdiff(c("query", "title"), has_args)
    cli::cli_abort(
      c(
        "{.arg update_fn} must accept at least two named arguments: {.val query} and {.val title}.",
        "x" = "{.val {missing_args}} argument{?s} {?was/were} missing."
      )
    )
  }

  db_type <- data_source$get_db_type()

  ellmer::tool(
    tool_update_dashboard_impl(data_source, update_fn),
    name = "querychat_update_dashboard",
    description = interpolate_package(
      "tool-update-dashboard.md",
      db_type = db_type
    ),
    arguments = list(
      query = ellmer::type_string(
        ellmer::interpolate(
          "A {{db_type}} SQL SELECT query that MUST return all existing schema columns (use SELECT * or explicitly list all columns). May include additional computed columns, subqueries, CTEs, WHERE clauses, ORDER BY, and any {{db_type}}-supported SQL functions.",
          db_type = db_type
        )
      ),
      title = ellmer::type_string(
        "A brief title for display purposes, summarizing the intent of the SQL query."
      )
    ),
    annotations = ellmer::tool_annotations(
      title = "Update Dashboard",
      icon = '<svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" class="bi bi-funnel-fill" viewBox="0 0 16 16"><path d="M1.5 1.5A.5.5 0 0 1 2 1h12a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.128.334L10 8.692V13.5a.5.5 0 0 1-.342.474l-3 1A.5.5 0 0 1 6 14.5V8.692L1.628 3.834A.5.5 0 0 1 1.5 3.5z"/></svg>'
    )
  )
}

tool_update_dashboard_impl <- function(data_source, update_fn) {
  force(data_source)

  function(query, title) {
    res <- querychat_tool_result(
      data_source,
      query = query,
      title = title,
      action = "update"
    )

    if (is.null(res@error)) {
      update_fn(query, title)
    }

    res
  }
}


tool_reset_dashboard <- function(reset_fn = identity) {
  check_function(reset_fn)

  ellmer::tool(
    reset_fn,
    name = "querychat_reset_dashboard",
    description = interpolate_package("tool-reset-dashboard.md"),
    arguments = list(),
    annotations = ellmer::tool_annotations(
      title = "Reset Dashboard",
      icon = '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 16 16" class="bi bi-arrow-counterclockwise " style="height:1em;width:1em;fill:currentColor;vertical-align:-0.125em;" aria-hidden="true" role="img" ><path fill-rule="evenodd" d="M8 3a5 5 0 1 1-4.546 2.914.5.5 0 0 0-.908-.417A6 6 0 1 0 8 2v1z"></path><path d="M8 4.466V.534a.25.25 0 0 0-.41-.192L5.23 2.308a.25.25 0 0 0 0 .384l2.36 1.966A.25.25 0 0 0 8 4.466z"></path></svg>'
    )
  )
}

# Perform a SQL query on the data, and return the results as JSON.
# @param query A SQL query; must be a SELECT statement.
# @return The results of the query as a data frame.
tool_query <- function(data_source) {
  check_data_source(data_source)

  db_type <- data_source$get_db_type()

  ellmer::tool(
    function(query, collapsed = NULL, `_intent` = "") {
      querychat_tool_result(
        data_source,
        query,
        action = "query",
        collapsed = collapsed
      )
    },
    name = "querychat_query",
    description = interpolate_package("tool-query.md", db_type = db_type),
    arguments = list(
      query = ellmer::type_string(
        ellmer::interpolate(
          "A valid {{db_type}} SQL SELECT statement. Must follow the database schema provided in the system prompt. Use clear column aliases (e.g., 'AVG(price) AS avg_price') and include SQL comments for complex logic. Subqueries and CTEs are encouraged for readability.",
          db_type = db_type
        )
      ),
      collapsed = ellmer::type_boolean(
        "Optional (default: true). The result card starts collapsed by default; the user can expand it to see the query and results. Set to false when the query result table is the primary answer to the user's question and should be immediately visible without expanding.",
        required = FALSE
      ),
      `_intent` = ellmer::type_string(
        "A brief, user-friendly description of what this query calculates or retrieves."
      )
    ),
    annotations = ellmer::tool_annotations(
      title = "Query Data",
      icon = '<svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" class="bi bi-table" viewBox="0 0 16 16"><path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2zm15 2h-4v3h4zm0 4h-4v3h4zm0 4h-4v3h3a1 1 0 0 0 1-1zm-5 3v-3H6v3zm-5 0v-3H1v2a1 1 0 0 0 1 1zm-4-4h4V8H1zm0-4h4V4H1zm5-3v3h4V4zm4 4H6v3h4z"/></svg>'
    )
  )
}

querychat_tool_details_option <- function() {
  opt <- getOption("querychat.tool_details", NULL)
  if (!is.null(opt)) {
    setting <- opt
  } else {
    env <- Sys.getenv("QUERYCHAT_TOOL_DETAILS", "")
    if (nzchar(env)) {
      setting <- env
    } else {
      return(NULL)
    }
  }

  setting <- tolower(setting)
  valid_settings <- c("expanded", "collapsed", "default")

  if (!setting %in% valid_settings) {
    cli::cli_warn(c(
      "Invalid value for {.code querychat.tool_details} option or {.envvar QUERYCHAT_TOOL_DETAILS} environment variable: {.val {setting}}",
      "i" = "Must be one of: {.or {.val {valid_settings}}}"
    ))
    return(NULL)
  }

  setting
}

querychat_tool_starts_open <- function(action) {
  setting <- querychat_tool_details_option()

  if (is.null(setting)) {
    return(action %in% c("update", "visualize"))
  }

  switch(
    setting,
    "expanded" = TRUE,
    "collapsed" = FALSE,
    action %in% c("update", "visualize")
  )
}

querychat_tool_result <- function(
  data_source,
  query,
  title = NULL,
  action = "update",
  collapsed = NULL
) {
  action <- arg_match(action, c("update", "query", "reset"))

  if (action == "reset") {
    query <- NULL
    title <- NULL
  }

  res <- tryCatch(
    switch(
      action,
      update = {
        data_source$test_query(query, require_all_columns = TRUE)
        NULL
      },
      query = data_source$execute_query(query),
      reset = "The dashboard has been reset to show all data."
    ),
    error = function(err) err
  )

  is_error <- is_condition(res)

  output <- ""
  if (!is_error && action == "query") {
    output <- utils::capture.output(print(res))
    output <- paste(
      c(
        "\n\n<details open><summary>Result</summary>\n\n```",
        output,
        "```\n\n</details>"
      ),
      collapse = "\n"
    )
  }

  if (!is_error && action %in% c("update", "reset")) {
    output <- format(
      shiny::tags$button(
        class = "btn btn-outline-primary btn-sm float-end mt-3 querychat-update-dashboard-btn",
        "data-query" = query,
        "data-title" = title,
        switch(action, update = "Apply Filter", reset = "Reset Filter")
      )
    )
    output <- paste0("\n\n", output)
  }

  value <-
    switch(
      action,
      update = "Dashboard updated. Use `querychat_query` tool to review results, if needed.",
      res
    )

  display_md <- switch(
    action,
    reset = output,
    sprintf("```sql\n%s\n```%s", query, output)
  )

  ellmer::ContentToolResult(
    value = if (!is_error) value,
    error = if (is_error) res,
    extra = list(
      display = list(
        title = if (action == "update" && !is.null(title)) title,
        show_request = is_error,
        markdown = display_md,
        open = if (!is.null(collapsed)) {
          !collapsed
        } else {
          querychat_tool_starts_open(action)
        }
      )
    )
  )
}

test_that("tool_update_dashboard() checks inputs", {
  skip_if_no_dataframe_engine()

  expect_snapshot(error = TRUE, tool_update_dashboard("foo"))

  df_source <- local_data_frame_source(new_test_df())
  expect_snapshot(error = TRUE, {
    tool_update_dashboard(df_source, update_fn = NULL)
    tool_update_dashboard(df_source, update_fn = function(query) {})
    tool_update_dashboard(df_source, update_fn = function(title, extra) {})
  })
})

test_that("tool_reset_dashboard() checks inputs", {
  expect_snapshot(error = TRUE, tool_reset_dashboard("not_a_function"))
})

test_that("tool_query() checks inputs", {
  expect_snapshot(error = TRUE, tool_query("invalid_source"))
})

describe("querychat_tool_starts_open()", {
  it("uses the tool default when options are unset", {
    withr::local_options(querychat.tool_details = NULL)
    withr::local_envvar(QUERYCHAT_TOOL_DETAILS = NA)

    expect_false(querychat_tool_starts_open("query"))
    expect_true(querychat_tool_starts_open("update"))
    expect_false(querychat_tool_starts_open("reset"))
  })

  it("uses the tool default when envvar is 'default'", {
    withr::local_options(querychat.tool_details = NULL)
    withr::local_envvar(QUERYCHAT_TOOL_DETAILS = "default")

    expect_false(querychat_tool_starts_open("query"))
    expect_true(querychat_tool_starts_open("update"))
    expect_false(querychat_tool_starts_open("reset"))
  })

  it("uses the tool default when option is 'default'", {
    withr::local_options(querychat.tool_details = "default")

    expect_false(querychat_tool_starts_open("query"))
    expect_true(querychat_tool_starts_open("update"))
    expect_false(querychat_tool_starts_open("reset"))
  })

  it("always expands when option is 'expanded'", {
    withr::local_options(querychat.tool_details = "expanded")

    expect_true(querychat_tool_starts_open("query"))
    expect_true(querychat_tool_starts_open("update"))
    expect_true(querychat_tool_starts_open("reset"))
  })

  it("always collapses when option is 'collapsed'", {
    withr::local_options(querychat.tool_details = "collapsed")

    expect_false(querychat_tool_starts_open("query"))
    expect_false(querychat_tool_starts_open("update"))
    expect_false(querychat_tool_starts_open("reset"))
  })
})

describe("querychat_tool_details_option()", {
  it("is case-insensitive", {
    withr::local_options(querychat.tool_details = "EXPANDED")
    expect_equal(querychat_tool_details_option(), "expanded")

    withr::local_options(querychat.tool_details = "Collapsed")
    expect_equal(querychat_tool_details_option(), "collapsed")
  })

  it("warns on invalid setting", {
    withr::local_options(querychat.tool_details = "invalid")

    expect_warning(
      querychat_tool_details_option(),
      "Invalid value for"
    )
  })

  it("gives option precedence over envvar", {
    withr::local_options(querychat.tool_details = "collapsed")
    withr::local_envvar(QUERYCHAT_TOOL_DETAILS = "expanded")

    expect_equal(querychat_tool_details_option(), "collapsed")
  })
})

describe("querychat_tool_result()", {
  skip_if_no_dataframe_engine()

  it("returns successful result for valid query action", {
    df_source <- local_data_frame_source(new_test_df())

    result <- querychat_tool_result(
      df_source,
      query = "SELECT * FROM test_table WHERE id = 1",
      action = "query"
    )

    expect_s7_class(result, ellmer::ContentToolResult)
    expect_null(result@error)
    expect_s3_class(result@value, "data.frame")
    expect_equal(nrow(result@value), 1)
  })

  it("returns successful result for valid update action", {
    df_source <- local_data_frame_source(new_test_df())

    result <- querychat_tool_result(
      df_source,
      query = "SELECT * FROM test_table WHERE value > 20",
      title = "High values",
      action = "update"
    )

    expect_s7_class(result, ellmer::ContentToolResult)
    expect_null(result@error)
    expect_equal(
      result@value,
      "Dashboard updated. Use `querychat_query` tool to review results, if needed."
    )
  })

  it("returns successful result for reset action", {
    df_source <- local_data_frame_source(new_test_df())

    result <- querychat_tool_result(
      df_source,
      query = NULL,
      action = "reset"
    )

    expect_s7_class(result, ellmer::ContentToolResult)
    expect_null(result@error)
    expect_equal(result@value, "The dashboard has been reset to show all data.")
  })

  it("handles query errors appropriately", {
    df_source <- local_data_frame_source(new_test_df())

    result <- querychat_tool_result(
      df_source,
      query = "SELECT * FROM nonexistent_table",
      action = "query"
    )

    expect_s7_class(result, ellmer::ContentToolResult)
    expect_s3_class(result@error, "error")
    expect_null(result@value)
  })

  it("handles update errors appropriately", {
    df_source <- local_data_frame_source(new_test_df())

    result <- querychat_tool_result(
      df_source,
      query = "INVALID SQL",
      action = "update"
    )

    expect_s7_class(result, ellmer::ContentToolResult)
    expect_s3_class(result@error, "error")
    expect_null(result@value)
  })

  it("formats query results with details block", {
    df_source <- local_data_frame_source(new_test_df())

    result <- querychat_tool_result(
      df_source,
      query = "SELECT * FROM test_table LIMIT 1",
      action = "query"
    )

    markdown <- result@extra$display$markdown
    expect_match(markdown, "```sql")
    expect_match(markdown, "SELECT \\* FROM test_table LIMIT 1")
    expect_match(markdown, "<details")
    expect_match(markdown, "</details>")
  })

  it("formats update results with button HTML", {
    df_source <- local_data_frame_source(new_test_df())

    result <- querychat_tool_result(
      df_source,
      query = "SELECT * FROM test_table",
      title = "Test Filter",
      action = "update"
    )

    markdown <- result@extra$display$markdown
    expect_match(markdown, "```sql")
    expect_match(markdown, "SELECT \\* FROM test_table")
    expect_match(markdown, "button")
    expect_match(markdown, "Apply Filter")
    expect_match(markdown, "data-query")
    expect_match(markdown, "data-title")
  })

  it("formats reset results with button HTML", {
    df_source <- local_data_frame_source(new_test_df())

    result <- querychat_tool_result(
      df_source,
      query = NULL,
      action = "reset"
    )

    markdown <- result@extra$display$markdown
    expect_match(markdown, "button")
    expect_match(markdown, "Reset Filter")
  })

  it("includes title in extra display metadata for update action", {
    df_source <- local_data_frame_source(new_test_df())

    result <- querychat_tool_result(
      df_source,
      query = "SELECT * FROM test_table",
      title = "Custom Title",
      action = "update"
    )

    expect_equal(result@extra$display$title, "Custom Title")
  })

  it("does not include title for query action", {
    df_source <- local_data_frame_source(new_test_df())

    result <- querychat_tool_result(
      df_source,
      query = "SELECT * FROM test_table",
      title = "Should be ignored",
      action = "query"
    )

    expect_null(result@extra$display$title)
  })

  it("sets open state based on action and tool details option", {
    df_source <- local_data_frame_source(new_test_df())
    withr::local_options(querychat.tool_details = NULL)

    query_result <- querychat_tool_result(
      df_source,
      query = "SELECT * FROM test_table",
      action = "query"
    )
    expect_false(query_result@extra$display$open)

    reset_result <- querychat_tool_result(
      df_source,
      query = NULL,
      action = "reset"
    )
    expect_false(reset_result@extra$display$open)
  })

  it("shows request on error", {
    df_source <- local_data_frame_source(new_test_df())

    result <- querychat_tool_result(
      df_source,
      query = "INVALID SQL",
      action = "query"
    )

    expect_true(result@extra$display$show_request)
  })

  it("hides request on success", {
    df_source <- local_data_frame_source(new_test_df())

    result <- querychat_tool_result(
      df_source,
      query = "SELECT * FROM test_table",
      action = "query"
    )

    expect_false(result@extra$display$show_request)
  })
})

describe("tool_query()", {
  skip_if_no_dataframe_engine()

  it("returns an ellmer tool object", {
    df_source <- local_data_frame_source(new_test_df())
    tool <- tool_query(df_source)

    expect_s3_class(tool, "ellmer::ToolDef")
    expect_equal(tool@name, "querychat_query")
  })

  it("includes database type in description", {
    df_source <- local_data_frame_source(new_test_df())
    tool <- tool_query(df_source)

    # DataFrameSource uses DuckDB
    expect_match(tool@description, "DuckDB|duckdb", ignore.case = TRUE)
  })

  it("creates a working tool function", {
    df_source <- local_data_frame_source(new_test_df())
    tool <- tool_query(df_source)

    # Execute the tool function
    result <- tool(query = "SELECT * FROM test_table LIMIT 1")

    expect_s7_class(result, ellmer::ContentToolResult)
    expect_null(result@error)
  })
})

describe("tool_update_dashboard()", {
  skip_if_no_dataframe_engine()

  it("returns an ellmer tool object", {
    df_source <- local_data_frame_source(new_test_df())

    tool <- tool_update_dashboard(df_source)

    expect_s3_class(tool, "ellmer::ToolDef")
    expect_equal(tool@name, "querychat_update_dashboard")
  })

  it("includes database type in description", {
    df_source <- local_data_frame_source(new_test_df())

    tool <- tool_update_dashboard(df_source)

    # DataFrameSource uses DuckDB
    expect_match(tool@description, "DuckDB|duckdb", ignore.case = TRUE)
  })

  it("creates a working tool function", {
    df_source <- local_data_frame_source(new_test_df())

    res_update <- NULL
    tool <- tool_update_dashboard(df_source, function(query, title) {
      res_update <<- list(query = query, title = title)
    })

    res_tool <- tool(
      query = "SELECT * FROM test_table WHERE id > 2",
      title = "Filtered View"
    )

    expect_s7_class(res_tool, ellmer::ContentToolResult)
    expect_equal(res_update$query, "SELECT * FROM test_table WHERE id > 2")
    expect_equal(res_update$title, "Filtered View")
  })
})

describe("tool_reset_dashboard()", {
  reset_fn <- function() {
    "Reset executed"
  }

  it("returns an ellmer tool object", {
    tool <- tool_reset_dashboard(reset_fn)

    expect_s3_class(tool, "ellmer::ToolDef")
    expect_equal(tool@name, "querychat_reset_dashboard")
  })

  it("uses the provided reset function", {
    tool <- tool_reset_dashboard(reset_fn)

    expect_s3_class(tool, "ellmer::ToolDef")
    expect_equal(tool(), "Reset executed")
  })
})

describe("tool_update_dashboard_impl()", {
  skip_if_no_dataframe_engine()

  it("returns a function", {
    df_source <- local_data_frame_source(new_test_df())
    current_query <- shiny::reactiveVal("SELECT * FROM test_table")
    current_title <- shiny::reactiveVal("All Data")

    impl_fn <- tool_update_dashboard_impl(df_source)

    expect_type(impl_fn, "closure")
  })

  it("updates reactive values on successful query", {
    df_source <- local_data_frame_source(new_test_df())

    res_update <- NULL
    impl_fn <- tool_update_dashboard_impl(df_source, function(query, title) {
      res_update <<- list(query = query, title = title)
    })

    res_tool <- impl_fn(
      query = "SELECT * FROM test_table WHERE id < 3",
      title = "First Two"
    )

    expect_equal(res_update$query, "SELECT * FROM test_table WHERE id < 3")
    expect_equal(res_update$title, "First Two")
    expect_null(res_tool@error)
  })

  it("does not update reactive values on query error", {
    df_source <- local_data_frame_source(new_test_df())

    res_update <- NULL
    impl_fn <- tool_update_dashboard_impl(df_source, function(query, title) {
      res_update <<- list(query = query, title = title)
    })

    res_tool <- impl_fn(
      query = "INVALID SQL",
      title = "Should Not Update"
    )

    # `update_fn` was not called
    expect_null(res_update)
    # because the tool resulted in an error
    expect_s3_class(res_tool@error, class = "error")
  })
})

library(shiny)
library(bslib, warn.conflicts = FALSE)
library(querychat)
library(DBI)
library(RSQLite)

# Mock chat function to avoid LLM API calls
MockChat <- R6::R6Class(
  "MockChat",
  inherit = asNamespace("ellmer")[["Chat"]],
  public = list(
    stream_async = function(message, ...) {
      "Welcome! This is a mock response for testing."
    }
  )
)

# Create test database
temp_db <- tempfile(fileext = ".db")
conn <- dbConnect(RSQLite::SQLite(), temp_db)
dbWriteTable(conn, "iris", iris, overwrite = TRUE)
dbDisconnect(conn)

# Setup database source and QueryChat instance
db_conn <- dbConnect(RSQLite::SQLite(), temp_db)

# Create QueryChat instance
qc <- QueryChat$new(
  data_source = db_conn,
  table_name = "iris",
  greeting = "Welcome to the test app!",
  client = MockChat$new(ellmer::Provider("test", "test", "test"))
)

ui <- page_sidebar(
  title = "Test Database App",
  sidebar = qc$sidebar(),
  h2("Data"),
  DT::DTOutput("data_table"),
  h3("SQL Query"),
  verbatimTextOutput("sql_query")
)

server <- function(input, output, session) {
  qc_vals <- qc$server()

  output$data_table <- DT::renderDT(
    {
      qc_vals$df()
    },
    options = list(pageLength = 5)
  )

  output$sql_query <- renderText({
    query <- qc_vals$sql()
    if (is.null(query) || !nzchar(query)) "No filter applied" else query
  })

  session$onSessionEnded(function() {
    if (DBI::dbIsValid(db_conn)) {
      DBI::dbDisconnect(db_conn)
    }
    unlink(temp_db)
  })
}

shinyApp(ui = ui, server = server)

# Test fixture constructors for data source tests

# Simple data frame with id, name, and value columns
new_test_df <- function(rows = 5) {
  data.frame(
    id = seq_len(rows),
    name = c("A", "B", "C", "D", "E")[seq_len(rows)],
    value = c(10, 20, 30, 40, 50)[seq_len(rows)],
    stringsAsFactors = FALSE
  )
}

# Data frame with multiple numeric columns for testing min/max ranges
new_metrics_df <- function() {
  data.frame(
    id = 1:5,
    score = c(10.5, 20.3, 15.7, 30.1, 25.9),
    count = c(100, 200, 150, 50, 75),
    stringsAsFactors = FALSE
  )
}

# Data frame with mixed types including boolean
new_mixed_types_df <- function() {
  data.frame(
    id = 1:5,
    name = c("A", "B", "C", "D", "E"),
    active = c(TRUE, FALSE, TRUE, TRUE, FALSE),
    stringsAsFactors = FALSE
  )
}

# Data frame for testing user data
new_users_df <- function() {
  data.frame(
    id = 1:5,
    name = c("Alice", "Bob", "Charlie", "Diana", "Eve"),
    age = c(25, 30, 35, 28, 32),
    stringsAsFactors = FALSE
  )
}

# Data frame with all data types for type testing
new_types_df <- function() {
  data.frame(
    id = 1:3,
    text_col = c("text1", "text2", "text3"),
    num_col = c(1.1, 2.2, 3.3),
    int_col = c(10L, 20L, 30L),
    bool_col = c(TRUE, FALSE, TRUE),
    stringsAsFactors = FALSE
  )
}

# Create a temporary SQLite connection with a test table
local_sqlite_connection <- function(
  data = new_test_df(),
  table_name = "test_table",
  env = parent.frame()
) {
  if (testthat::is_testing()) {
    skip_if_not_installed("RSQLite")
  }

  temp_db <- withr::local_tempfile(fileext = ".db", .local_envir = env)
  conn <- DBI::dbConnect(RSQLite::SQLite(), temp_db)
  withr::defer(DBI::dbDisconnect(conn), envir = env)

  DBI::dbWriteTable(conn, table_name, data, overwrite = TRUE)

  list(conn = conn, path = temp_db)
}

# Skip test if no DataFrameSource engine is available
skip_if_no_dataframe_engine <- function() {
  if (!rlang::is_installed("duckdb") && !rlang::is_installed("RSQLite")) {
    skip("Neither duckdb nor RSQLite is installed")
  }
}

# Create a DataFrameSource with automatic cleanup
local_data_frame_source <- function(
  data,
  table_name = "test_table",
  engine = "duckdb",
  env = parent.frame()
) {
  df_source <- DataFrameSource$new(data, table_name, engine = engine)
  withr::defer(df_source$cleanup(), envir = env)
  df_source
}

local_querychat <- function(
  data_source = new_test_df(),
  table_name = "test_table",
  ...,
  env = parent.frame()
) {
  qc <- QueryChat$new(data_source, table_name, ...)
  withr::defer(qc$cleanup(), envir = env)
  qc
}

# Create a TblSqlSource with DuckDB and automatic cleanup
local_tbl_sql_source <- function(
  data = new_test_df(),
  table_name = "test_table",
  tbl_transform = identity,
  env = parent.frame()
) {
  skip_if_not_installed("duckdb")
  skip_if_not_installed("dbplyr")
  skip_if_not_installed("dplyr")

  conn <- DBI::dbConnect(duckdb::duckdb(), dbdir = ":memory:")
  withr::defer(DBI::dbDisconnect(conn, shutdown = TRUE), envir = env)

  DBI::dbWriteTable(conn, table_name, data, overwrite = TRUE)
  tbl <- dplyr::tbl(conn, table_name)
  tbl <- tbl_transform(tbl)

  TblSqlSource$new(tbl, table_name)
}

# Create a DuckDB connection with multiple tables for JOIN tests
local_duckdb_multi_table <- function(
  env = parent.frame()
) {
  skip_if_not_installed("duckdb")

  conn <- DBI::dbConnect(duckdb::duckdb(), dbdir = ":memory:")
  withr::defer(DBI::dbDisconnect(conn, shutdown = TRUE), envir = env)

  # Table A with id and name
  DBI::dbWriteTable(
    conn,
    "table_a",
    data.frame(
      id = 1:3,
      name = c("Alice", "Bob", "Carol"),
      stringsAsFactors = FALSE
    )
  )

  # Table B with id and value
  DBI::dbWriteTable(
    conn,
    "table_b",
    data.frame(
      id = 1:3,
      value = c(100, 200, 300),
      stringsAsFactors = FALSE
    )
  )

  conn
}

mock_ellmer_chat_client <- function(
  public = list(),
  private = list(),
  env = parent.frame()
) {
  MockChat <- R6::R6Class(
    "MockChat",
    inherit = asNamespace("ellmer")[["Chat"]],
    public = public,
    private = private
  )

  MockChat$new(ellmer::Provider("test", "test", "test"))
}

describe("extract_visualise_table()", {
  it("extracts bare identifier", {
    expect_equal(
      extract_visualise_table(
        "VISUALISE x AS x, y AS y FROM my_table DRAW point"
      ),
      "my_table"
    )
  })

  it("extracts quoted identifier", {
    expect_equal(
      extract_visualise_table('VISUALISE x AS x FROM "My Table" DRAW point'),
      '"My Table"'
    )
  })

  it("returns NULL when no FROM in VISUALISE", {
    expect_null(extract_visualise_table("VISUALISE x AS x, y AS y DRAW point"))
  })

  it("ignores FROM inside DRAW clause", {
    visual <- "VISUALISE x AS x FROM data DRAW point MAPPING x FROM other"
    expect_equal(extract_visualise_table(visual), "data")
  })
})

describe("has_layer_level_source()", {
  it("detects DRAW-level FROM source", {
    expect_true(
      has_layer_level_source(
        "VISUALISE x AS x DRAW point MAPPING x FROM my_table"
      )
    )
  })

  it("ignores VISUALISE-level FROM", {
    expect_false(
      has_layer_level_source(
        "VISUALISE x AS x FROM my_table DRAW point"
      )
    )
  })

  it("ignores SCALE FROM", {
    expect_false(
      has_layer_level_source(
        "VISUALISE x AS x DRAW bar SCALE fill FROM 'red'"
      )
    )
  })
})

describe("execute_ggsql()", {
  skip_if_no_dataframe_engine()
  skip_if_not_installed("ggsql")

  it("executes a basic ggsql query", {
    ds <- local_data_frame_source(new_test_df())
    validated <- ggsql::ggsql_validate(
      "SELECT * FROM test_table VISUALISE value AS x DRAW histogram"
    )
    spec <- execute_ggsql(ds, validated)
    expect_s3_class(spec, "Spec")
  })

  it("supports uppercase column references without renaming", {
    df <- data.frame(
      ROOM_TYPE = c("Entire home", "Private room"),
      listings = c(10, 20)
    )
    ds <- local_data_frame_source(df, table_name = "upper_table")
    validated <- ggsql::ggsql_validate(
      paste(
        "SELECT ROOM_TYPE, listings FROM upper_table",
        "VISUALISE ROOM_TYPE AS x, listings AS y DRAW bar"
      )
    )
    spec <- execute_ggsql(ds, validated)
    expect_s3_class(spec, "Spec")
  })

  it("passes original column names through to ggsql", {
    df <- data.frame(
      ROOM_TYPE = c("Entire home", "Private room"),
      listings = c(10, 20)
    )
    ds <- local_data_frame_source(df, table_name = "upper_table")
    validated <- list(
      sql = "SELECT ROOM_TYPE, listings FROM upper_table",
      visual = "VISUALISE ROOM_TYPE AS x, listings AS y DRAW bar"
    )
    captured_names <- NULL

    local_mocked_bindings(
      duckdb_reader = function(...) "reader",
      ggsql_register = function(reader, df, name) {
        captured_names <<- names(df)
        invisible(NULL)
      },
      ggsql_execute = function(reader, query) {
        structure(list(), class = "Spec")
      },
      .package = "ggsql"
    )

    spec <- execute_ggsql(ds, validated)
    expect_s3_class(spec, "Spec")
    expect_equal(captured_names, c("ROOM_TYPE", "listings"))
  })

  it("supports FROM before VISUALISE", {
    ds <- local_data_frame_source(new_test_df())
    validated <- ggsql::ggsql_validate(
      "FROM test_table VISUALISE value AS x DRAW histogram"
    )
    expect_true(validated$valid)
    expect_true(validated$has_visual)
    expect_equal(validated$sql, "SELECT * FROM test_table")
    expect_equal(validated$visual, "VISUALISE value AS x DRAW histogram")

    spec <- execute_ggsql(ds, validated)
    expect_s3_class(spec, "Spec")
  })

  it("rejects layer-level FROM sources", {
    ds <- local_data_frame_source(new_test_df())
    # This is a synthetic test — construct a validated object whose
    # visual has a layer-level source
    expect_error(
      {
        validated <- list(
          sql = "SELECT * FROM test_table",
          visual = "VISUALISE value AS x DRAW point MAPPING x FROM other_table"
        )
        execute_ggsql(ds, validated)
      },
      "Layer-specific sources"
    )
  })
})

describe("QueryChatSystemPrompt$new()", {
  skip_if_no_dataframe_engine()

  it("initializes with string template", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "Template: {{schema}}",
      data_source = ds
    )

    expect_type(sp$template, "character")
    expect_true(grepl("Template:", sp$template))
    expect_type(sp$schema, "character")
    expect_equal(sp$categorical_threshold, 10)
  })

  it("initializes with file path template", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    temp_file <- withr::local_tempfile(fileext = ".md")
    writeLines("File template: {{db_type}}", temp_file)

    sp <- QueryChatSystemPrompt$new(
      prompt_template = temp_file,
      data_source = ds
    )

    expect_type(sp$template, "character")
    expect_true(grepl("File template:", sp$template))
  })

  it("initializes with string data_description", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "Template",
      data_source = ds,
      data_description = "Test data description"
    )

    expect_equal(sp$data_description, "Test data description")
  })

  it("initializes with file path data_description", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    temp_file <- withr::local_tempfile(fileext = ".txt")
    writeLines("Data from file", temp_file)

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "Template",
      data_source = ds,
      data_description = temp_file
    )

    expect_equal(trimws(sp$data_description), "Data from file")
  })

  it("initializes with string extra_instructions", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "Template",
      data_source = ds,
      extra_instructions = "Extra instructions here"
    )

    expect_equal(sp$extra_instructions, "Extra instructions here")
  })

  it("initializes with file path extra_instructions", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    temp_file <- withr::local_tempfile(fileext = ".txt")
    writeLines("Instructions from file", temp_file)

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "Template",
      data_source = ds,
      extra_instructions = temp_file
    )

    expect_equal(trimws(sp$extra_instructions), "Instructions from file")
  })

  it("stores categorical_threshold", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "Template",
      data_source = ds,
      categorical_threshold = 25
    )

    expect_equal(sp$categorical_threshold, 25)
  })

  it("handles NULL data_description and extra_instructions", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "Template",
      data_source = ds
    )

    expect_null(sp$data_description)
    expect_null(sp$extra_instructions)
  })
})

describe("QueryChatSystemPrompt$render()", {
  it("renders with both tools", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    template <- paste(
      "{{#has_tool_update}}update enabled{{/has_tool_update}}",
      "{{#has_tool_query}}query enabled{{/has_tool_query}}",
      "{{#include_query_guidelines}}query guidelines{{/include_query_guidelines}}",
      sep = "\n"
    )

    sp <- QueryChatSystemPrompt$new(
      prompt_template = template,
      data_source = ds
    )

    result <- sp$render(c("update", "query"))

    expect_true(grepl("update enabled", result))
    expect_true(grepl("query enabled", result))
    expect_true(grepl("query guidelines", result))
  })

  it("renders with query only", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    template <- paste(
      "{{#has_tool_update}}update enabled{{/has_tool_update}}",
      "{{#has_tool_query}}query enabled{{/has_tool_query}}",
      "{{#include_query_guidelines}}query guidelines{{/include_query_guidelines}}",
      sep = "\n"
    )

    sp <- QueryChatSystemPrompt$new(
      prompt_template = template,
      data_source = ds
    )

    result <- sp$render("query")

    expect_false(grepl("update enabled", result))
    expect_true(grepl("query enabled", result))
    expect_true(grepl("query guidelines", result))
  })

  it("renders with update only", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    template <- paste(
      "{{#has_tool_update}}update enabled{{/has_tool_update}}",
      "{{#has_tool_query}}query enabled{{/has_tool_query}}",
      "{{#include_query_guidelines}}query guidelines{{/include_query_guidelines}}",
      sep = "\n"
    )

    sp <- QueryChatSystemPrompt$new(
      prompt_template = template,
      data_source = ds
    )

    result <- sp$render("update")

    expect_true(grepl("update enabled", result))
    expect_false(grepl("query enabled", result))
    expect_true(grepl("query guidelines", result))
  })

  it("renders with NULL tools", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    template <- paste(
      "{{#has_tool_update}}update enabled{{/has_tool_update}}",
      "{{#has_tool_query}}query enabled{{/has_tool_query}}",
      "{{#include_query_guidelines}}query guidelines{{/include_query_guidelines}}",
      "Always shown",
      sep = "\n"
    )

    sp <- QueryChatSystemPrompt$new(
      prompt_template = template,
      data_source = ds
    )

    result <- sp$render(NULL)

    expect_false(grepl("update enabled", result))
    expect_false(grepl("query enabled", result))
    expect_false(grepl("query guidelines", result))
    expect_true(grepl("Always shown", result))
  })

  it("includes schema in rendered output", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "Schema: {{schema}}",
      data_source = ds
    )

    result <- sp$render(NULL)

    expect_true(grepl("Schema:", result))
    expect_true(grepl("test_table", result))
  })

  it("includes db_type in rendered output", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "Database: {{db_type}}",
      data_source = ds
    )

    result <- sp$render(NULL)

    expect_true(grepl("Database:", result))
    expect_true(grepl("DuckDB", result))
  })

  it("includes data_description when provided", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    template <- paste(
      "{{#data_description}}Description: {{data_description}}{{/data_description}}",
      sep = "\n"
    )

    sp <- QueryChatSystemPrompt$new(
      prompt_template = template,
      data_source = ds,
      data_description = "My test data"
    )

    result <- sp$render(NULL)

    expect_true(grepl("Description: My test data", result))
  })

  it("excludes data_description when NULL", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    template <- paste(
      "{{#data_description}}Description: {{data_description}}{{/data_description}}",
      "Other content",
      sep = "\n"
    )

    sp <- QueryChatSystemPrompt$new(
      prompt_template = template,
      data_source = ds
    )

    result <- sp$render(NULL)

    expect_false(grepl("Description:", result))
    expect_true(grepl("Other content", result))
  })

  it("includes extra_instructions when provided", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    template <- paste(
      "{{#extra_instructions}}Instructions: {{extra_instructions}}{{/extra_instructions}}",
      sep = "\n"
    )

    sp <- QueryChatSystemPrompt$new(
      prompt_template = template,
      data_source = ds,
      extra_instructions = "Be concise"
    )

    result <- sp$render(NULL)

    expect_true(grepl("Instructions: Be concise", result))
  })

  it("excludes extra_instructions when NULL", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    template <- paste(
      "{{#extra_instructions}}Instructions: {{extra_instructions}}{{/extra_instructions}}",
      "Other content",
      sep = "\n"
    )

    sp <- QueryChatSystemPrompt$new(
      prompt_template = template,
      data_source = ds
    )

    result <- sp$render(NULL)

    expect_false(grepl("Instructions:", result))
    expect_true(grepl("Other content", result))
  })

  it("detects DuckDB correctly", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    template <- "{{#is_duck_db}}DuckDB detected{{/is_duck_db}}"

    sp <- QueryChatSystemPrompt$new(
      prompt_template = template,
      data_source = ds
    )

    result <- sp$render(NULL)

    # DataFrameSource uses DuckDB under the hood
    expect_true(grepl("DuckDB detected", result))
  })

  it("returns character string", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "Simple template",
      data_source = ds
    )

    result <- sp$render(NULL)

    expect_type(result, "character")
    expect_true(nchar(result) > 0)
  })
})

describe("QueryChatSystemPrompt with full prompt.md template", {
  it("renders full template with data_description", {
    df <- new_test_df(3)
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = system.file(
        "prompts",
        "prompt.md",
        package = "querychat"
      ),
      data_source = ds,
      data_description = "A test dataframe"
    )
    prompt <- sp$render(NULL)

    expect_type(prompt, "character")
    expect_true(nchar(prompt) > 0)
    expect_match(prompt, "A test dataframe")
    expect_match(prompt, "Table: test_table")
  })

  it("includes DuckDB-specific content for DuckDB sources", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = system.file(
        "prompts",
        "prompt.md",
        package = "querychat"
      ),
      data_source = ds
    )
    sys_prompt <- sp$render(NULL)

    expect_equal(ds$get_db_type(), "DuckDB")
    expect_true(grepl("DuckDB SQL Tips", sys_prompt, fixed = TRUE))
  })

  it("handles categorical_threshold with full template", {
    # Create a source with categorical data
    df_with_categories <- data.frame(
      id = 1:10,
      category = rep(c("A", "B", "C", "D", "E"), each = 2)
    )
    cat_source <- DataFrameSource$new(df_with_categories, "test_table")
    withr::defer(cat_source$cleanup())

    # With low threshold, categories should not be listed
    sp_low <- QueryChatSystemPrompt$new(
      prompt_template = system.file(
        "prompts",
        "prompt.md",
        package = "querychat"
      ),
      data_source = cat_source,
      categorical_threshold = 3
    )
    prompt_low <- sp_low$render(NULL)
    expect_false(grepl("Categorical values:", prompt_low))

    # With high threshold, categories should be listed
    sp_high <- QueryChatSystemPrompt$new(
      prompt_template = system.file(
        "prompts",
        "prompt.md",
        package = "querychat"
      ),
      data_source = cat_source,
      categorical_threshold = 10
    )
    prompt_high <- sp_high$render(NULL)
    expect_match(prompt_high, "Categorical values:")
  })
})

describe("viz prompt conditionals", {
  skip_if_no_dataframe_engine()

  default_prompt <- function() {
    system.file("prompts", "prompt.md", package = "querychat")
  }

  it("includes viz section when visualize tool is active", {
    ds <- local_data_frame_source(new_test_df())
    sp <- QueryChatSystemPrompt$new(
      prompt_template = default_prompt(),
      data_source = ds
    )
    rendered <- sp$render(tools = c("query", "visualize"))
    expect_match(rendered, "querychat_visualize")
    expect_match(rendered, "ggsql")
    expect_match(rendered, "VISUALISE")
  })

  it("excludes viz section without visualize tool", {
    ds <- local_data_frame_source(new_test_df())
    sp <- QueryChatSystemPrompt$new(
      prompt_template = default_prompt(),
      data_source = ds
    )
    rendered <- sp$render(tools = c("query"))
    expect_no_match(rendered, "querychat_visualize")
  })

  it("shows fallback message without visualize tool", {
    ds <- local_data_frame_source(new_test_df())
    sp <- QueryChatSystemPrompt$new(
      prompt_template = default_prompt(),
      data_source = ds
    )
    rendered <- sp$render(tools = c("query"))
    expect_match(rendered, "visualization is not currently enabled")
  })

  it("includes ggsql syntax reference as partial", {
    ds <- local_data_frame_source(new_test_df())
    sp <- QueryChatSystemPrompt$new(
      prompt_template = default_prompt(),
      data_source = ds
    )
    rendered <- sp$render(tools = c("visualize"))
    expect_match(rendered, "ggsql-syntax-reference")
    expect_match(rendered, "FROM sales")
    expect_match(rendered, "DRAW range")
    expect_match(rendered, "xend")
    expect_no_match(rendered, "errorbar", ignore.case = TRUE)
    expect_no_match(rendered, "DRAW segment MAPPING 0 AS yend", fixed = TRUE)
    expect_no_match(rendered, "Column casing in VISUALISE", fixed = TRUE)
    expect_no_match(rendered, "ROOM_TYPE", fixed = TRUE)
  })

  it("includes collapsed guidance when both query and visualize active", {
    ds <- local_data_frame_source(new_test_df())
    sp <- QueryChatSystemPrompt$new(
      prompt_template = default_prompt(),
      data_source = ds
    )
    rendered <- sp$render(tools = c("query", "visualize"))
    expect_match(rendered, "Avoid redundant expanded results")
  })

  it("excludes collapsed guidance with visualize only", {
    ds <- local_data_frame_source(new_test_df())
    sp <- QueryChatSystemPrompt$new(
      prompt_template = default_prompt(),
      data_source = ds
    )
    rendered <- sp$render(tools = c("visualize"))
    expect_no_match(rendered, "Avoid redundant expanded results")
  })
})

describe("Schema inference skip", {
  skip_if_no_dataframe_engine()

  it("skips schema when template doesn't reference {{schema}}", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "No schema here: {{db_type}}",
      data_source = ds
    )

    expect_equal(sp$schema, "")
  })

  it("computes schema when template uses {{schema}}", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "Schema: {{schema}}",
      data_source = ds
    )

    expect_true(nchar(sp$schema) > 0)
    expect_match(sp$schema, "test_table")
  })

  it("computes schema for {{{schema}}} triple braces", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "Schema: {{{schema}}}",
      data_source = ds
    )

    expect_true(nchar(sp$schema) > 0)
  })

  it("computes schema for {{#schema}} conditional sections", {
    df <- new_test_df()
    ds <- DataFrameSource$new(df, "test_table")
    withr::defer(ds$cleanup())

    sp <- QueryChatSystemPrompt$new(
      prompt_template = "{{#schema}}Has schema{{/schema}}",
      data_source = ds
    )

    expect_true(nchar(sp$schema) > 0)
  })
})

describe("DBISource$new()", {
  it("creates proper R6 object for DBISource", {
    db <- local_sqlite_connection(new_users_df(), "users")

    db_source <- DBISource$new(db$conn, "users")
    expect_s3_class(db_source, "DBISource")
    expect_s3_class(db_source, "DataSource")
    expect_equal(db_source$table_name, "users")
  })

  it("errors with non-DBI connection", {
    expect_snapshot(error = TRUE, {
      DBISource$new(list(fake = "connection"), "test_table")
    })

    expect_snapshot(error = TRUE, {
      DBISource$new(NULL, "test_table")
    })

    expect_snapshot(error = TRUE, {
      DBISource$new("not a connection", "test_table")
    })
  })

  it("errors with invalid table_name types", {
    db <- local_sqlite_connection(new_test_df())

    expect_snapshot(error = TRUE, {
      DBISource$new(db$conn, 123)
    })

    expect_snapshot(error = TRUE, {
      DBISource$new(db$conn, c("table1", "table2"))
    })

    expect_snapshot(error = TRUE, {
      DBISource$new(db$conn, list(name = "table"))
    })
  })

  it("errors when table does not exist", {
    db <- local_sqlite_connection(new_test_df(), "existing_table")

    expect_snapshot(error = TRUE, {
      DBISource$new(db$conn, "non_existent_table")
    })
  })
})

describe("DBISource$test_query()", {
  test_df <- new_users_df()
  db <- local_sqlite_connection(test_df, "test_table")
  dbi_source <- DBISource$new(db$conn, "test_table")

  it("correctly retrieves one row of data", {
    result <- dbi_source$test_query("SELECT * FROM test_table")
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 1)
    expect_equal(result$id, 1)

    result <- dbi_source$test_query("SELECT * FROM test_table WHERE age > 29")
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 1)
    expect_equal(result$age, 30)

    result <- dbi_source$test_query(
      "SELECT * FROM test_table ORDER BY age DESC"
    )
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 1)
    expect_equal(result$age, 35)

    result <- dbi_source$test_query(
      "SELECT * FROM test_table WHERE age > 100"
    )
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 0)
  })

  it("handles errors correctly", {
    expect_error(dbi_source$test_query("SELECT * WRONG SYNTAX"))

    expect_error(dbi_source$test_query("SELECT * FROM non_existent_table"))

    expect_error(dbi_source$test_query(
      "SELECT non_existent_column FROM test_table"
    ))
  })

  it("works with different data types", {
    db <- local_sqlite_connection(new_types_df(), "types_table")
    dbi_source <- DBISource$new(db$conn, "types_table")

    result <- dbi_source$test_query("SELECT * FROM types_table")
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 1)
    expect_type(result$text_col, "character")
    expect_type(result$num_col, "double")
    expect_type(result$int_col, "integer")
    expect_type(result$bool_col, "integer")
  })
})

describe("DataSource base class", {
  it("throws not_implemented_error for all abstract methods", {
    # Create a base DataSource object (shouldn't be done in practice)
    base_source <- DataSource$new()

    expect_snapshot(error = TRUE, {
      base_source$get_db_type()
    })

    expect_snapshot(error = TRUE, {
      base_source$get_schema()
    })

    expect_snapshot(error = TRUE, {
      base_source$execute_query("SELECT * FROM test")
    })

    expect_snapshot(error = TRUE, {
      base_source$test_query("SELECT * FROM test LIMIT 1")
    })

    expect_snapshot(error = TRUE, {
      base_source$get_data()
    })

    expect_snapshot(error = TRUE, {
      base_source$cleanup()
    })
  })
})


describe("DataSource$get_schema()", {
  it("returns proper schema for DataFrameSource", {
    skip_if_no_dataframe_engine()

    df_source <- local_data_frame_source(new_mixed_types_df())

    schema <- df_source$get_schema()
    expect_type(schema, "character")
    expect_match(schema, "Table: test_table")
    expect_match(schema, "id \\(INTEGER\\)")
    expect_match(schema, "name \\(TEXT\\)")
    expect_match(schema, "active \\(BOOLEAN\\)")
    expect_match(schema, "Categorical values")

    expect_match(schema, "- id \\(INTEGER\\)\\n  Range: 1 to 5")
  })

  it("returns proper schema for DBISource", {
    db <- local_sqlite_connection(new_test_df())

    dbi_source <- DBISource$new(db$conn, "test_table")
    schema <- dbi_source$get_schema()
    expect_type(schema, "character")
    expect_match(schema, "Table: `test_table`")
    expect_match(schema, "id \\(INTEGER\\)")
    expect_match(schema, "name \\(TEXT\\)")

    expect_match(schema, "- id \\(INTEGER\\)\\n  Range: 1 to 5")
  })

  it("correctly reports min/max values for numeric columns", {
    skip_if_no_dataframe_engine()

    df_source <- local_data_frame_source(new_metrics_df())

    schema <- df_source$get_schema()

    expect_match(schema, "- id \\(INTEGER\\)\\n  Range: 1 to 5")
    expect_match(schema, "- score \\(FLOAT\\)\\n  Range: 10\\.5 to 30\\.1")
    expect_match(schema, "- count \\(FLOAT\\)\\n  Range: 50 to 200")
  })
})

describe("DataSource$get_db_type()", {
  it("returns correct database type for DataFrameSource", {
    skip_if_no_dataframe_engine()

    df_source <- local_data_frame_source(new_test_df())
    db_type <- df_source$get_db_type()
    expect_true(db_type %in% c("DuckDB", "SQLite"))
  })

  it("returns correct type for SQLite connections", {
    skip_if_not_installed("RSQLite")

    db <- local_sqlite_connection(new_test_df())
    db_source <- DBISource$new(db$conn, "test_table")

    expect_equal(db_source$get_db_type(), "SQLite")
  })
})

describe("DataSource$get_data()", {
  test_df <- new_test_df()

  it("returns all data for both DataFrameSource and DBISource", {
    skip_if_no_dataframe_engine()
    skip_if_not_installed("RSQLite")

    df_source <- local_data_frame_source(test_df)

    result <- df_source$get_data()
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 5)
    expect_equal(ncol(result), 3)

    db <- local_sqlite_connection(test_df)

    dbi_source <- DBISource$new(db$conn, "test_table")
    result <- dbi_source$get_data()
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 5)
    expect_equal(ncol(result), 3)
  })
})

describe("DataSource$execute_query()", {
  skip_if_no_dataframe_engine()
  skip_if_not_installed("RSQLite")

  test_df <- new_test_df(rows = 4)
  df_source <- local_data_frame_source(test_df)
  db <- local_sqlite_connection(test_df)
  dbi_source <- DBISource$new(db$conn, "test_table")

  it("executes queries for DataFrameSource", {
    result <- df_source$execute_query(
      "SELECT * FROM test_table WHERE value > 25"
    )
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 2)
  })

  it("executes queries for DBISource", {
    result <- dbi_source$execute_query(
      "SELECT * FROM test_table WHERE value > 25"
    )
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 2)
  })

  it("handles empty and null queries for DataFrameSource", {
    result_null <- df_source$execute_query(NULL)
    expect_equal(result_null, test_df)

    result_empty <- df_source$execute_query("")
    expect_equal(result_empty, test_df)
  })

  it("handles empty and null queries for DBISource", {
    result_null <- dbi_source$execute_query(NULL)
    expect_equal(result_null, test_df)

    result_empty <- dbi_source$execute_query("")
    expect_equal(result_empty, test_df)
  })

  it("handles filter and sort queries in DBISource", {
    test_users_df <- new_users_df()
    db <- local_sqlite_connection(test_users_df, "users")

    db_source <- DBISource$new(db$conn, "users")

    result <- db_source$execute_query("SELECT * FROM users WHERE age > 30")
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 2)
    expect_equal(result$name, c("Charlie", "Eve"))

    all_data <- db_source$execute_query(NULL)
    expect_equal(all_data, test_users_df)

    ordered_result <- db_source$execute_query(
      "SELECT * FROM users ORDER BY age DESC"
    )
    expect_equal(ordered_result$name[1], "Charlie")
  })

  it("handles SQL with inline comments", {
    inline_comment_query <- "
    SELECT id, value -- This is a comment
    FROM test_table
    WHERE value > 25 -- Filter for higher values
    "

    result <- df_source$execute_query(inline_comment_query)
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 2)
    expect_equal(ncol(result), 2)

    multiple_comments_query <- "
    SELECT -- Get only these columns
      id, -- ID column
      value -- Value column
    FROM test_table -- Our test table
    WHERE value > 25 -- Only higher values
    "

    result <- df_source$execute_query(multiple_comments_query)
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 2)
    expect_equal(ncol(result), 2)
  })

  it("handles SQL with multiline comments", {
    multiline_comment_query <- "
    /*
     * This is a multiline comment
     * that spans multiple lines
     */
    SELECT id, value
    FROM test_table
    WHERE value > 25
    "

    result <- df_source$execute_query(multiline_comment_query)
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 2)
    expect_equal(ncol(result), 2)

    embedded_multiline_query <- "
    SELECT id, /* comment between columns */ value
    FROM /* this is
         * a multiline
         * comment
         */ test_table
    WHERE value /* another comment */ > 25
    "

    result <- df_source$execute_query(embedded_multiline_query)
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 2)
    expect_equal(ncol(result), 2)
  })

  it("handles SQL with trailing semicolons", {
    query_with_semicolon <- "
    SELECT id, value
    FROM test_table
    WHERE value > 25;
    "

    result <- df_source$execute_query(query_with_semicolon)
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 2)
    expect_equal(ncol(result), 2)

    query_with_multiple_semicolons <- "
    SELECT id, value
    FROM test_table
    WHERE value > 25;;;;
    "

    result <- df_source$execute_query(query_with_multiple_semicolons)
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 2)
    expect_equal(ncol(result), 2)
  })

  it("handles SQL with mixed comments and semicolons", {
    complex_query <- "
    /*
     * This is a complex query with different comment styles
     */
    SELECT
      id, -- This is the ID column
      value /* Value column */
    FROM
      test_table -- Our test table
    WHERE
      /* Only get higher values */
      value > 25; -- End of query
    "

    result <- df_source$execute_query(complex_query)
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 2)
    expect_equal(ncol(result), 2)

    tricky_comment_query <- "
    SELECT id, value
    FROM test_table
    /* Comment with SQL-like syntax:
     * SELECT * FROM another_table;
     */
    WHERE value > 25 -- WHERE id = 'value; DROP TABLE test;'
    "

    result <- df_source$execute_query(tricky_comment_query)
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 2)
    expect_equal(ncol(result), 2)
  })

  it("handles SQL with unusual whitespace patterns", {
    unusual_whitespace_query <- "

       SELECT   id,    value

      FROM     test_table

      WHERE    value>25

    "

    result <- df_source$execute_query(unusual_whitespace_query)
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 2)
    expect_equal(ncol(result), 2)
  })
})


describe("test_query() column validation", {
  skip_if_no_dataframe_engine()

  it("allows all columns through, regardless of order", {
    source <- local_data_frame_source(new_test_df())

    # Should succeed with all columns
    result <- source$test_query(
      "SELECT * FROM test_table",
      require_all_columns = TRUE
    )
    expect_s3_class(result, "data.frame")
    expect_equal(names(result), c("id", "name", "value"))

    # Should succeed with all columns in different order
    result <- source$test_query(
      "SELECT value, id, name FROM test_table",
      require_all_columns = TRUE
    )
    expect_s3_class(result, "data.frame")
    expect_equal(sort(names(result)), c("id", "name", "value"))
  })

  it("allows additional computed columns when require_all_columns=TRUE", {
    source <- local_data_frame_source(new_test_df())

    # Should succeed with all columns plus computed columns
    result <- source$test_query(
      "SELECT *, value * 2 as doubled FROM test_table",
      require_all_columns = TRUE
    )
    expect_s3_class(result, "data.frame")
    expect_true(all(c("id", "name", "value", "doubled") %in% names(result)))
  })

  it("fails when columns are missing and require_all_columns=TRUE", {
    source <- local_data_frame_source(new_test_df())

    # Should fail when missing one column
    expect_error(
      source$test_query(
        "SELECT id, name FROM test_table",
        require_all_columns = TRUE
      ),
      class = "querychat_missing_columns_error"
    )

    # Should fail when missing multiple columns
    expect_error(
      source$test_query(
        "SELECT id FROM test_table",
        require_all_columns = TRUE
      ),
      class = "querychat_missing_columns_error"
    )
  })

  it("does not validate when require_all_columns=FALSE (default)", {
    source <- local_data_frame_source(new_test_df())

    # Should succeed with subset of columns when not validating
    result <- source$test_query("SELECT id FROM test_table")
    expect_s3_class(result, "data.frame")
    expect_equal(names(result), "id")

    result <- source$test_query(
      "SELECT id FROM test_table",
      require_all_columns = FALSE
    )
    expect_s3_class(result, "data.frame")
    expect_equal(names(result), "id")
  })

  it("provides helpful error message listing missing columns", {
    source <- local_data_frame_source(new_test_df())

    expect_snapshot(error = TRUE, {
      source$test_query(
        "SELECT id FROM test_table",
        require_all_columns = TRUE
      )
    })

    expect_snapshot(error = TRUE, {
      source$test_query(
        "SELECT id, name FROM test_table",
        require_all_columns = TRUE
      )
    })
  })

  it("works with DBISource as well", {
    db <- local_sqlite_connection(new_test_df(), "test_table")
    source <- DBISource$new(db$conn, "test_table")

    # Should succeed with all columns
    result <- source$test_query(
      "SELECT * FROM test_table",
      require_all_columns = TRUE
    )
    expect_s3_class(result, "data.frame")
    expect_equal(names(result), c("id", "name", "value"))

    # Should fail when missing columns
    expect_error(
      source$test_query(
        "SELECT id FROM test_table",
        require_all_columns = TRUE
      ),
      class = "querychat_missing_columns_error"
    )
  })

  it("handles empty result sets correctly", {
    source <- local_data_frame_source(new_test_df())

    # Query with no matches should still validate columns
    result <- source$test_query(
      "SELECT * FROM test_table WHERE id > 999",
      require_all_columns = TRUE
    )
    expect_s3_class(result, "data.frame")
    expect_equal(nrow(result), 0)
    expect_equal(names(result), c("id", "name", "value"))
  })
})

describe("check_query() blocks dangerous operations", {
  skip_if_no_dataframe_engine()

  df_source <- local_data_frame_source(new_test_df())

  it("allows valid SELECT queries", {
    expect_no_error(check_query("SELECT * FROM test_table"))
    expect_no_error(check_query("select * from test_table"))
    expect_no_error(check_query("  SELECT * FROM test_table  "))
    expect_no_error(check_query("\nSELECT * FROM test_table\n"))
  })

  it("blocks always-blocked keywords", {
    always_blocked <- c(
      "DELETE",
      "TRUNCATE",
      "CREATE",
      "DROP",
      "ALTER",
      "GRANT",
      "REVOKE",
      "EXEC",
      "EXECUTE",
      "CALL"
    )

    for (keyword in always_blocked) {
      expect_error(
        check_query(paste(keyword, "something")),
        regexp = "disallowed operation",
        info = paste("Failed for keyword:", keyword)
      )
    }
  })

  it("blocks update keywords by default", {
    update_keywords <- c("INSERT", "UPDATE", "MERGE", "REPLACE", "UPSERT")

    for (keyword in update_keywords) {
      expect_error(
        check_query(paste(keyword, "something")),
        regexp = "update operation",
        info = paste("Failed for keyword:", keyword)
      )
    }
  })

  it("normalizes whitespace and case", {
    expect_error(check_query("  delete   FROM table  "), regexp = "disallowed")
    expect_error(check_query("\n\nDELETE\n\nFROM table"), regexp = "disallowed")
    expect_error(check_query("\tDELETE\tFROM\ttable"), regexp = "disallowed")
    expect_error(check_query("DeLeTe FROM table"), regexp = "disallowed")
  })

  it("escape hatch via option enables update keywords", {
    withr::local_options(querychat.enable_update_queries = TRUE)

    expect_no_error(check_query("INSERT INTO table VALUES (1)"))
    expect_no_error(check_query("UPDATE table SET x = 1"))
    expect_no_error(check_query("MERGE INTO table USING"))
    expect_no_error(check_query("REPLACE INTO table VALUES (1)"))
    expect_no_error(check_query("UPSERT INTO table VALUES (1)"))
  })

  it("escape hatch via envvar enables update keywords", {
    withr::local_envvar(QUERYCHAT_ENABLE_UPDATE_QUERIES = "true")

    expect_no_error(check_query("INSERT INTO table VALUES (1)"))
    expect_no_error(check_query("UPDATE table SET x = 1"))

    # Also accepts other truthy values
    withr::local_envvar(QUERYCHAT_ENABLE_UPDATE_QUERIES = "1")
    expect_no_error(check_query("INSERT INTO table VALUES (1)"))

    withr::local_envvar(QUERYCHAT_ENABLE_UPDATE_QUERIES = "YES")
    expect_no_error(check_query("INSERT INTO table VALUES (1)"))
  })

  it("escape hatch does NOT enable always-blocked keywords", {
    withr::local_options(querychat.enable_update_queries = TRUE)

    expect_error(check_query("DELETE FROM table"), regexp = "disallowed")
    expect_error(check_query("DROP TABLE table"), regexp = "disallowed")
    expect_error(check_query("TRUNCATE TABLE table"), regexp = "disallowed")
  })

  it("is integrated into execute_query()", {
    expect_error(
      df_source$execute_query("DELETE FROM test_table"),
      regexp = "disallowed operation"
    )
    expect_error(
      df_source$execute_query("INSERT INTO test_table VALUES (1, 'a', 1)"),
      regexp = "update operation"
    )
  })

  it("does not block keywords in column names or values", {
    expect_no_error(check_query("SELECT update_count FROM table"))
    expect_no_error(check_query("SELECT * FROM delete_logs"))
  })
})

test_that("subclasses of DataSource implement its methods", {
  # Get all exported objects from the package namespace
  ns <- asNamespace("querychat")
  exported_names <- getNamespaceExports(ns)

  # Helper function to check if an R6 class inherits from DataSource
  inherits_from_datasource <- function(r6_class) {
    if (is.null(r6_class) || !R6::is.R6Class(r6_class)) {
      return(FALSE)
    }
    if (r6_class$classname == "DataSource") {
      return(TRUE)
    }
    # Check parent class recursively using get_inherit()
    tryCatch(
      {
        parent <- r6_class$get_inherit()
        if (!is.null(parent)) {
          return(inherits_from_datasource(parent))
        }
      },
      error = function(e) {
        # No parent class
      }
    )
    return(FALSE)
  }

  # Find all R6 classes that inherit from DataSource
  datasource_subclasses <- list()
  for (name in exported_names) {
    obj <- get(name, envir = ns)
    if (R6::is.R6Class(obj) && obj$classname != "DataSource") {
      if (inherits_from_datasource(obj)) {
        datasource_subclasses[[name]] <- obj
      }
    }
  }

  # Ensure we found some subclasses
  skip_if(
    length(datasource_subclasses) == 0,
    "querychat doesn't include any DataSource subclasses?"
  )

  # Get the base DataSource methods and their formal arguments
  base_methods <- lapply(DataSource$public_methods, formals)

  # Helper to get all methods (including inherited) for a class
  get_all_methods <- function(r6_class) {
    all_methods <- list()
    current_class <- r6_class

    while (!is.null(current_class) && R6::is.R6Class(current_class)) {
      # Add methods from current class
      for (method_name in names(current_class$public_methods)) {
        if (!method_name %in% names(all_methods)) {
          all_methods[[method_name]] <- current_class$public_methods[[
            method_name
          ]]
        }
      }

      # Move to parent class
      tryCatch(
        {
          current_class <- current_class$get_inherit()
        },
        error = function(e) {
          current_class <- NULL
        }
      )
    }

    all_methods
  }

  # Check each subclass
  for (class_name in names(datasource_subclasses)) {
    subclass <- datasource_subclasses[[class_name]]

    # Get all methods including inherited ones
    all_methods <- get_all_methods(subclass)

    for (method_name in names(base_methods)) {
      # Check that the method exists (including inherited methods)
      expect(
        method_name %in% names(all_methods),
        cli::format_inline(
          "{.cls {class_name}} must implement required `DataSource` method {.code {method_name}()}."
        )
      )

      if (method_name %in% names(all_methods)) {
        # Get the method's formal arguments
        subclass_formals <- formals(all_methods[[method_name]])
        base_formals <- base_methods[[method_name]]

        # Get argument names (excluding ...)
        base_args <- names(base_formals)
        subclass_args <- names(subclass_formals)

        # Check that all base arguments are present in subclass
        # (subclass can have additional arguments)
        missing_args <- setdiff(base_args, subclass_args)

        expect(
          length(missing_args) == 0,
          cli::format_inline(
            "{.code {class_name}${method_name}()} does not implement all arguments from {.code DataSource${method_name}()}. ",
            "\nMissing: {.and {.code {missing_args}}}"
          )
        )
      }
    }
  }
})

describe("query tool collapsed parameter", {
  skip_if_no_dataframe_engine()

  it("collapsed=TRUE sets open=FALSE in display", {
    ds <- local_data_frame_source(new_test_df())
    tool <- tool_query(ds)
    result <- tool(
      query = "SELECT * FROM test_table",
      `_intent` = "",
      collapsed = TRUE
    )
    expect_false(result@extra$display$open)
  })

  it("collapsed=FALSE sets open=TRUE in display", {
    ds <- local_data_frame_source(new_test_df())
    tool <- tool_query(ds)
    result <- tool(
      query = "SELECT * FROM test_table",
      `_intent` = "",
      collapsed = FALSE
    )
    expect_true(result@extra$display$open)
  })

  it("collapsed defaults to TRUE (open=FALSE for query)", {
    ds <- local_data_frame_source(new_test_df())
    tool <- tool_query(ds)
    result <- tool(query = "SELECT * FROM test_table", `_intent` = "")
    expect_false(result@extra$display$open)
  })

  it("collapsed=TRUE overrides QUERYCHAT_TOOL_DETAILS=expanded", {
    withr::local_options(querychat.tool_details = "expanded")
    ds <- local_data_frame_source(new_test_df())
    tool <- tool_query(ds)
    result <- tool(
      query = "SELECT * FROM test_table",
      `_intent` = "",
      collapsed = TRUE
    )
    expect_false(result@extra$display$open)
  })

  it("collapsed=FALSE overrides QUERYCHAT_TOOL_DETAILS=collapsed", {
    withr::local_options(querychat.tool_details = "collapsed")
    ds <- local_data_frame_source(new_test_df())
    tool <- tool_query(ds)
    result <- tool(
      query = "SELECT * FROM test_table",
      `_intent` = "",
      collapsed = FALSE
    )
    expect_true(result@extra$display$open)
  })
})

test_that("Shiny app example loads without errors", {
  skip_if_not_installed("DT")
  skip_if_not_installed("RSQLite")
  skip_if_not_installed("shinytest2")

  # Create a simplified test app with mocked ellmer
  test_app_dir <- withr::local_tempdir()
  test_app_file <- file.path(test_app_dir, "app.R")
  dir.create(dirname(test_app_file), showWarnings = FALSE)

  file.copy(test_path("apps/basic/app.R"), test_app_file)

  # Test that the app can be loaded without immediate errors
  expect_no_error({
    # Try to parse and evaluate the app code
    source(test_app_file, local = TRUE)
  })
})

test_that("mod_server() passes visualize callback and tools to client factory", {
  skip_if_no_dataframe_engine()

  ds <- local_data_frame_source(new_test_df())
  captured <- NULL

  client_factory <- function(...) {
    captured <<- list(...)
    structure(list(), class = "MockChat")
  }

  shiny::testServer(
    mod_server,
    args = list(
      id = "test",
      data_source = ds,
      greeting = "Hello",
      client = client_factory,
      tools = c("query", "visualize"),
      enable_bookmarking = FALSE
    ),
    {
      expect_type(captured, "list")
      expect_equal(captured$tools, c("query", "visualize"))
      expect_true(is.function(captured$visualize))
      expect_true(is.function(captured$update_dashboard))
      expect_true(is.function(captured$reset_dashboard))
    }
  )
})

test_that("restored viz widgets survive a second bookmark cycle", {
  skip_if_no_dataframe_engine()

  ds <- local_data_frame_source(new_test_df())
  callbacks <- NULL
  bookmark_fn <- NULL
  restore_fn <- NULL
  restored_args <- NULL

  client_factory <- function(...) {
    callbacks <<- list(...)
    structure(list(), class = "MockChat")
  }

  local_mocked_bindings(
    chat_restore = function(id, chat, session) {},
    .package = "shinychat"
  )
  local_mocked_bindings(
    onBookmark = function(fun) {
      bookmark_fn <<- fun
    },
    onRestore = function(fun) {
      restore_fn <<- fun
    },
    .package = "shiny"
  )
  local_mocked_bindings(
    restore_viz_widgets = function(data_source, saved_widgets, session) {
      restored_args <<- list(
        data_source = data_source,
        saved_widgets = saved_widgets,
        session = session
      )
      saved_widgets
    },
    .package = "querychat"
  )

  shiny::testServer(
    mod_server,
    args = list(
      id = "test",
      data_source = ds,
      greeting = "Hello",
      client = client_factory,
      tools = c("query", "visualize"),
      enable_bookmarking = TRUE
    ),
    {
      expect_true(is.function(bookmark_fn))
      expect_true(is.function(restore_fn))
      expect_true(is.function(callbacks$visualize))

      saved <- list(
        list(
          widget_id = "querychat_viz_1",
          ggsql = "SELECT 1 VISUALISE 1 AS x DRAW point"
        )
      )

      shiny::isolate(callbacks$visualize(saved[[1]]))

      first_state <- new.env(parent = emptyenv())
      first_state$values <- list()
      shiny::isolate(bookmark_fn(first_state))
      expect_equal(first_state$values$querychat_viz_widgets, saved)

      restore_state <- new.env(parent = emptyenv())
      restore_state$values <- first_state$values
      shiny::isolate(restore_fn(restore_state))
      expect_identical(restored_args$data_source, ds)
      expect_equal(restored_args$saved_widgets, saved)

      second_state <- new.env(parent = emptyenv())
      second_state$values <- list()
      shiny::isolate(bookmark_fn(second_state))
      expect_equal(second_state$values$querychat_viz_widgets, saved)
    }
  )
})

#' querychat: Chat with Your Data Using Natural Language
#'
#' @description
#' querychat provides an interactive chat interface for querying data using
#' natural language. It translates your questions into SQL queries, executes
#' them against your data, and displays the results. The package works with
#' both data frames and database connections.
#'
#' @section Quick Start:
#' The easiest way to get started is with the [QueryChat] R6 class:
#'
#' ```r
#' library(querychat)
#'
#' # Create a QueryChat object (table name inferred from variable)
#' qc <- QueryChat$new(mtcars)
#'
#' # Option 1: Run a complete app with sensible defaults
#' qc$app()
#'
#' # Option 2: Build a custom Shiny app
#' ui <- page_sidebar(
#'   qc$sidebar(),
#'   dataTableOutput("data")
#' )
#'
#' server <- function(input, output, session) {
#'   qc$server()
#'   output$data <- renderDataTable(qc$df())
#' }
#'
#' shinyApp(ui, server)
#' ```
#'
#' @section Key Features:
#' - **Natural language queries**: Ask questions in plain English
#' - **SQL transparency**: See the generated SQL queries
#' - **Multiple data sources**: Works with data frames and database connections
#' - **Customizable**: Add data descriptions, extra instructions, and custom greetings
#' - **LLM agnostic**: Works with OpenAI, Anthropic, Google, and other providers via ellmer
#'
#' @section Main Components:
#' - [QueryChat]: The main R6 class for creating chat interfaces
#' - [DataSource], [DataFrameSource], [DBISource]: R6 classes for data sources
#'
#' @section Examples:
#' To see examples included with the package, run:
#'
#' ```r
#' shiny::runExample(package = "querychat")
#' ```
#'
#' This provides a list of available examples. To run a specific example, like
#' '01-hello-app', use:
#'
#' ```r
#' shiny::runExample("01-hello-app", package = "querychat")
#' ```
#'
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom lifecycle deprecated
#' @importFrom R6 R6Class
#' @importFrom bslib sidebar
#' @import rlang
## usethis namespace: end
NULL

# @staticimports pkg:staticimports
#   read_utf8

# enable usage of <S7_object>@name in package code
#' @rawNamespace if (getRversion() < "4.3.0") importFrom("S7", "@")
NULL

release_bullets <- function() {
  c(
    "Run `staticimports::import()` to update static imports",
    "Enable `development.mode: auto` in `_pkgdown.yml` and remove this release bullet."
  )
}

suppress_rcmdcheck <- function() {
  S7::S7_class
  whisker::whisker.render
}

# Main module UI function
mod_ui <- function(id, ..., enable_cancel = TRUE) {
  ns <- shiny::NS(id)
  htmltools::tagList(
    htmltools::htmlDependency(
      "querychat",
      version = "0.0.1",
      package = "querychat",
      src = "htmldep",
      script = "querychat.js",
      stylesheet = "styles.css"
    ),
    shinychat::chat_ui(
      ns("chat"),
      height = "100%",
      class = "querychat",
      enable_cancel = enable_cancel,
      ...
    )
  )
}

# Main module server function
mod_server <- function(
  id,
  data_source,
  greeting,
  client,
  tools,
  enable_bookmarking = FALSE
) {
  shiny::moduleServer(id, function(input, output, session) {
    current_title <- shiny::reactiveVal(NULL, label = "current_title")
    current_query <- shiny::reactiveVal(NULL, label = "current_query")
    has_greeted <- shiny::reactiveVal(FALSE, label = "has_greeted")
    filtered_df <- shiny::reactive(label = "filtered_df", {
      data_source$execute_query(query = current_query())
    })

    append_output <- function(...) {
      txt <- paste0(...)
      shinychat::chat_append_message(
        "chat",
        list(role = "assistant", content = txt),
        chunk = TRUE,
        operation = "append",
        session = session
      )
    }

    update_dashboard <- function(query, title) {
      if (!is.null(query)) {
        current_query(query)
      }
      if (!is.null(title)) {
        current_title(title)
      }
    }

    reset_query <- function() {
      current_query(NULL)
      current_title(NULL)
      querychat_tool_result(action = "reset")
    }

    # Non-reactive bookkeeping for bookmark save/restore of viz widgets
    viz_widgets <- list()

    on_visualize <- function(data) {
      viz_widgets[[length(viz_widgets) + 1L]] <<- list(
        widget_id = data$widget_id,
        ggsql = data$ggsql
      )
    }

    # Set up the chat object for this session
    check_function(client)
    chat <- client(
      update_dashboard = update_dashboard,
      reset_dashboard = reset_query,
      visualize = on_visualize,
      tools = tools,
      session = session
    )

    # Prepopulate the chat UI with a welcome message that appears to be from the
    # chat model (but is actually hard-coded). This is just for the user, not for
    # the chat model to see.
    shiny::observe(label = "greet_on_startup", {
      if (has_greeted()) {
        return()
      }

      greeting_content <- if (!is.null(greeting) && any(nzchar(greeting))) {
        greeting
      } else {
        cli::cli_warn(c(
          "No {.arg greeting} provided to {.fn QueryChat}. Using the LLM {.arg client} to generate one now.",
          "i" = "For faster startup, lower cost, and determinism, consider providing a {.arg greeting} to {.fn QueryChat}.",
          "i" = "You can use your {.help querychat::QueryChat} object's {.fn $generate_greeting} method to generate a greeting."
        ))
        chat$stream_async(GREETING_PROMPT)
      }

      shinychat::chat_append("chat", greeting_content)
      has_greeted(TRUE)
    })

    ctrl <- ellmer::stream_controller()

    append_stream_task <- shiny::ExtendedTask$new(
      function(client, user_input, controller = NULL) {
        stream <- client$stream_async(
          user_input,
          stream = "content",
          controller = controller
        )

        p <- promises::promise_resolve(stream)
        promises::then(p, function(stream) {
          shinychat::chat_append("chat", stream)
        })
      }
    )

    shiny::observeEvent(input$chat_user_input, label = "on_chat_user_input", {
      append_stream_task$invoke(chat, input$chat_user_input, controller = ctrl)
    })

    shiny::observeEvent(input$chat_cancel, label = "on_chat_cancel", {
      ctrl$cancel()
    })

    shiny::observeEvent(input$chat_update, label = "on_chat_update", {
      current_query(input$chat_update$query)
      current_title(input$chat_update$title)
    })

    if (enable_bookmarking) {
      shinychat::chat_restore("chat", chat, session = session)

      shiny::onBookmark(function(state) {
        state$values$querychat_sql <- current_query()
        state$values$querychat_title <- current_title()
        state$values$querychat_has_greeted <- has_greeted()
        if (length(viz_widgets) > 0) {
          state$values$querychat_viz_widgets <- viz_widgets
        }
      })

      shiny::onRestore(function(state) {
        if (!is.null(state$values$querychat_sql)) {
          current_query(state$values$querychat_sql)
        }
        if (!is.null(state$values$querychat_title)) {
          current_title(state$values$querychat_title)
        }
        if (!is.null(state$values$querychat_has_greeted)) {
          has_greeted(state$values$querychat_has_greeted)
        }
        if (!is.null(state$values$querychat_viz_widgets)) {
          restored <- restore_viz_widgets(
            data_source,
            state$values$querychat_viz_widgets,
            session
          )
          viz_widgets <<- restored
        }
      })
    }

    list(
      client = chat,
      sql = current_query,
      title = current_title,
      df = filtered_df
    )
  })
}

# TODO: Make this dependent on enabled tools
GREETING_PROMPT <- paste(
  "Please give me a friendly greeting.",
  "Include a few sample suggestions grouped under ##### headings,",
  "using the suggestion card format from your instructions."
)

restore_viz_widgets <- function(data_source, saved_widgets, session) {
  if (!rlang::is_installed("ggsql")) {
    warning(
      "ggsql is not installed; skipping restoration of visualization widgets.",
      call. = FALSE
    )
    return(list())
  }

  restored <- list()
  for (entry in saved_widgets) {
    tryCatch(
      {
        validated <- ggsql::ggsql_validate(entry$ggsql)
        spec <- execute_ggsql(data_source, validated)
        session$output[[entry$widget_id]] <- ggsql::renderGgsql(spec)
        restored <- c(restored, list(entry))
      },
      error = function(e) {
        warning(
          sprintf(
            "Failed to restore visualization widget '%s' on bookmark restore.",
            entry$widget_id
          ),
          call. = FALSE
        )
      }
    )
  }
  restored
}

#' DBI Source
#'
#' A DataSource implementation for DBI database connections (SQLite, PostgreSQL,
#' MySQL, etc.). This class wraps a DBI connection and provides SQL query
#' execution against a single table in the database.
#'
#' @examplesIf rlang::is_installed("RSQLite")
#' # Connect to a database
#' con <- DBI::dbConnect(RSQLite::SQLite(), ":memory:")
#' DBI::dbWriteTable(con, "mtcars", mtcars)
#'
#' # Create a DBI source
#' db_source <- DBISource$new(con, "mtcars")
#'
#' # Get database type
#' db_source$get_db_type()  # Returns "SQLite"
#'
#' # Execute a query
#' result <- db_source$execute_query("SELECT * FROM mtcars WHERE mpg > 25")
#'
#' # Note: cleanup() will disconnect the connection
#' # If you want to keep the connection open, don't call cleanup()
#' db_source$cleanup()
#'
#' @export
DBISource <- R6::R6Class(
  "DBISource",
  inherit = DataSource,
  private = list(
    conn = NULL
  ),
  public = list(
    #' @description
    #' Create a new DBISource
    #'
    #' @param conn A DBI connection object
    #' @param table_name Name of the table in the database. Can be a character
    #'   string or a [DBI::Id()] object for tables in catalogs/schemas
    #'
    #' @return A new DBISource object
    initialize = function(conn, table_name) {
      if (!inherits(conn, "DBIConnection")) {
        cli::cli_abort(
          "{.arg conn} must be a {.cls DBIConnection}, not {.obj_type_friendly {conn}}"
        )
      }

      # Validate table_name type
      if (inherits(table_name, "Id")) {
        # DBI::Id object - keep as is
      } else if (is.character(table_name) && length(table_name) == 1) {
        # Character string - keep as is
      } else {
        cli::cli_abort(
          "{.arg table_name} must be a single character string or a {.fn DBI::Id} object"
        )
      }

      # Check if table exists
      if (!DBI::dbExistsTable(conn, table_name)) {
        cli::cli_abort(c(
          "Table {.val {DBI::dbQuoteIdentifier(conn, table_name)}} not found in database",
          "i" = "If you're using a table in a catalog or schema, pass a {.fn DBI::Id} object to {.arg table_name}"
        ))
      }

      private$conn <- conn
      self$table_name <- table_name

      # Store original column names for validation
      # Use WHERE 1=0 instead of LIMIT 0 for SQL Server compatibility
      private$colnames <- colnames(DBI::dbGetQuery(
        conn,
        sprintf(
          "SELECT * FROM %s WHERE 1=0",
          DBI::dbQuoteIdentifier(conn, table_name)
        )
      ))
    },

    #' @description Get the database type
    #' @return A string identifying the database type
    get_db_type = function() {
      # Special handling for known database types
      if (inherits(private$conn, "duckdb_connection")) {
        return("DuckDB")
      }
      if (inherits(private$conn, "SQLiteConnection")) {
        return("SQLite")
      }

      # Default to 'POSIX' if dbms name not found
      conn_info <- DBI::dbGetInfo(private$conn)
      dbms_name <- getElement(conn_info, "dbms.name") %||% "POSIX"

      # Remove ' SQL', if exists (SQL is already in the prompt)
      gsub(" SQL", "", dbms_name)
    },

    #' @description
    #' Get schema information for the database table
    #'
    #' @param categorical_threshold Maximum number of unique values for a text
    #'   column to be considered categorical (default: 20)
    #' @return A string describing the schema
    get_schema = function(categorical_threshold = 20) {
      check_number_whole(categorical_threshold, min = 1)
      get_schema_impl(private$conn, self$table_name, categorical_threshold)
    },

    #' @description
    #' Get information about semantic views (if any) for the system prompt.
    #' @return A string with semantic view information, or empty string if none
    get_semantic_views_description = function() {
      if (!is_snowflake_connection(private$conn)) {
        return("")
      }
      views <- discover_semantic_views_impl(private$conn)
      if (length(views) == 0) {
        return("")
      }
      format_semantic_views(views)
    },

    #' @description
    #' Execute a SQL query
    #'
    #' @param query SQL query string. If NULL or empty, returns all data
    #' @return A data frame with query results
    execute_query = function(query) {
      check_string(query, allow_null = TRUE, allow_empty = TRUE)
      if (is.null(query) || !nzchar(query)) {
        query <- paste0(
          "SELECT * FROM ",
          DBI::dbQuoteIdentifier(private$conn, self$table_name)
        )
      }

      check_query(query)
      DBI::dbGetQuery(private$conn, query)
    },

    #' @description
    #' Test a SQL query by fetching only one row
    #'
    #' @param query SQL query string
    #' @param require_all_columns If `TRUE`, validates that the result includes
    #'   all original table columns (default: `FALSE`)
    #' @return A data frame with one row of results
    test_query = function(query, require_all_columns = FALSE) {
      check_string(query)
      check_bool(require_all_columns)
      check_query(query)

      rs <- DBI::dbSendQuery(private$conn, query)
      df <- DBI::dbFetch(rs, n = 1)
      DBI::dbClearResult(rs)

      if (require_all_columns) {
        result_columns <- names(df)
        missing_columns <- setdiff(private$colnames, result_columns)

        if (length(missing_columns) > 0) {
          missing_list <- paste0("'", missing_columns, "'", collapse = ", ")
          cli::cli_abort(
            c(
              "Query result missing required columns: {missing_list}",
              "i" = "The query must return all original table columns (in any order)."
            ),
            class = "querychat_missing_columns_error"
          )
        }
      }

      df
    },

    #' @description
    #' Get all data from the table
    #'
    #' @return A data frame containing all data
    get_data = function() {
      self$execute_query(NULL)
    },

    #' @description
    #' Disconnect from the database
    #'
    #' @return NULL (invisibly)
    cleanup = function() {
      if (!is.null(private$conn) && DBI::dbIsValid(private$conn)) {
        DBI::dbDisconnect(private$conn)
      }
      invisible(NULL)
    }
  )
)


get_schema_impl <- function(
  conn,
  table_name,
  categorical_threshold = 20,
  columns = NULL,
  prep_query = identity
) {
  check_function(prep_query)

  # Get column information
  columns <- columns %||% DBI::dbListFields(conn, table_name)

  schema_lines <- c(
    paste("Table:", DBI::dbQuoteIdentifier(conn, table_name)),
    "Columns:"
  )

  # Build single query to get column statistics
  select_parts <- character(0)
  numeric_columns <- character(0)
  text_columns <- character(0)

  # Get sample of data to determine types
  # Use dbFetch(n=1) instead of LIMIT 1 for SQL Server compatibility
  sample_query <- paste0(
    "SELECT * FROM ",
    DBI::dbQuoteIdentifier(conn, table_name)
  )
  rs <- DBI::dbSendQuery(conn, prep_query(sample_query))
  sample_data <- DBI::dbFetch(rs, n = 1)
  DBI::dbClearResult(rs)

  for (col in columns) {
    col_class <- class(sample_data[[col]])[1]

    if (
      col_class %in%
        c("integer", "numeric", "double", "Date", "POSIXct", "POSIXt")
    ) {
      numeric_columns <- c(numeric_columns, col)
      select_parts <- c(
        select_parts,
        paste0(
          "MIN(",
          DBI::dbQuoteIdentifier(conn, col),
          ") as ",
          DBI::dbQuoteIdentifier(conn, paste0(col, '__min'))
        ),
        paste0(
          "MAX(",
          DBI::dbQuoteIdentifier(conn, col),
          ") as ",
          DBI::dbQuoteIdentifier(conn, paste0(col, '__max'))
        )
      )
    } else if (col_class %in% c("character", "factor")) {
      text_columns <- c(text_columns, col)
      select_parts <- c(
        select_parts,
        paste0(
          "COUNT(DISTINCT ",
          DBI::dbQuoteIdentifier(conn, col),
          ") as ",
          DBI::dbQuoteIdentifier(conn, paste0(col, '__distinct_count'))
        )
      )
    }
  }

  # Execute statistics query
  column_stats <- list()
  if (length(select_parts) > 0) {
    tryCatch(
      {
        stats_query <- paste0(
          "SELECT ",
          paste0(select_parts, collapse = ", "),
          " FROM ",
          DBI::dbQuoteIdentifier(conn, table_name)
        )
        result <- DBI::dbGetQuery(conn, prep_query(stats_query))
        if (nrow(result) > 0) {
          column_stats <- as.list(result[1, ])
        }
      },
      error = function(e) {
        # Fall back to no statistics if query fails
      }
    )
  }

  # Get categorical values for text columns below threshold
  categorical_values <- list()
  text_cols_to_query <- character(0)

  for (col_name in text_columns) {
    distinct_count_key <- paste0(col_name, "__distinct_count")
    if (
      distinct_count_key %in%
        names(column_stats) &&
        !is.na(column_stats[[distinct_count_key]]) &&
        column_stats[[distinct_count_key]] <= categorical_threshold
    ) {
      text_cols_to_query <- c(text_cols_to_query, col_name)
    }
  }

  # Remove duplicates
  text_cols_to_query <- unique(text_cols_to_query)

  # Get categorical values
  if (length(text_cols_to_query) > 0) {
    for (col_name in text_cols_to_query) {
      tryCatch(
        {
          cat_query <- paste0(
            "SELECT DISTINCT ",
            DBI::dbQuoteIdentifier(conn, col_name),
            " FROM ",
            DBI::dbQuoteIdentifier(conn, table_name),
            " WHERE ",
            DBI::dbQuoteIdentifier(conn, col_name),
            " IS NOT NULL ORDER BY ",
            DBI::dbQuoteIdentifier(conn, col_name)
          )
          result <- DBI::dbGetQuery(conn, prep_query(cat_query))
          if (nrow(result) > 0) {
            categorical_values[[col_name]] <- result[[1]]
          }
        },
        error = function(e) {
          # Skip categorical values if query fails
        }
      )
    }
  }

  # Build schema description
  for (col in columns) {
    col_class <- class(sample_data[[col]])[1]
    sql_type <- r_class_to_sql_type(col_class)

    column_info <- paste0("- ", col, " (", sql_type, ")")

    # Add range info for numeric columns
    if (col %in% numeric_columns) {
      min_key <- paste0(col, "__min")
      max_key <- paste0(col, "__max")
      if (
        min_key %in%
          names(column_stats) &&
          max_key %in% names(column_stats) &&
          !is.na(column_stats[[min_key]]) &&
          !is.na(column_stats[[max_key]])
      ) {
        range_info <- paste0(
          "  Range: ",
          column_stats[[min_key]],
          " to ",
          column_stats[[max_key]]
        )
        column_info <- paste(column_info, range_info, sep = "\n")
      }
    }

    # Add categorical values for text columns
    if (col %in% names(categorical_values)) {
      values <- categorical_values[[col]]
      if (length(values) > 0) {
        values_str <- paste0("'", values, "'", collapse = ", ")
        cat_info <- paste0("  Categorical values: ", values_str)
        column_info <- paste(column_info, cat_info, sep = "\n")
      }
    }

    schema_lines <- c(schema_lines, column_info)
  }

  paste(schema_lines, collapse = "\n")
}


# nocov start
# Map R classes to SQL types
r_class_to_sql_type <- function(r_class) {
  switch(
    r_class,
    "integer" = "INTEGER",
    "numeric" = "FLOAT",
    "double" = "FLOAT",
    "logical" = "BOOLEAN",
    "Date" = "DATE",
    "POSIXct" = "TIMESTAMP",
    "POSIXt" = "TIMESTAMP",
    "character" = "TEXT",
    "factor" = "TEXT",
    "TEXT" # default
  )
}
# nocov end

# Snowflake Semantic Views Support ----

#' Check if a connection is a Snowflake connection
#'
#' @param conn A DBI connection object
#' @return TRUE if the connection is to Snowflake
#' @noRd
is_snowflake_connection <- function(conn) {
  if (!inherits(conn, "DBIConnection")) {
    return(FALSE)
  }

  # Check for known Snowflake connection classes
  if (inherits(conn, "Snowflake")) {
    return(TRUE)
  }

  # Check dbms.name from connection info
  tryCatch(
    {
      conn_info <- DBI::dbGetInfo(conn)
      dbms_name <- tolower(conn_info[["dbms.name"]] %||% "")
      grepl("snowflake", dbms_name, ignore.case = TRUE)
    },
    error = function(e) FALSE
  )
}

#' Discover Semantic Views in Snowflake
#'
#' @param conn A DBI connection to Snowflake
#' @return A list of semantic views with name and ddl
#' @noRd
discover_semantic_views_impl <- function(conn) {
  # Check env var for early exit
  if (nzchar(Sys.getenv("QUERYCHAT_DISABLE_SEMANTIC_VIEWS", ""))) {
    return(list())
  }

  semantic_views <- list()

  # Check for semantic views in the current schema
  result <- DBI::dbGetQuery(conn, "SHOW SEMANTIC VIEWS")

  if (nrow(result) == 0) {
    cli::cli_inform(
      c("i" = "No semantic views found in current schema"),
      .frequency = "once",
      .frequency_id = "querychat_no_semantic_views"
    )
    return(list())
  }

  for (i in seq_len(nrow(result))) {
    row <- result[i, ]
    view_name <- row[["name"]]
    database_name <- row[["database_name"]]
    schema_name <- row[["schema_name"]]

    if (is.null(view_name) || is.na(view_name)) {
      next
    }

    # Build fully qualified name
    fq_name <- paste(database_name, schema_name, view_name, sep = ".")

    # Get the DDL for this semantic view
    ddl <- get_semantic_view_ddl(conn, fq_name)
    if (!is.null(ddl)) {
      semantic_views <- c(
        semantic_views,
        list(
          list(
            name = fq_name,
            ddl = ddl
          )
        )
      )
    }
  }

  semantic_views
}

#' Get the DDL for a Semantic View
#'
#' @param conn A DBI connection to Snowflake
#' @param fq_name Fully qualified name (database.schema.view_name)
#' @return The DDL text, or NULL if retrieval failed
#' @noRd
get_semantic_view_ddl <- function(conn, fq_name) {
  # Escape single quotes to prevent SQL injection
  safe_name <- gsub("'", "''", fq_name, fixed = TRUE)
  query <- sprintf("SELECT GET_DDL('SEMANTIC_VIEW', '%s')", safe_name)
  result <- DBI::dbGetQuery(conn, query)
  if (nrow(result) > 0 && ncol(result) > 0) {
    as.character(result[[1, 1]])
  } else {
    NULL
  }
}

#' Format Semantic View DDLs
#'
#' @param semantic_views A list of semantic view info (name and ddl)
#' @return A formatted string with just the DDL definitions
#' @noRd
format_semantic_view_ddls <- function(semantic_views) {
  lines <- character(0)

  for (sv in semantic_views) {
    lines <- c(
      lines,
      sprintf("### Semantic View: `%s`", sv$name),
      "",
      "```sql",
      sv$ddl,
      "```",
      ""
    )
  }

  paste(lines, collapse = "\n")
}

#' Build the complete semantic views section for the prompt
#'
#' @param semantic_views A list of semantic view info (name and ddl)
#' @return A formatted string with the full semantic views section
#' @noRd
format_semantic_views <- function(semantic_views) {
  if (length(semantic_views) == 0) {
    return("")
  }

  prompts_dir <- system.file("prompts", "semantic-views", package = "querychat")
  prompt_text <- readLines(file.path(prompts_dir, "prompt.md"), warn = FALSE)
  syntax_text <- readLines(file.path(prompts_dir, "syntax.md"), warn = FALSE)
  ddls_text <- format_semantic_view_ddls(semantic_views)

  paste(
    paste(prompt_text, collapse = "\n"),
    "",
    paste(syntax_text, collapse = "\n"),
    "",
    "<semantic_views>",
    ddls_text,
    "</semantic_views>",
    sep = "\n"
  )
}

# Tests for Snowflake semantic view functionality in DBISource

describe("format_semantic_view_ddls()", {
  it("formats single semantic view correctly", {
    views <- list(
      list(name = "db.schema.view", ddl = "CREATE SEMANTIC VIEW test_view")
    )
    result <- format_semantic_view_ddls(views)

    expect_match(result, "db.schema.view")
    expect_match(result, "CREATE SEMANTIC VIEW test_view")
    expect_match(result, "```sql")
  })

  it("formats multiple views", {
    views <- list(
      list(name = "db.schema.view1", ddl = "CREATE SEMANTIC VIEW v1"),
      list(name = "db.schema.view2", ddl = "CREATE SEMANTIC VIEW v2")
    )
    result <- format_semantic_view_ddls(views)

    expect_match(result, "db.schema.view1")
    expect_match(result, "db.schema.view2")
    expect_match(result, "CREATE SEMANTIC VIEW v1")
    expect_match(result, "CREATE SEMANTIC VIEW v2")
  })
})

describe("format_semantic_views()", {
  it("includes IMPORTANT notice", {
    views <- list(
      list(name = "test", ddl = "DDL")
    )
    result <- format_semantic_views(views)
    expect_match(result, "\\*\\*IMPORTANT\\*\\*")
  })

  it("includes section header", {
    views <- list(
      list(name = "test", ddl = "DDL")
    )
    result <- format_semantic_views(views)
    expect_match(result, "## Semantic Views")
  })

  it("returns empty string for empty views list", {
    result <- format_semantic_views(list())
    expect_equal(result, "")
  })
})

describe("SQL escaping in get_semantic_view_ddl()", {
  it("escapes single quotes in view names", {
    # We can't test the full function without a Snowflake connection,
    # but we can test the escaping logic directly
    fq_name <- "db.schema.test'view"
    safe_name <- gsub("'", "''", fq_name, fixed = TRUE)

    expect_equal(safe_name, "db.schema.test''view")
  })

  it("leaves normal names unchanged", {
    fq_name <- "db.schema.normal_view"
    safe_name <- gsub("'", "''", fq_name, fixed = TRUE)

    expect_equal(safe_name, "db.schema.normal_view")
  })
})

describe("is_snowflake_connection()", {
  it("returns FALSE for non-Snowflake connections", {
    skip_if_not_installed("RSQLite")

    conn <- DBI::dbConnect(RSQLite::SQLite(), ":memory:")
    withr::defer(DBI::dbDisconnect(conn))

    expect_false(is_snowflake_connection(conn))
  })

  it("returns FALSE for non-DBI objects", {
    expect_false(is_snowflake_connection(NULL))
    expect_false(is_snowflake_connection("not a connection"))
    expect_false(is_snowflake_connection(list(fake = "connection")))
    expect_false(is_snowflake_connection(123))
  })
})

describe("DBISource semantic views", {
  it("get_semantic_views_description() returns empty for non-Snowflake", {
    skip_if_not_installed("RSQLite")

    conn <- DBI::dbConnect(RSQLite::SQLite(), ":memory:")
    withr::defer(DBI::dbDisconnect(conn))
    DBI::dbWriteTable(conn, "test_table", data.frame(x = 1:3))

    source <- DBISource$new(conn, "test_table")
    expect_equal(source$get_semantic_views_description(), "")
  })
})

describe("discover_semantic_views_impl()", {
  it("propagates errors (not swallowed)", {
    skip_if_not_installed("RSQLite")

    # SQLite doesn't have SHOW SEMANTIC VIEWS, so it should error
    conn <- DBI::dbConnect(RSQLite::SQLite(), ":memory:")
    withr::defer(DBI::dbDisconnect(conn))

    # Without tryCatch wrapping, this should error (not return empty list)
    # The error is a syntax error since SQLite doesn't support SHOW command
    expect_error(
      discover_semantic_views_impl(conn),
      "SHOW"
    )
  })

  it("respects QUERYCHAT_DISABLE_SEMANTIC_VIEWS env var", {
    skip_if_not_installed("RSQLite")

    conn <- DBI::dbConnect(RSQLite::SQLite(), ":memory:")
    withr::defer(DBI::dbDisconnect(conn))

    withr::with_envvar(c("QUERYCHAT_DISABLE_SEMANTIC_VIEWS" = "1"), {
      # Should return empty list without querying (no error from SQLite)
      result <- discover_semantic_views_impl(conn)
      expect_equal(result, list())
    })
  })
})

# Generated by staticimports; do not edit by hand.
# ======================================================================
# Imported from pkg:staticimports
# ======================================================================

raw_to_utf8 <- function(data) {
  res <- rawToChar(data)
  Encoding(res) <- "UTF-8"
  res
}

read_raw <- function(file) {
  readBin(file, "raw", n = file.info(file, extra_cols = FALSE)$size)
}

# Read file as UTF-8
read_utf8 <- function(file) {
  res <- read_raw(file)
  raw_to_utf8(res)
}

tool_visualize_dashboard <- function(
  data_source,
  session,
  update_fn = function(data) {},
  has_tool_query = FALSE
) {
  check_data_source(data_source)
  check_function(update_fn)

  db_type <- data_source$get_db_type()

  ellmer::tool(
    tool_visualize_impl(data_source, session, update_fn),
    name = "querychat_visualize",
    description = render_viz_tool_description(
      db_type = db_type,
      has_tool_query = has_tool_query
    ),
    arguments = list(
      ggsql = ellmer::type_string(
        ellmer::interpolate(
          "A full ggsql query. Must include a VISUALISE clause and at least one DRAW clause. The SELECT portion uses {{db_type}} SQL; VISUALISE and MAPPING accept column names only, not expressions. Do NOT include `LABEL title => ...` in the query -- use the `title` parameter instead.",
          db_type = db_type
        )
      ),
      title = ellmer::type_string(
        "A brief, user-friendly title for this visualization. This is displayed as the card header above the chart."
      )
    ),
    annotations = ellmer::tool_annotations(
      title = "Query Visualization",
      icon = viz_icon()
    )
  )
}

tool_visualize_impl <- function(data_source, session, update_fn) {
  force(data_source)
  force(session)
  force(update_fn)

  function(ggsql, title) {
    visualize_result(data_source, session, update_fn, ggsql, title)
  }
}

random_hex <- function(n_bytes = 8) {
  paste0(
    format(as.hexmode(sample(256, n_bytes, replace = TRUE) - 1L), width = 2),
    collapse = ""
  )
}

visualize_result <- function(
  data_source,
  session,
  update_fn,
  ggsql_str,
  title
) {
  rlang::check_installed("ggsql", reason = "for visualization support.")

  validated <- ggsql::ggsql_validate(ggsql_str)
  has_visual <- ggsql::ggsql_has_visual(validated)

  if (!has_visual) {
    rlang::abort(
      "Query must include a VISUALISE clause. Use querychat_query for queries without visualization."
    )
  }

  if (!isTRUE(validated$valid)) {
    rlang::abort(collapse_validation_errors(validated))
  }

  spec <- execute_ggsql(data_source, validated)

  widget_id <- paste0("querychat_viz_", random_hex())

  viz_container <- NULL
  if (!is.null(session)) {
    session$output[[widget_id]] <- ggsql::renderGgsql(spec)
    viz_container <- htmltools::div(
      class = "querychat-viz-container",
      bslib::as_fill_carrier(),
      ggsql::ggsqlOutput(session$ns(widget_id)),
      viz_dep()
    )
  } else {
    # Non-Shiny usage: print the Spec to display via the interactive viewer
    print(spec)
    update_fn(list(ggsql = ggsql_str, title = title, widget_id = widget_id))
    return(ellmer::ContentToolResult(
      value = sprintf(
        "Chart displayed%s.",
        if (nzchar(title)) sprintf(" with title '%s'", title) else ""
      )
    ))
  }

  # PNG snapshot for LLM feedback (best-effort; requires V8 + rsvg)
  png_file <- tempfile(fileext = ".png")
  on.exit(unlink(png_file), add = TRUE)
  png_content <- tryCatch(
    {
      ggsql::ggsql_save(spec, png_file, width = 500, height = 300)
      ellmer::content_image_file(png_file)
    },
    error = function(e) {
      cli::cli_warn(c(
        "Unable to render PNG preview for the visualization card.",
        "i" = "The interactive chart will still render, but the LLM will not receive the static image preview for this visualization.",
        "i" = "PNG preview generation requires optional dependencies used by {.fn ggsql::ggsql_save}, typically {.pkg V8} and {.pkg rsvg}.",
        "x" = "Underlying error: {.msg {conditionMessage(e)}}"
      ))
      NULL
    }
  )

  title_display <- if (nzchar(title)) {
    sprintf(" with title '%s'", title)
  } else {
    ""
  }
  text <- sprintf("Chart displayed%s.", title_display)

  # All list elements must be Content S7 objects for ellmer's
  # expand_content_if_needed() to handle mixed text+image results.
  value <- if (!is.null(png_content)) {
    list(ellmer::ContentText(text), png_content)
  } else {
    text
  }

  update_fn(list(ggsql = ggsql_str, title = title, widget_id = widget_id))

  footer <- build_viz_footer(
    ggsql_str,
    title,
    widget_id,
    dom_widget_id = session$ns(widget_id)
  )
  extra <- list(
    display = list(
      html = viz_container,
      title = if (nzchar(title)) title else "Query Visualization",
      show_request = FALSE,
      open = querychat_tool_starts_open("visualize"),
      full_screen = TRUE,
      icon = viz_icon(),
      footer = footer
    )
  )

  ellmer::ContentToolResult(value = value, extra = extra)
}

collapse_validation_errors <- function(validated) {
  errors <- validated$errors
  if (is.null(errors) || !nrow(errors)) {
    return("Invalid ggsql query.")
  }

  messages <- errors$message
  messages <- messages[!is.na(messages) & nzchar(messages)]
  if (!length(messages)) {
    return("Invalid ggsql query.")
  }

  paste(messages, collapse = "\n")
}

build_viz_footer <- function(
  ggsql_str,
  title,
  widget_id,
  dom_widget_id
) {
  footer_id <- paste0("querychat_footer_", random_hex())
  query_section_id <- paste0(footer_id, "_query")
  code_editor_id <- paste0(footer_id, "_code")

  code_editor <- bslib::input_code_editor(
    id = code_editor_id,
    value = ggsql_str,
    language = "ggsql",
    read_only = TRUE,
    line_numbers = FALSE,
    height = "auto",
    theme_dark = "github-dark"
  )

  query_section <- shiny::tags$div(
    class = "querychat-query-section",
    id = query_section_id,
    code_editor
  )

  buttons_row <- shiny::tags$div(
    class = "querychat-footer-buttons",
    # Left: Show Query toggle
    shiny::tags$div(
      class = "querychat-footer-left",
      shiny::tags$button(
        class = "querychat-show-query-btn",
        `data-querychat-action` = "show-query",
        `data-target` = query_section_id,
        shiny::tags$span(class = "querychat-query-chevron", "\u25b6"),
        shiny::tags$span(class = "querychat-query-label", "Show Query")
      )
    ),
    # Right: Save dropdown
    shiny::tags$div(
      class = "querychat-footer-right",
      shiny::tags$div(
        class = "querychat-save-dropdown",
        shiny::tags$button(
          class = "querychat-save-btn",
          `data-querychat-action` = "save-toggle",
          `data-widget-id` = dom_widget_id,
          bsicons::bs_icon("download", class = "querychat-icon"),
          "Save",
          bsicons::bs_icon("chevron-down", class = "querychat-dropdown-chevron")
        ),
        shiny::tags$div(
          class = "querychat-save-menu",
          shiny::tags$button(
            class = "querychat-save-png-btn",
            `data-querychat-action` = "save-png",
            `data-widget-id` = dom_widget_id,
            `data-title` = title,
            "Save as PNG"
          ),
          shiny::tags$button(
            class = "querychat-save-svg-btn",
            `data-querychat-action` = "save-svg",
            `data-widget-id` = dom_widget_id,
            `data-title` = title,
            "Save as SVG"
          )
        )
      )
    )
  )

  htmltools::tagList(buttons_row, query_section)
}

viz_icon <- function() {
  '<svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" class="bi bi-graph-up" viewBox="0 0 16 16"><path fill-rule="evenodd" d="M0 0h1v15h15v1H0zm14.817 3.113a.5.5 0 0 1 .07.704l-4.5 5.5a.5.5 0 0 1-.74.037L7.06 6.767l-3.656 5.027a.5.5 0 0 1-.808-.588l4-5.5a.5.5 0 0 1 .758-.06l2.609 2.61 4.15-5.073a.5.5 0 0 1 .704-.07"/></svg>'
}

viz_dep <- function() {
  htmltools::htmlDependency(
    name = "querychat-viz",
    version = utils::packageVersion("querychat"),
    package = "querychat",
    src = "htmldep",
    stylesheet = "viz.css",
    script = "viz.js"
  )
}

render_viz_tool_description <- function(db_type, has_tool_query = FALSE) {
  path <- system.file("prompts", "tool-visualize.md", package = "querychat")
  stopifnot(nzchar(path), file.exists(path))
  template <- paste(readLines(path, warn = FALSE), collapse = "\n")
  whisker::whisker.render(
    template,
    list(
      db_type = db_type,
      has_tool_query = if (isTRUE(has_tool_query)) "true"
    )
  )
}

#' Execute a pre-validated ggsql query against a DataSource
#'
#' Executes the SQL portion through a DataSource (preserving database pushdown),
#' then feeds the result into a ggsql DuckDB reader to produce a Spec.
#'
#' @param data_source A querychat DataSource R6 object.
#' @param validated A pre-validated ggsql query (from `ggsql::ggsql_validate()`).
#'   Must be a list with `$sql` and `$visual` fields.
#'
#' @return A `ggsql::Spec` R6 object (the writer-independent plot specification).
#'
#' @keywords internal
execute_ggsql <- function(data_source, validated) {
  rlang::check_installed("ggsql", reason = "for visualization support.")

  visual <- validated$visual

  if (has_layer_level_source(visual)) {
    cli::cli_abort(
      "Layer-specific sources are not currently supported in querychat visual queries. Rewrite the query so that all layers come from the final SQL result."
    )
  }

  df <- data_source$execute_query(validated$sql)

  if (inherits(df, "tbl_sql")) {
    # Materialize the query for ggsql, {dplyr} guaranteed by TblSqlSource
    df <- dplyr::collect(df)
  }

  reader <- ggsql::duckdb_reader()
  table <- extract_visualise_table(visual)

  if (!is.null(table)) {
    # VISUALISE [mappings] FROM <table> — register data under the
    # referenced table name and execute the visual part directly.
    name <- if (startsWith(table, '"') && endsWith(table, '"')) {
      substr(table, 2, nchar(table) - 1)
    } else {
      table
    }
    ggsql::ggsql_register(reader, df, name)
    ggsql::ggsql_execute(reader, visual)
  } else {
    # SELECT ... VISUALISE — no FROM in VISUALISE clause, so register
    # under a synthetic name and prepend a SELECT.
    ggsql::ggsql_register(reader, df, "_data")
    ggsql::ggsql_execute(reader, paste("SELECT * FROM _data", visual))
  }
}

#' Extract the table name from a VISUALISE clause's FROM, if present
#'
#' Looks only in the portion of the visual string before the first DRAW keyword,
#' so FROM clauses inside DRAW (e.g., MAPPING x FROM other) are ignored.
#'
#' @param visual A ggsql VISUALISE string.
#' @return The table name string (possibly quoted), or `NULL` if not present.
#'
#' @keywords internal
extract_visualise_table <- function(visual) {
  draw_pos <- regexpr("\\bDRAW\\b", visual, ignore.case = TRUE, perl = TRUE)
  vis_clause <- if (draw_pos > 0) substr(visual, 1, draw_pos - 1L) else visual
  m <- regmatches(
    vis_clause,
    regexpr(
      '\\bFROM\\s+("[^"]+?"|\\S+)',
      vis_clause,
      ignore.case = TRUE,
      perl = TRUE
    )
  )
  if (length(m) == 0 || !nzchar(m)) {
    return(NULL)
  }
  sub("^(?i)FROM\\s+", "", m, perl = TRUE)
}

#' Detect whether a VISUALISE string has a layer-level FROM source
#'
#' Returns `TRUE` when a DRAW clause defines its own `FROM <source>` via a
#' MAPPING sub-clause. Querychat replays VISUALISE against a single local
#' relation, so layer-specific sources cannot be preserved reliably.
#'
#' @param visual A ggsql VISUALISE string.
#' @return `TRUE` if any DRAW clause contains a MAPPING ... FROM source.
#'
#' @keywords internal
has_layer_level_source <- function(visual) {
  # Split at clause boundaries (DRAW, SCALE, etc.) using a lookbehind for
  # whitespace rather than \b, which can split mid-word in R's PCRE engine.
  clauses <- strsplit(
    visual,
    "(?i)(?<=\\s)(?=DRAW|SCALE|PROJECT|FACET|PLACE|LABEL|THEME)",
    perl = TRUE
  )[[1]]
  for (clause in clauses) {
    if (!grepl("^\\s*DRAW\\b", clause, ignore.case = TRUE, perl = TRUE)) {
      next
    }
    if (
      grepl(
        "\\bMAPPING\\b[\\s\\S]*?\\bFROM\\s+(\"[^\"]+?\"|\\S+)",
        clause,
        ignore.case = TRUE,
        perl = TRUE
      )
    ) {
      return(TRUE)
    }
  }
  FALSE
}

describe("tool_visualize_dashboard()", {
  skip_if_no_dataframe_engine()
  skip_if_not_installed("ggsql")

  it("creates a tool with correct name", {
    ds <- local_data_frame_source(new_test_df())
    session <- structure(
      list(
        output = list(),
        ns = identity
      ),
      class = "MockShinySession"
    )
    tool <- tool_visualize_dashboard(
      ds,
      session = session,
      update_fn = function(data) {}
    )
    expect_equal(tool@name, "querychat_visualize")
  })

  it("renders query fallback guidance only when query tool is enabled", {
    ds <- local_data_frame_source(new_test_df())
    session <- structure(
      list(
        output = list(),
        ns = identity
      ),
      class = "MockShinySession"
    )

    with_query <- tool_visualize_dashboard(
      ds,
      session = session,
      update_fn = function(data) {},
      has_tool_query = TRUE
    )
    without_query <- tool_visualize_dashboard(
      ds,
      session = session,
      update_fn = function(data) {},
      has_tool_query = FALSE
    )

    expect_match(
      with_query@description,
      "use `querychat_query` instead",
      fixed = TRUE
    )
    expect_no_match(
      without_query@description,
      "use `querychat_query` instead",
      fixed = TRUE
    )
    expect_no_match(
      without_query@description,
      "fall back to `querychat_query`",
      fixed = TRUE
    )
  })

  it("describes current ggsql 0.3 visualization rules", {
    ds <- local_data_frame_source(new_test_df())
    session <- structure(
      list(
        output = list(),
        ns = identity
      ),
      class = "MockShinySession"
    )
    tool <- tool_visualize_dashboard(
      ds,
      session = session,
      update_fn = function(data) {}
    )

    expect_match(
      tool@description,
      "All data transformations must happen in the `SELECT` clause.",
      fixed = TRUE
    )
    expect_match(tool@description, "DRAW range", fixed = TRUE)
    expect_no_match(
      tool@description,
      "using single quotes for strings",
      fixed = TRUE
    )
  })

  it("prints the spec and returns a simple result when session is NULL", {
    ds <- local_data_frame_source(new_test_df())

    tool <- tool_visualize_dashboard(
      ds,
      session = NULL,
      update_fn = function(data) {}
    )

    result <- tool(
      ggsql = "SELECT * FROM test_table VISUALISE value AS x DRAW histogram",
      title = "Test"
    )

    expect_match(
      result@value,
      "Chart displayed with title 'Test'.",
      fixed = TRUE
    )
    expect_equal(result@extra, list())
  })

  it("calls update_fn on success", {
    ds <- local_data_frame_source(new_test_df())
    session <- structure(
      list(
        output = list(),
        ns = identity
      ),
      class = "MockShinySession"
    )
    callback_data <- NULL
    footer_data <- NULL
    local_mocked_bindings(
      build_viz_footer = function(...) {
        footer_data <<- list(...)
        htmltools::tagList()
      },
      .package = "querychat"
    )
    tool <- tool_visualize_dashboard(
      ds,
      session = session,
      update_fn = function(data) {
        callback_data <<- data
      }
    )
    tool(
      ggsql = "SELECT * FROM test_table VISUALISE value AS x DRAW histogram",
      title = "Test"
    )
    expect_type(callback_data, "list")
    expect_true(all(c("ggsql", "title", "widget_id") %in% names(callback_data)))
    expect_identical(footer_data$dom_widget_id, footer_data[[3]])
  })

  it("warns when PNG preview generation fails", {
    ds <- local_data_frame_source(new_test_df())
    session <- structure(
      list(
        output = list(),
        ns = identity
      ),
      class = "MockShinySession"
    )

    local_mocked_bindings(
      execute_ggsql = function(...) structure(list(), class = "ggsql_spec"),
      build_viz_footer = function(...) htmltools::tagList(),
      .package = "querychat"
    )
    local_mocked_bindings(
      ggsql_validate = function(...) {
        structure(list(valid = TRUE), class = "ggsql_validated")
      },
      ggsql_has_visual = function(...) TRUE,
      renderGgsql = function(...) shiny::renderText("ok"),
      ggsqlOutput = function(id) htmltools::div(id = id),
      ggsql_save = function(...) rlang::abort("V8 is not installed"),
      .package = "ggsql"
    )

    tool <- tool_visualize_dashboard(
      ds,
      session = session,
      update_fn = function(data) {}
    )

    expect_warning(
      result <- tool(
        ggsql = "SELECT * FROM test_table VISUALISE value AS x DRAW histogram",
        title = "Test"
      ),
      "Unable to render PNG preview",
      fixed = TRUE
    )
    expect_match(
      result@value,
      "Chart displayed with title 'Test'.",
      fixed = TRUE
    )
  })

  it("raises an error for query without VISUALISE", {
    ds <- local_data_frame_source(new_test_df())
    session <- structure(
      list(
        output = list(),
        ns = identity
      ),
      class = "MockShinySession"
    )
    tool <- tool_visualize_dashboard(
      ds,
      session = session,
      update_fn = function(data) {}
    )
    expect_error(
      tool(
        ggsql = "SELECT * FROM test_table",
        title = "No viz"
      ),
      "Query must include a VISUALISE clause"
    )
  })

  it("raises an error for bad query", {
    ds <- local_data_frame_source(new_test_df())
    session <- structure(
      list(
        output = list(),
        ns = identity
      ),
      class = "MockShinySession"
    )
    tool <- tool_visualize_dashboard(
      ds,
      session = session,
      update_fn = function(data) {}
    )
    expect_error(
      tool(
        ggsql = paste(
          "SELECT value AS x, 1 AS y FROM nonexistent_table",
          "VISUALISE x AS x, y AS y DRAW point"
        ),
        title = "Bad"
      ),
      "Catalog Error"
    )
  })

  it("uses ggsql validation errors and does not execute invalid queries", {
    ds <- local_data_frame_source(new_test_df())
    session <- structure(
      list(
        output = list(),
        ns = identity
      ),
      class = "MockShinySession"
    )
    validated <- structure(
      list(
        valid = FALSE,
        errors = data.frame(
          message = c("first ggsql error", "second ggsql error")
        )
      ),
      class = "ggsql_validated"
    )
    executed <- FALSE

    local_mocked_bindings(
      ggsql_validate = function(...) validated,
      ggsql_has_visual = function(...) TRUE,
      .package = "ggsql"
    )
    local_mocked_bindings(
      execute_ggsql = function(...) {
        executed <<- TRUE
        rlang::abort("execute_ggsql should not be called")
      },
      .package = "querychat"
    )

    tool <- tool_visualize_dashboard(
      ds,
      session = session,
      update_fn = function(data) {}
    )

    expect_error(
      tool(
        ggsql = "SELECT * FROM test_table VISUALISE value AS x DRAW point",
        title = "Bad Viz"
      ),
      "first ggsql error\nsecond ggsql error",
      fixed = TRUE
    )
    expect_false(executed)
  })

  it("does not fall back to stale QueryChat VISUALISE guidance for invalid ggsql", {
    ds <- local_data_frame_source(new_test_df())
    session <- structure(
      list(
        output = list(),
        ns = identity
      ),
      class = "MockShinySession"
    )
    validated <- structure(
      list(
        valid = FALSE,
        errors = data.frame(message = "upstream validation failed")
      ),
      class = "ggsql_validated"
    )

    local_mocked_bindings(
      ggsql_validate = function(...) validated,
      ggsql_has_visual = function(...) TRUE,
      .package = "ggsql"
    )

    tool <- tool_visualize_dashboard(
      ds,
      session = session,
      update_fn = function(data) {}
    )

    err <- tryCatch(
      {
        tool(
          ggsql = "SELECT * FROM test_table VISUALISE value AS x DRAW point",
          title = "Bad Viz"
        )
        NULL
      },
      error = identity
    )

    expect_s3_class(err, "error")
    expect_match(
      conditionMessage(err),
      "upstream validation failed",
      fixed = TRUE
    )
    expect_no_match(
      conditionMessage(err),
      "VISUALISE clause was not recognized",
      fixed = TRUE
    )
  })

  it("passes the namespaced DOM id into the footer", {
    ds <- local_data_frame_source(new_test_df())
    session <- structure(
      list(
        output = list(),
        ns = function(id) paste0("repro-", id)
      ),
      class = "MockShinySession"
    )
    footer_data <- NULL
    local_mocked_bindings(
      build_viz_footer = function(...) {
        footer_data <<- list(...)
        htmltools::tagList()
      },
      .package = "querychat"
    )
    tool <- tool_visualize_dashboard(
      ds,
      session = session,
      update_fn = function(data) {}
    )

    tool(
      ggsql = "SELECT * FROM test_table VISUALISE value AS x DRAW histogram",
      title = "Test"
    )

    expect_identical(
      footer_data$dom_widget_id,
      paste0("repro-", footer_data[[3]])
    )
  })

  it("lets visualize_result() errors bubble up unchanged", {
    ds <- local_data_frame_source(new_test_df())
    long_msg <- paste(rep("word", 200), collapse = " ")
    session <- structure(
      list(
        output = list(),
        ns = identity
      ),
      class = "MockShinySession"
    )

    local_mocked_bindings(
      visualize_result = function(...) {
        rlang::abort(long_msg)
      },
      .package = "querychat"
    )

    impl <- tool_visualize_impl(
      ds,
      session = session,
      update_fn = function(data) {}
    )
    expect_error(
      impl(
        ggsql = "SELECT * FROM test_table VISUALISE value AS x DRAW histogram",
        title = "Bad"
      ),
      long_msg,
      fixed = TRUE
    )
  })
})

describe("collapse_validation_errors()", {
  it("joins multiple upstream validation messages", {
    validated <- structure(
      list(
        errors = data.frame(
          message = c("first ggsql error", "second ggsql error")
        )
      ),
      class = "ggsql_validated"
    )

    expect_equal(
      collapse_validation_errors(validated),
      "first ggsql error\nsecond ggsql error"
    )
  })

  it("falls back when no validation messages are available", {
    validated <- structure(
      list(
        errors = data.frame(message = character())
      ),
      class = "ggsql_validated"
    )

    expect_equal(
      collapse_validation_errors(validated),
      "Invalid ggsql query."
    )
  })
})

#' Deprecated functions
#'
#' These functions have been replaced by the new `QueryChat` R6 class API.
#' Please update your code to use the new class-based approach.
#'
#' @return Please see the updated function details for return values.
#'
#' @name deprecated
#' @keywords internal
NULL

#' @describeIn deprecated was replaced with the `$new()` method of [QueryChat].
#' @export
querychat_init <- function(...) {
  lifecycle::deprecate_stop(
    when = "0.1.0",
    what = "querychat_init()",
    with = "QueryChat$new()",
    details = c(
      "Old code:",
      "  config <- querychat_init(mtcars, greeting = 'Hello!')",
      "  ui <- page_sidebar(sidebar = querychat_sidebar('chat'), ...)",
      "  server <- function(input, output, session) {",
      "    chat <- querychat_server('chat', config)",
      "    output$data <- renderDataTable(chat$df())",
      "  }",
      "",
      "New code:",
      "  qc <- QueryChat$new(mtcars, 'mtcars', greeting = 'Hello!')",
      "  ui <- page_sidebar(sidebar = qc$sidebar(), ...)",
      "  server <- function(input, output, session) {",
      "    qc$server()",
      "    output$data <- renderDataTable(qc$df())",
      "  }",
      "",
      "See ?QueryChat for more information."
    )
  )
}

#' @describeIn deprecated was replaced with the `$sidebar()` method of
#'   [QueryChat].
#' @export
querychat_sidebar <- function(...) {
  lifecycle::deprecate_stop(
    when = "0.1.0",
    what = "querychat_sidebar()",
    with = "QueryChat$sidebar()",
    details = c(
      "Old code:",
      "  querychat_sidebar('chat')",
      "",
      "New code:",
      "  qc <- QueryChat$new(data, 'table_name')",
      "  qc$sidebar()",
      "",
      "See ?QueryChat for more information."
    )
  )
}

#' @describeIn deprecated was replaced with the `$ui()` method of [QueryChat].
#' @export
querychat_ui <- function(...) {
  lifecycle::deprecate_stop(
    when = "0.1.0",
    what = "querychat_ui()",
    with = "QueryChat$ui()",
    details = c(
      "Old code:",
      "  querychat_ui('chat')",
      "",
      "New code:",
      "  qc <- QueryChat$new(data, 'table_name')",
      "  qc$ui()",
      "",
      "See ?QueryChat for more information."
    )
  )
}

#' @describeIn deprecated was replaced with the `$server()` method of
#'   [QueryChat].
#' @export
querychat_server <- function(...) {
  lifecycle::deprecate_stop(
    when = "0.1.0",
    what = "querychat_server()",
    with = "QueryChat$server()",
    details = c(
      "Old code:",
      "  chat <- querychat_server('chat', config)",
      "  output$data <- renderDataTable(chat$df())",
      "",
      "New code:",
      "  qc <- QueryChat$new(data, 'table_name')",
      "  qc$server()  # Must be called within server function",
      "  output$data <- renderDataTable(qc$df())",
      "",
      "See ?QueryChat for more information."
    )
  )
}

#' @describeIn deprecated was replaced with the `$generate_greeting()` method of
#'   [QueryChat].
#' @export
querychat_greeting <- function(...) {
  lifecycle::deprecate_stop(
    when = "0.1.0",
    what = "querychat_greeting()",
    with = "QueryChat$generate_greeting()",
    details = c(
      "Old code:",
      "  greeting <- querychat_greeting(config)",
      "",
      "New code:",
      "  qc <- QueryChat$new(data, 'table_name')",
      "  greeting <- qc$generate_greeting(echo = 'text')",
      "",
      "See ?QueryChat for more information."
    )
  )
}

#' @describeIn deprecated was replaced with the `$new()` method of [QueryChat].
#' @export
querychat_data_source <- function(...) {
  lifecycle::deprecate_stop(
    when = "0.1.0",
    what = "querychat_data_source()",
    with = "QueryChat$new()",
    details = c(
      "Old code:",
      "  data_source <- querychat_data_source(db_connection, 'table_name')",
      "",
      "New code:",
      "  qc <- QueryChat$new(db_connection, 'table_name')",
      "",
      "See ?QueryChat for more information."
    )
  )
}

test_that("deprecated functions throw deprecation warnings", {
  lifecycle::expect_defunct(querychat_init())
  lifecycle::expect_defunct(querychat_sidebar())
  lifecycle::expect_defunct(querychat_ui())
  lifecycle::expect_defunct(querychat_server())
  lifecycle::expect_defunct(querychat_greeting())
  lifecycle::expect_defunct(querychat_data_source())
})

#' System Prompt Management for QueryChat
#'
#' @description
#' `QueryChatSystemPrompt` is an R6 class that encapsulates system prompt
#' template and component management for QueryChat. It handles loading of
#' prompt templates, data descriptions, and extra instructions from files or
#' strings, and renders them with tool configuration using Mustache templates.
#'
#' @noRd
QueryChatSystemPrompt <- R6::R6Class(
  "QueryChatSystemPrompt",

  public = list(
    #' @field template The Mustache template string for the system prompt, or
    #'   a path to a file containing a system prompt template.
    template = NULL,

    #' @field data_description Optional description of the data context, or a
    #'   path to a file containing the data description.
    data_description = NULL,

    #' @field extra_instructions Optional custom instructions for the LLM, or a
    #'   path to a file containing the extra instructions.
    extra_instructions = NULL,

    #' @field schema The database schema information.
    schema = NULL,

    #' @field categorical_threshold Threshold for categorical column detection.
    categorical_threshold = NULL,

    #' @field data_source Reference to the data source object.
    data_source = NULL,

    #' @description
    #' Create a new QueryChatSystemPrompt object.
    #'
    #' @param prompt_template Path to template file or template string.
    #' @param data_source A DataSource object for schema access.
    #' @param data_description Optional path to data description file or description string.
    #' @param extra_instructions Optional path to instructions file or instructions string.
    #' @param categorical_threshold Threshold for categorical column detection (default: 10).
    #'
    #' @return A new `QueryChatSystemPrompt` object.
    initialize = function(
      prompt_template,
      data_source,
      data_description = NULL,
      extra_instructions = NULL,
      categorical_threshold = 10
    ) {
      # Load template (file or string) using helper
      self$template <- read_text(prompt_template)

      # Load data_description (file, string, or NULL)
      if (!is.null(data_description)) {
        self$data_description <- read_text(data_description)
      }

      # Load extra_instructions (file, string, or NULL)
      if (!is.null(extra_instructions)) {
        self$extra_instructions <- read_text(extra_instructions)
      }

      # Only compute schema if the template references it
      if (grepl("\\{\\{[{#^/]?\\s*schema\\b", self$template)) {
        self$schema <- data_source$get_schema(
          categorical_threshold = categorical_threshold
        )
      } else {
        self$schema <- ""
      }
      self$categorical_threshold <- categorical_threshold
      self$data_source <- data_source
    },

    #' @description
    #' Render the system prompt with tool configuration.
    #'
    #' @param tools Character vector of tool names to enable (e.g.,
    #'   `c("query", "update"))`, or `NULL` for no tools.
    #'
    #' @return A character string containing the rendered system prompt.
    render = function(tools) {
      # Build context for whisker rendering
      db_type <- self$data_source$get_db_type()
      is_duck_db <- tolower(db_type) == "duckdb"

      # Get semantic views description (available with DBISource for Snowflake)
      semantic_views <- ""
      if (inherits(self$data_source, "DBISource")) {
        semantic_views <- self$data_source$get_semantic_views_description()
      }

      context <- list(
        db_type = db_type,
        is_duck_db = is_duck_db,
        semantic_views = semantic_views,
        schema = self$schema,
        data_description = self$data_description,
        extra_instructions = self$extra_instructions,
        has_tool_update = if ("update" %in% tools) "true",
        has_tool_query = if ("query" %in% tools) "true",
        has_tool_visualize = if ("visualize" %in% tools) "true",
        include_query_guidelines = if (length(tools) > 0) "true"
      )

      partials <- list()
      syntax_path <- system.file(
        "prompts",
        "ggsql-syntax.md",
        package = "querychat"
      )
      if (nzchar(syntax_path)) {
        partials[["ggsql-syntax"]] <- paste(
          readLines(syntax_path),
          collapse = "\n"
        )
      }

      whisker::whisker.render(self$template, context, partials = partials)
    }
  )
)

# Utility function for loading file or string content
read_text <- function(x) {
  if (file.exists(x)) {
    read_utf8(x)
  } else {
    paste(x, collapse = "\n")
  }
}

#' Data Source: SQL Tibble
#'
#' @description
#' A DataSource implementation for lazy SQL tibbles connected to databases via
#' [dbplyr::tbl_sql()] or [dplyr::sql()].
#'
#' @examplesIf rlang::is_installed("dbplyr") && rlang::is_installed("dplyr") && rlang::is_installed("duckdb")
#' con <- DBI::dbConnect(duckdb::duckdb())
#' DBI::dbWriteTable(con, "mtcars", mtcars)
#'
#' mtcars_source <- TblSqlSource$new(dplyr::tbl(con, "mtcars"))
#' mtcars_source$get_db_type()  # "DuckDB"
#'
#' result <- mtcars_source$execute_query("SELECT * FROM mtcars WHERE cyl > 4")
#'
#' # Note, the result is not the *full* data frame, but a lazy SQL tibble
#' result
#'
#' # You can chain this result into a dplyr pipeline
#' dplyr::count(result, cyl, gear)
#'
#' # Or collect the entire data frame into local memory
#' dplyr::collect(result)
#'
#' # Finally, clean up when done with the database (closes the DB connection)
#' mtcars_source$cleanup()
#'
#' @export
TblSqlSource <- R6::R6Class(
  "TblSqlSource",
  inherit = DBISource,
  private = list(
    tbl = NULL,
    tbl_cte = NULL
  ),
  public = list(
    #' @field table_name Name of the table to be used in SQL queries
    table_name = NULL,

    #' @description
    #' Create a new TblSqlSource
    #'
    #' @param tbl A [dbplyr::tbl_sql()] (or SQL tibble via [dplyr::tbl()]).
    #' @param table_name Name of the table in the database. Can be a character
    #'   string, or will be inferred from the `tbl` argument, if possible.
    #'
    #' @return A new TblSqlSource object
    initialize = function(tbl, table_name = missing_arg()) {
      check_installed("dbplyr")
      check_installed("dplyr")

      if (!inherits(tbl, "tbl_sql")) {
        cli::cli_abort(
          "{.arg tbl} must be a SQL tibble connected to a database, not {.obj_type_friendly {tbl}}"
        )
      }

      private$conn <- dbplyr::remote_con(tbl)
      private$tbl <- tbl

      # Collect various signals to infer the table name
      obj_name <- deparse1(substitute(tbl))

      # Get the exact table name, if tbl directly references a single table
      remote_name <- dbplyr::remote_name(private$tbl)

      use_cte <- FALSE

      if (!is_missing(table_name)) {
        check_sql_table_name(table_name)
        self$table_name <- table_name
        use_cte <- !identical(table_name, remote_name)
      } else if (!is.null(remote_name)) {
        # Remote name is non-NULL when it points to a table, so we use that next
        self$table_name <- remote_name
        use_cte <- FALSE
      } else if (is_valid_sql_table_name(obj_name)) {
        self$table_name <- obj_name
        use_cte <- TRUE
      } else {
        id <- as.integer(runif(1) * 1e6)
        self$table_name <- sprintf("querychat_cte_%d", id)
        use_cte <- TRUE
      }

      if (use_cte) {
        # We received a complicated tbl expression, we'll have to use a CTE
        private$tbl_cte <- dbplyr::remote_query(private$tbl)
      }
    },

    #' @description
    #' Get the database type
    #'
    #' @return A string describing the database type (e.g., "DuckDB", "SQLite")
    get_db_type = function() {
      super$get_db_type()
    },

    #' @description
    #' Get schema information about the table
    #'
    #' @param categorical_threshold Maximum number of unique values for a text
    #'   column to be considered categorical
    #' @return A string containing schema information formatted for LLM prompts
    get_schema = function(categorical_threshold = 20) {
      get_schema_impl(
        private$conn,
        self$table_name,
        categorical_threshold,
        columns = colnames(private$tbl),
        prep_query = self$prep_query
      )
    },

    #' @description
    #' Execute a SQL query and return results
    #'
    #' @param query SQL query string to execute
    #' @return A data frame containing query results
    execute_query = function(query) {
      sql_query <- self$prep_query(query)
      dplyr::tbl(private$conn, dplyr::sql(sql_query))
    },

    #' @description
    #' Test a SQL query by fetching only one row
    #'
    #' @param query SQL query string to test
    #' @param require_all_columns If `TRUE`, validates that the result includes
    #'   all original table columns (default: `FALSE`)
    #' @return A data frame containing one row of results (or empty if no matches)
    test_query = function(query, require_all_columns = FALSE) {
      super$test_query(
        query = self$prep_query(query),
        require_all_columns = require_all_columns
      )
    },

    #' @description
    #' Prepare a generic `SELECT * FROM ____` query to work with the SQL tibble
    #'
    #' @param query SQL query as a string
    #' @return A complete SQL query string
    prep_query = function(query) {
      check_string(query)

      if (is.null(private$tbl_cte)) {
        return(query)
      }

      sprintf(
        "WITH %s AS (\n%s\n)\n%s",
        DBI::dbQuoteIdentifier(private$conn, self$table_name),
        private$tbl_cte,
        query
      )
    },

    #' @description
    #' Get the unfiltered data as a SQL tibble
    #'
    #' @return A [dbplyr::tbl_sql()] containing the original, unfiltered data
    get_data = function() {
      private$tbl
    },

    #' @description
    #' Clean up resources (close connections, etc.)
    #'
    #' @return NULL (invisibly)
    cleanup = function() {
      super$cleanup()
    }
  )
)

check_data_source <- function(
  x,
  ...,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!inherits(x, "DataSource")) {
    cli::cli_abort(
      "{.arg {arg}} must be a {.cls DataSource} object, not {.obj_type_friendly {x}}.",
      call = call
    )
  }
}


# SQL table name validation ----------------------------------------------

#' Check SQL table name validity
#'
#' Validates that a string is a valid SQL table name. A valid SQL table name
#' must begin with a letter and contain only letters, numbers, and underscores.
#'
#' @param x The value to check
#' @param ... These dots are for future extensions and must be empty.
#' @param allow_null Logical. If `TRUE`, `NULL` is accepted as a valid value.
#' @param arg Argument name to use in error messages
#' @param call Calling environment for error messages
#'
#' @return Invisibly returns `NULL` if validation passes. Otherwise throws an error.
#' @noRd
check_sql_table_name <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  check_dots_empty()

  # Check if NULL is allowed
  if (allow_null && is.null(x)) {
    return(invisible(NULL))
  }

  # First check it's a string
  check_string(x, allow_null = allow_null, arg = arg, call = call)

  # Then validate SQL table name pattern
  if (!is_valid_sql_table_name(x)) {
    cli::cli_abort(
      c(
        "{.arg {arg}} must be a valid SQL table name",
        "i" = "Table names must begin with a letter and contain only letters, numbers, and underscores",
        "x" = "You provided: {.val {x}}"
      ),
      call = call
    )
  }

  invisible(NULL)
}

is_valid_sql_table_name <- function(x) {
  grepl("^[a-zA-Z][a-zA-Z0-9_]*$", x)
}


# SQL query validation --------------------------------------------------------

#' Check SQL query for disallowed operations
#'
#' Validates that a SQL query does not start with a disallowed operation.
#' This is a simple safety check, not full query parsing.
#'
#' @param query The SQL query string to check
#' @param ... These dots are for future extensions and must be empty.
#' @param arg Argument name to use in error messages
#' @param call Calling environment for error messages
#'
#' @details
#' Two categories of keywords are checked:
#'
#' **Always blocked** (no escape hatch):
#' DELETE, TRUNCATE, CREATE, DROP, ALTER, GRANT, REVOKE, EXEC, EXECUTE, CALL
#'
#' **Blocked unless escape hatch enabled**:
#' INSERT, UPDATE, MERGE, REPLACE, UPSERT
#'
#' The escape hatch can be enabled via
#' `options(querychat.enable_update_queries = TRUE)` or by setting the
#' environment variable `QUERYCHAT_ENABLE_UPDATE_QUERIES=true`.
#'
#' @return Invisibly returns `NULL` if validation passes. Otherwise throws an
#'   error.
#'
#' @noRd
check_query <- function(
  query,
  ...,
  arg = caller_arg(query),
  call = caller_env()
) {
  check_dots_empty()
  check_string(query, arg = arg, call = call)

  # Normalize: newlines/tabs -> space, collapse multiple spaces, trim, uppercase
  normalized <- query
  normalized <- gsub("[\r\n\t]+", " ", normalized)
  normalized <- gsub(" +", " ", normalized)
  normalized <- trimws(normalized)
  normalized <- toupper(normalized)

  # Always blocked - destructive/schema/admin operations
  always_blocked <- c(
    "DELETE",
    "TRUNCATE",
    "CREATE",
    "DROP",
    "ALTER",
    "GRANT",
    "REVOKE",
    "EXEC",
    "EXECUTE",
    "CALL"
  )

  # Blocked unless escape hatch enabled - data modification
  update_keywords <- c("INSERT", "UPDATE", "MERGE", "REPLACE", "UPSERT")

  # Check always-blocked keywords first
  always_pattern <- paste0("^(", paste(always_blocked, collapse = "|"), ")\\b")
  if (grepl(always_pattern, normalized)) {
    matched <- regmatches(normalized, regexpr(always_pattern, normalized))
    cli::cli_abort(
      c(
        "Query appears to contain a disallowed operation: {matched}",
        "i" = "Only SELECT queries are allowed."
      ),
      call = call
    )
  }

  # Check update keywords (can be enabled via option or envvar)
  enable_updates <- isTRUE(getOption("querychat.enable_update_queries", FALSE))
  if (!enable_updates) {
    envvar <- Sys.getenv("QUERYCHAT_ENABLE_UPDATE_QUERIES", "")
    enable_updates <- tolower(envvar) %in% c("true", "1", "yes")
  }

  if (!enable_updates) {
    update_pattern <- paste0(
      "^(",
      paste(update_keywords, collapse = "|"),
      ")\\b"
    )
    if (grepl(update_pattern, normalized)) {
      matched <- regmatches(normalized, regexpr(update_pattern, normalized))
      cli::cli_abort(
        c(
          "Query appears to contain an update operation: {matched}",
          "i" = "Only SELECT queries are allowed.",
          "i" = "Set {.code options(querychat.enable_update_queries = TRUE)} or {.envvar QUERYCHAT_ENABLE_UPDATE_QUERIES=true} to allow update queries."
        ),
        call = call
      )
    }
  }

  invisible(NULL)
}

#' QueryChat: Interactive Data Querying with Natural Language
#'
#' @description
#' `QueryChat` is an R6 class built on Shiny, shinychat, and ellmer to enable
#' interactive querying of data using natural language. It leverages large
#' language models (LLMs) to translate user questions into SQL queries, execute
#' them against a data source (data frame or database), and various ways of
#' accessing/displaying the results.
#'
#' The `QueryChat` class takes your data (a data frame or database connection)
#' as input and provides methods to:
#'
#' - Generate a chat UI for natural language queries (e.g., `$app()`,
#'   `$sidebar()`)
#' - Initialize server logic that returns session-specific reactive values (via
#'   `$server()`)
#' - Access reactive data, SQL queries, and titles through the returned server
#'   values
#'
#' @section Usage in Shiny Apps:
#' ```r
#' library(querychat)
#'
#' # Create a QueryChat object
#' qc <- QueryChat$new(mtcars)
#'
#' # Quick start: run a complete app
#' qc$app()
#'
#' # Or build a custom Shiny app
#' ui <- page_sidebar(
#'   qc$sidebar(),
#'   verbatimTextOutput("sql"),
#'   dataTableOutput("data")
#' )
#'
#' server <- function(input, output, session) {
#'   qc_vals <- qc$server()
#'
#'   output$sql <- renderText(qc_vals$sql())
#'   output$data <- renderDataTable(qc_vals$df())
#' }
#'
#' shinyApp(ui, server)
#' ```
#'
#' @examplesIf rlang::is_installed("duckdb") || rlang::is_installed("RSQLite")
#' # Basic usage with a data frame
#' qc <- QueryChat$new(mtcars)
#' \dontrun{
#' app <- qc$app()
#' }
#'
#' # With a custom greeting
#' greeting <- "Welcome! Ask me about the mtcars dataset."
#' qc <- QueryChat$new(mtcars, greeting = greeting)
#'
#' # With a specific LLM provider
#' qc <- QueryChat$new(mtcars, client = "anthropic/claude-sonnet-4-5")
#'
#' # Generate a greeting for reuse (requires internet/API access)
#' \dontrun{
#' qc <- QueryChat$new(mtcars)
#' greeting <- qc$generate_greeting(echo = "text")
#' # Save greeting for next time
#' writeLines(greeting, "mtcars_greeting.md")
#' }
#'
#' # Or specify greeting and additional options at initialization
#' qc <- QueryChat$new(
#'   mtcars,
#'   greeting = "Welcome to the mtcars explorer!",
#'   client = "openai/gpt-4o",
#'   data_description = "Motor Trend car road tests dataset"
#' )
#'
#' @examplesIf rlang::is_installed("RSQLite")
#' # Create a QueryChat object from a database connection
#' # 1. Set up the database connection
#' con <- DBI::dbConnect(RSQLite::SQLite(), ":memory:")
#'
#' # 2. (For this demo) Create a table in the database
#' DBI::dbWriteTable(con, "mtcars", mtcars)
#'
#' # 3. Pass the connection and table name to `QueryChat`
#' qc <- QueryChat$new(con, "mtcars")
#'
#' @export
QueryChat <- R6::R6Class(
  "QueryChat",
  private = list(
    server_values = NULL,
    .data_source = NULL,
    .table_name = NULL,
    .client_spec = NULL,
    .client_console = NULL,
    .system_prompt = NULL,
    # Store init parameters for deferred system prompt building
    .prompt_template = NULL,
    .data_description = NULL,
    .extra_instructions = NULL,
    .categorical_threshold = NULL,

    require_data_source = function(method_name) {
      if (is.null(private$.data_source)) {
        cli::cli_abort(
          "{.arg data_source} must be set before calling {.fn ${method_name}}.
           Either pass {.arg data_source} to {.fn $new}, set the
           {.field $data_source} property, or pass {.arg data_source} to {.fn $server}."
        )
      }
    },

    build_system_prompt = function() {
      if (is.null(private$.data_source)) {
        cli::cli_abort("Cannot build system prompt without data_source")
      }

      prompt_template <- private$.prompt_template
      if (is.null(prompt_template)) {
        prompt_template <- system.file(
          "prompts",
          "prompt.md",
          package = "querychat"
        )
      }

      private$.system_prompt <- QueryChatSystemPrompt$new(
        prompt_template = prompt_template,
        data_source = private$.data_source,
        data_description = private$.data_description,
        extra_instructions = private$.extra_instructions,
        categorical_threshold = private$.categorical_threshold
      )
    },

    create_session_client = function(
      client_spec = NULL,
      tools = NA,
      session = NULL,
      update_dashboard = function(query, title) {},
      reset_dashboard = function() {},
      visualize = function(data) {}
    ) {
      spec <- client_spec %||% private$.client_spec
      chat <- as_querychat_client(spec)
      chat <- chat$clone()
      chat$set_turns(list())

      if (is_na(tools)) {
        tools <- self$tools
      }

      chat$set_system_prompt(private$.system_prompt$render(tools = tools))

      if (is.null(tools)) {
        return(chat)
      }

      if ("update" %in% tools) {
        chat$register_tool(
          tool_update_dashboard(
            private$.data_source,
            update_fn = update_dashboard
          )
        )
        chat$register_tool(tool_reset_dashboard(reset_dashboard))
      }

      if ("query" %in% tools) {
        chat$register_tool(tool_query(private$.data_source))
      }

      if ("visualize" %in% tools) {
        rlang::check_installed(
          "ggsql",
          reason = "for visualization support."
        )
        chat$register_tool(
          tool_visualize_dashboard(
            private$.data_source,
            session = session,
            update_fn = visualize,
            has_tool_query = "query" %in% tools
          )
        )
      }

      chat
    }
  ),
  public = list(
    #' @field greeting The greeting message displayed to users.
    greeting = NULL,
    #' @field id ID for the QueryChat instance.
    id = NULL,
    #' @field tools The allowed tools for the chat client.
    tools = c("filter", "query"),

    #' @description
    #' Create a new QueryChat object.
    #'
    #' @param data_source Either a data.frame, a database connection (e.g., DBI
    #'   connection), or `NULL` to defer setting the data source until later.
    #'   When `NULL`, the data source must be set via the `$data_source` property
    #'   or passed to `$server()` before calling methods that require data access.
    #' @param table_name A string specifying the table name to use in SQL
    #'   queries. If `data_source` is a data.frame, this is the name to refer to
    #'   it by in queries (typically the variable name). If not provided, will
    #'   be inferred from the variable name for data.frame inputs. For database
    #'   connections or `NULL` data sources, this parameter is required.
    #' @param ... Additional arguments (currently unused).
    #' @param id Optional module ID for the QueryChat instance. If not provided,
    #'   will be auto-generated from `table_name`. The ID is used to namespace
    #'   the Shiny module.
    #' @param greeting Optional initial message to display to users. Can be a
    #'   character string (in Markdown format) or a file path. If not provided,
    #'   a greeting will be generated at the start of each conversation using
    #'   the LLM, which adds latency and cost. Use `$generate_greeting()` to
    #'   create a greeting to save and reuse.
    #' @param client Optional chat client. Can be:
    #'   - An [ellmer::Chat] object
    #'   - A string to pass to [ellmer::chat()] (e.g., `"openai/gpt-4o"`)
    #'   - `NULL` (default): Uses the `querychat.client` option, the
    #'     `QUERYCHAT_CLIENT` environment variable, or defaults to
    #'     [ellmer::chat_openai()]
    #' @param tools Which querychat tools to include in the chat client, by
    #'   default. `"filter"` includes the tools for filtering and resetting the
    #'   dashboard and `"query"` includes the tool for executing SQL queries.
    #'   Use `tools = "filter"` when you only want the dashboard filtering tools,
    #'   or when you want to disable the querying tool entirely to prevent the
    #'   LLM from seeing any of the data in your dataset. The legacy name
    #'   `"update"` is still accepted as an alias for `"filter"`.
    #' @param data_description Optional description of the data in plain text or
    #'   Markdown. Can be a string or a file path. This provides context to the
    #'   LLM about what the data represents.
    #' @param categorical_threshold For text columns, the maximum number of
    #'   unique values to consider as a categorical variable. Default is 20.
    #' @param extra_instructions Optional additional instructions for the chat
    #'   model in plain text or Markdown. Can be a string or a file path.
    #' @param prompt_template Optional path to or string of a custom prompt
    #'   template file. If not provided, the default querychat template will be
    #'   used. See the package prompts directory for the default template
    #'   format.
    #' @param cleanup Whether or not to automatically run `$cleanup()` when the
    #'   Shiny session/app stops. By default, cleanup only occurs if `QueryChat`
    #'   gets created within a Shiny session. Set to `TRUE` to always clean up,
    #'   or `FALSE` to never clean up automatically.
    #'
    #' @return A new `QueryChat` object.
    initialize = function(
      data_source,
      table_name = missing_arg(),
      ...,
      id = NULL,
      greeting = NULL,
      client = NULL,
      tools = c("filter", "query"),
      data_description = NULL,
      categorical_threshold = 20,
      extra_instructions = NULL,
      prompt_template = NULL,
      cleanup = NA
    ) {
      check_dots_empty()

      # Validate arguments
      check_string(id, allow_null = TRUE)
      check_string(greeting, allow_null = TRUE)
      arg_match(
        tools,
        values = c("filter", "update", "query", "visualize"),
        multiple = TRUE
      )
      tools <- normalize_tools(tools)
      check_string(data_description, allow_null = TRUE)
      check_number_whole(categorical_threshold, min = 1)
      check_string(extra_instructions, allow_null = TRUE)
      check_string(prompt_template, allow_null = TRUE)
      check_bool(cleanup, allow_na = TRUE)

      # Handle table_name inference for non-NULL data sources
      if (is_missing(table_name)) {
        if (is.null(data_source)) {
          cli::cli_abort(
            "{.arg table_name} is required when {.arg data_source} is {.val NULL}."
          )
        }
        if (is.data.frame(data_source) || inherits(data_source, "tbl_sql")) {
          table_name <- deparse1(substitute(data_source))
        }
      }

      # Store table_name for later normalization (needed for deferred pattern)
      private$.table_name <- table_name

      # Store init parameters for deferred system prompt building
      private$.prompt_template <- prompt_template
      private$.data_description <- data_description
      private$.extra_instructions <- extra_instructions
      private$.categorical_threshold <- categorical_threshold

      self$id <- id %||% sprintf("querychat_%s", table_name)
      self$tools <- tools

      if (!is.null(greeting) && file.exists(greeting)) {
        greeting <- read_utf8(greeting)
      }
      self$greeting <- greeting

      # Initialize data source (may be NULL for deferred pattern)
      if (!is.null(data_source)) {
        private$.data_source <- normalize_data_source(data_source, table_name)
        private$build_system_prompt()
      }

      private$.client_spec <- client

      # By default, only close automatically if a Shiny session is active
      if (is.na(cleanup)) {
        cleanup <- shiny::isRunning()
      }

      if (cleanup) {
        shiny::onStop(function() {
          cli::cli_inform("Closing data source...")
          self$cleanup()
        })
      }
    },

    #' @description
    #' Create a chat client, complete with registered tools, for the current
    #' data source.
    #'
    #' @param tools Which querychat tools to include in the chat client.
    #'   `"filter"` includes the tools for filtering and resetting the dashboard
    #'   and `"query"` includes the tool for executing SQL queries. By default,
    #'   when `tools = NA`, the values provided at initialization are used.
    #'   The legacy name `"update"` is still accepted as an alias for `"filter"`.
    #' @param update_dashboard Optional function to call with the `query` and
    #'   `title` generated by the LLM for the `update_dashboard` tool.
    #' @param reset_dashboard Optional function to call when the
    #'   `reset_dashboard` tool is called.
    #' @param visualize Optional function to call with a list containing
    #'   `ggsql`, `title`, and `widget_id` when a visualization succeeds.
    #' @param session A Shiny session object. Required when `"visualize"` is
    #'   in `tools` and you want interactive chart rendering. When `NULL`
    #'   (the default), visualizations still execute but are not rendered
    #'   as Shiny outputs.
    client = function(
      tools = NA,
      update_dashboard = function(query, title) {},
      reset_dashboard = function() {},
      visualize = function(data) {},
      session = NULL
    ) {
      private$require_data_source("$client")

      if (!is_na(tools) && !is.null(tools)) {
        tools <- arg_match(
          tools,
          values = c("filter", "update", "query", "visualize"),
          multiple = TRUE
        )
        tools <- normalize_tools(tools)
      }

      private$create_session_client(
        tools = tools,
        session = session,
        update_dashboard = update_dashboard,
        reset_dashboard = reset_dashboard,
        visualize = visualize
      )
    },

    #' @description
    #' Launch a console-based chat interface with the data source.
    #'
    #' @param new Whether to create a new chat client instance or continue the
    #'   conversation from the last console chat session (the default).
    #' @param ... Additional arguments passed to the `$client()` method.
    #' @param tools Which querychat tools to include in the chat client. See
    #'   `$client()` for details. Ignored when not creating a new chat client.
    #'   By default, only the `"query"` tool is included, regardless of the
    #'   `tools` set at initialization.
    console = function(new = FALSE, ..., tools = "query") {
      private$require_data_source("$console")
      check_bool(new)
      if (new || is.null(private$.client_console)) {
        private$.client_console <- self$client(tools = tools, ...)
      }

      ellmer::live_console(private$.client_console)
    },

    #' @description
    #' Create and run a Shiny gadget for chatting with data
    #'
    #' Runs a Shiny gadget (designed for interactive use) that provides a
    #' complete interface for chatting with your data using natural language. If
    #' you're looking to deploy this app or run it through some other means, see
    #' `$app_obj()`.
    #'
    #' ```r
    #' library(querychat)
    #'
    #' qc <- QueryChat$new(mtcars)
    #' qc$app()
    #' ```
    #'
    #' @param ... Arguments passed to `$app_obj()`.
    #' @param bookmark_store The bookmarking storage method. Passed to
    #'   [shiny::enableBookmarking()]. If `"url"` or `"server"`, the chat state
    #'   (including current query) will be bookmarked. Default is `"url"`.
    #'
    #' @return Invisibly returns a list of session-specific values:
    #'  - `df`: The final filtered data frame
    #'  - `sql`: The final SQL query string
    #'  - `title`: The final title
    #'  - `client`: The session-specific chat client instance
    app = function(..., bookmark_store = "url") {
      app <- self$app_obj(..., bookmark_store = bookmark_store)
      vals <- tryCatch(shiny::runGadget(app), interrupt = function(cnd) NULL)
      invisible(vals)
    },

    #' @description
    #' A streamlined Shiny app for chatting with data
    #'
    #' Creates a Shiny app designed for chatting with data, with:
    #' - A sidebar containing the chat interface
    #' - A card displaying the current SQL query
    #' - A card displaying the filtered data table
    #' - A reset button to clear the query
    #'
    #' ```r
    #' library(querychat)
    #'
    #' qc <- QueryChat$new(mtcars)
    #' app <- qc$app_obj()
    #' shiny::runApp(app)
    #' ```
    #'
    #' @param ... Additional arguments (currently unused).
    #' @param bookmark_store The bookmarking storage method. Passed to
    #'  [shiny::enableBookmarking()]. If `"url"` or `"server"`, the chat state
    #'  (including current query) will be bookmarked. Default is `"url"`.
    #'
    #' @return A Shiny app object that can be run with `shiny::runApp()`.
    app_obj = function(..., bookmark_store = "url") {
      private$require_data_source("$app_obj")
      check_installed("DT")
      check_dots_empty()

      table_name <- private$.data_source$table_name

      ui <- function(req) {
        bslib::page_sidebar(
          title = shiny::HTML(
            sprintf(
              "<span>querychat with <code>%s</code></span>",
              table_name
            )
          ),
          class = "bslib-page-dashboard",
          sidebar = self$sidebar(),
          shiny::useBusyIndicators(pulse = TRUE, spinners = FALSE),
          bslib::card(
            fill = FALSE,
            style = bslib::css(max_height = "33%"),
            bslib::card_header(
              shiny::div(
                class = "hstack",
                shiny::div(
                  bsicons::bs_icon("terminal-fill"),
                  shiny::textOutput("query_title", inline = TRUE)
                ),
                shiny::div(
                  class = "ms-auto",
                  shiny::uiOutput("ui_reset", inline = TRUE)
                )
              )
            ),
            shiny::uiOutput("sql_output")
          ),
          bslib::card(
            full_screen = TRUE,
            bslib::card_header(bsicons::bs_icon("table"), "Data"),
            DT::DTOutput("dt")
          ),
          shiny::actionButton(
            "close_btn",
            label = "",
            class = "btn-close",
            style = "position: fixed; top: 6px; right: 6px;"
          )
        )
      }

      server <- function(input, output, session) {
        # Enable bookmarking if bookmark_store is enabled
        enable_bookmarking <- bookmark_store %in% c("url", "server")
        qc_vals <- self$server(enable_bookmarking = enable_bookmarking)

        output$query_title <- shiny::renderText({
          if (shiny::isTruthy(qc_vals$title())) {
            qc_vals$title()
          } else {
            "SQL Query"
          }
        })

        output$ui_reset <- shiny::renderUI({
          shiny::req(qc_vals$sql())

          shiny::actionButton(
            "reset_query",
            label = "Reset Query",
            class = "btn btn-outline-danger btn-sm lh-1"
          )
        })

        shiny::observeEvent(input$reset_query, label = "on_reset_query", {
          qc_vals$sql(NULL)
          qc_vals$title(NULL)
        })

        output$dt <- DT::renderDT({
          df <- qc_vals$df()
          if (inherits(df, "tbl_sql")) {
            # Materialize the query for DT, {dplyr} guaranteed by TblSqlSource
            df <- dplyr::collect(df)
          }

          DT::datatable(
            df,
            fillContainer = TRUE,
            options = list(pageLength = 25, scrollX = TRUE)
          )
        })

        output$sql_output <- shiny::renderUI({
          sql <- if (shiny::isTruthy(qc_vals$sql())) {
            qc_vals$sql()
          } else {
            paste("SELECT * FROM", table_name)
          }

          sql_code <- paste(c("```sql", sql, "```"), collapse = "\n")

          shinychat::output_markdown_stream(
            "sql_code",
            content = sql_code,
            auto_scroll = FALSE,
            width = "100%"
          )
        })

        shiny::observeEvent(input$close_btn, label = "on_close_btn", {
          shiny::stopApp(
            list(
              df = qc_vals$df(),
              sql = qc_vals$sql(),
              title = qc_vals$title(),
              client = qc_vals$client
            )
          )
        })
      }

      shiny::shinyApp(ui, server, enableBookmarking = bookmark_store)
    },

    #' @description
    #' Create a sidebar containing the querychat UI.
    #'
    #' This method generates a [bslib::sidebar()] component containing the chat
    #' interface, suitable for use with [bslib::page_sidebar()] or similar
    #' layouts.
    #'
    #' ```r
    #' qc <- QueryChat$new(mtcars)
    #'
    #' ui <- page_sidebar(
    #'   qc$sidebar(),
    #'   # Main content here
    #' )
    #' ```
    #'
    #' @param ... Additional arguments passed to [bslib::sidebar()].
    #' @param width Width of the sidebar in pixels. Default is 400.
    #' @param height Height of the sidebar. Default is "100%".
    #' @param fillable Whether the sidebar should be fillable. Default is
    #'   `TRUE`.
    #' @param id Optional ID for the QueryChat instance. If not provided, will
    #'   use the ID provided at initialization. If using `$sidebar()` in a Shiny
    #'   module, you'll need to provide `id = ns("your_id")` where `ns` is the
    #'   namespacing function from [shiny::NS()].
    #'
    #' @return A [bslib::sidebar()] UI component.
    sidebar = function(
      ...,
      width = 400,
      height = "100%",
      fillable = TRUE,
      id = NULL
    ) {
      bslib::sidebar(
        width = width,
        height = height,
        fillable = fillable,
        class = "querychat-sidebar",
        ...,
        self$ui(id = id)
      )
    },

    #' @description
    #' Create the UI for the querychat chat interface.
    #'
    #' This method generates the chat UI component. Typically you'll use
    #' `$sidebar()` instead, which wraps this in a sidebar layout.
    #'
    #' ```r
    #' qc <- QueryChat$new(mtcars)
    #'
    #' ui <- fluidPage(
    #'   qc$ui()
    #' )
    #' ```
    #'
    #' @param ... Additional arguments passed to [shinychat::chat_ui()].
    #' @param id Optional ID for the QueryChat instance. If not provided,
    #'   will use the ID provided at initialization. If using `$ui()` in a Shiny
    #'   module, you'll need to provide `id = ns("your_id")` where `ns` is the
    #'   namespacing function from [shiny::NS()].
    #'
    #' @return A UI component containing the chat interface.
    ui = function(..., id = NULL) {
      check_string(id, allow_null = TRUE, allow_empty = FALSE)

      # If called within another module, the UI id needs to be namespaced
      # by that "parent" module. If called in a module *server* context, we
      # can infer the namespace from the session, but if not, the user
      # will need to provide it.
      # NOTE: this isn't a problem for Python since id namespacing is handled
      # implicitly by UI functions like shinychat.chat_ui().
      id <- id %||% namespaced_id(self$id)

      mod_ui(id, ...)
    },

    #' @description
    #' Initialize the querychat server logic.
    #'
    #' This method must be called within a Shiny server function. It sets up the
    #' reactive logic for the chat interface and returns session-specific
    #' reactive values.
    #'
    #' ```r
    #' qc <- QueryChat$new(mtcars)
    #'
    #' server <- function(input, output, session) {
    #'   qc_vals <- qc$server(enable_bookmarking = TRUE)
    #'
    #'   output$data <- renderDataTable(qc_vals$df())
    #'   output$query <- renderText(qc_vals$sql())
    #'   output$title <- renderText(qc_vals$title() %||% "No Query")
    #' }
    #' ```
    #'
    #' @param data_source Optional data source to use. If provided, sets the
    #'   data_source property before initializing server logic. This is useful
    #'   for the deferred pattern where data_source is not known at
    #'   initialization time (e.g., when the data source depends on session-
    #'   specific authentication).
    #' @param client Optional chat client override for this session. Can be an
    #'   [ellmer::Chat] object or a string (e.g., `"openai/gpt-4o"`). If provided,
    #'   overrides the client set at initialization for this session only —
    #'   other sessions are unaffected. This is useful when the client must be
    #'   created within a session scope (e.g., Posit Connect managed credentials).
    #' @param enable_bookmarking Whether to enable bookmarking for the chat
    #'   state. Default is `FALSE`. When enabled, the chat state (including
    #'   current query, title, and chat history) will be saved and restored
    #'   with Shiny bookmarks. This requires that the Shiny app has bookmarking
    #'   enabled via `shiny::enableBookmarking()` or the `enableBookmarking`
    #'   parameter of `shiny::shinyApp()`.
    #' @param ... Ignored.
    #' @param id Optional module ID for the QueryChat instance. If not provided,
    #'   will use the ID provided at initialization. When used in Shiny modules,
    #'   this `id` should match the `id` used in the corresponding UI function
    #'   (i.e., `qc$ui(id = ns("your_id"))` pairs with `qc$server(id =
    #'   "your_id")`).
    #' @param session The Shiny session object.
    #'
    #' @return A list containing session-specific reactive values and the chat
    #'   client with the following elements:
    #'   - `df`: Reactive expression returning the current filtered data frame
    #'   - `sql`: Reactive value for the current SQL query string
    #'   - `title`: Reactive value for the current title
    #'   - `client`: The session-specific chat client instance
    #'
    server = function(
      data_source = NULL,
      client = NULL,
      enable_bookmarking = FALSE,
      ...,
      id = NULL,
      session = shiny::getDefaultReactiveDomain()
    ) {
      check_string(id, allow_null = TRUE, allow_empty = FALSE)
      check_dots_empty()

      if (is.null(session)) {
        cli::cli_abort(
          "{.fn $server} must be called within a Shiny server function"
        )
      }

      if (!is.null(data_source)) {
        self$data_source <- data_source
      }

      private$require_data_source("$server")

      resolved_client_spec <- client %||% private$.client_spec

      create_session_client <- function(...) {
        private$create_session_client(
          client_spec = resolved_client_spec,
          ...
        )
      }

      mod_server(
        id %||% self$id,
        data_source = private$.data_source,
        greeting = self$greeting,
        client = create_session_client,
        tools = self$tools,
        enable_bookmarking = enable_bookmarking
      )
    },

    #' @description
    #' Generate a welcome greeting for the chat.
    #'
    #' By default, `QueryChat$new()` generates a greeting at the start of every
    #' new conversation, which is convenient for getting started and
    #' development, but also might add unnecessary latency and cost. Use this
    #' method to generate a greeting once and save it for reuse.
    #'
    #' ```r
    #' # Create QueryChat object
    #' qc <- QueryChat$new(mtcars)
    #'
    #' # Generate a greeting and save it
    #' greeting <- qc$generate_greeting()
    #' writeLines(greeting, "mtcars_greeting.md")
    #'
    #' # Later, use the saved greeting
    #' qc2 <- QueryChat$new(mtcars, greeting = "mtcars_greeting.md")
    #' ```
    #'
    #' @param echo Whether to print the greeting to the console. Options are
    #'   `"none"` (default, no output) or `"output"` (print to console).
    #'
    #' @return The greeting string in Markdown format.
    generate_greeting = function(echo = c("none", "output")) {
      private$require_data_source("$generate_greeting")
      chat <- private$create_session_client()
      as.character(chat$chat(GREETING_PROMPT, echo = echo))
    },

    #' @description
    #' Clean up resources associated with the data source.
    #'
    #' This method releases any resources (e.g., database connections)
    #' associated with the data source. Call this when you are done using the
    #' QueryChat object to avoid resource leaks.
    #'
    #' Note: If `auto_cleanup` was set to `TRUE` in the constructor, this will
    #' be called automatically when the Shiny app stops.
    #'
    #' @return Invisibly returns `NULL`. Resources are cleaned up internally.
    cleanup = function() {
      if (!is.null(private$.data_source)) {
        private$.data_source$cleanup()
      }
      invisible(NULL)
    }
  ),
  active = list(
    #' @field system_prompt Get the system prompt.
    system_prompt = function() {
      private$require_data_source("$system_prompt")
      private$.system_prompt$render(tools = self$tools)
    },

    #' @field data_source Get or set the current data source. When setting,
    #'   the value is normalized and the system prompt is rebuilt.
    data_source = function(value) {
      if (missing(value)) {
        private$.data_source
      } else {
        private$.data_source <- normalize_data_source(
          value,
          private$.table_name
        )
        private$build_system_prompt()
        invisible(self)
      }
    }
  )
)

#' QueryChat convenience functions
#'
#' Convenience functions for wrapping [QueryChat] creation (i.e., `querychat()`)
#' and app launching (i.e., `querychat_app()`).
#'
#' @examplesIf rlang::is_interactive() && rlang::is_installed("RSQLite")
#' # Quick start - chat with mtcars dataset in one line
#' querychat_app(mtcars)
#'
#' # Add options
#' querychat_app(
#'   mtcars,
#'   greeting = "Welcome to the mtcars explorer!",
#'   client = "openai/gpt-4o"
#' )
#'
#' # Chat with a database table (table_name required)
#' con <- DBI::dbConnect(RSQLite::SQLite(), ":memory:")
#' DBI::dbWriteTable(con, "mtcars", mtcars)
#' querychat_app(con, "mtcars")
#'
#' # Create QueryChat class object
#' qc <- querychat(mtcars, greeting = "Welcome to the mtcars explorer!")
#'
#' # Run the app later
#' qc$app()
#'
#' @param data_source Either a data.frame or a database connection (e.g., DBI
#'   connection).
#' @param table_name A string specifying the table name to use in SQL queries.
#'   If `data_source` is a data.frame, this is the name to refer to it by in
#'   queries (typically the variable name). If not provided, will be inferred
#'   from the variable name for data.frame inputs. For database connections,
#'   this parameter is required.
#' @param ... Additional arguments (currently unused).
#' @param id Optional module ID for the QueryChat instance. If not provided,
#'   will be auto-generated from `table_name`. The ID is used to namespace
#'   the Shiny module.
#' @param greeting Optional initial message to display to users. Can be a
#'   character string (in Markdown format) or a file path. If not provided,
#'   a greeting will be generated at the start of each conversation using the
#'   LLM, which adds latency and cost. Use `$generate_greeting()` to create
#'   a greeting to save and reuse.
#' @param client Optional chat client. Can be:
#'   - An [ellmer::Chat] object
#'   - A string to pass to [ellmer::chat()] (e.g., `"openai/gpt-4o"`)
#'   - `NULL` (default): Uses the `querychat.client` option, the
#'     `QUERYCHAT_CLIENT` environment variable, or defaults to
#'     [ellmer::chat_openai()]
#' @param tools Which querychat tools to include in the chat client, by
#'   default. `"filter"` includes the tools for filtering and resetting the
#'   dashboard and `"query"` includes the tool for executing SQL queries.
#'   Use `tools = "filter"` when you only want the dashboard filtering tools,
#'   or when you want to disable the querying tool entirely to prevent the
#'   LLM from seeing any of the data in your dataset. The legacy name
#'   `"update"` is still accepted as an alias for `"filter"`.
#' @param data_description Optional description of the data in plain text or
#'   Markdown. Can be a string or a file path. This provides context to the
#'   LLM about what the data represents.
#' @param categorical_threshold For text columns, the maximum number of unique
#'   values to consider as a categorical variable. Default is 20.
#' @param extra_instructions Optional additional instructions for the chat
#'   model in plain text or Markdown. Can be a string or a file path.
#' @param prompt_template Optional path to or string of a custom prompt
#'   template file. If not provided, the default querychat template will be
#'   used. See the package prompts directory for the default template format.
#' @param cleanup Whether or not to automatically run `$cleanup()` when the
#'   Shiny session/app stops. By default, cleanup only occurs if `QueryChat`
#'   is created within a Shiny app. Set to `TRUE` to always clean up, or
#'   `FALSE` to never clean up automatically.
#'
#'   In `querychat_app()`, in-memory databases created for data frames are
#'   always cleaned up.
#'
#' @return A `QueryChat` object. See [QueryChat] for available methods.
#'
#' @rdname querychat-convenience
#' @export
querychat <- function(
  data_source,
  table_name = missing_arg(),
  ...,
  id = NULL,
  greeting = NULL,
  client = NULL,
  tools = c("filter", "query"),
  data_description = NULL,
  categorical_threshold = 20,
  extra_instructions = NULL,
  prompt_template = NULL,
  cleanup = NA
) {
  if (is_missing(table_name)) {
    if (is.data.frame(data_source) || inherits(data_source, "tbl_sql")) {
      table_name <- deparse1(substitute(data_source))
    }
  }

  QueryChat$new(
    data_source = data_source,
    table_name = table_name,
    ...,
    id = id,
    greeting = greeting,
    client = client,
    tools = tools,
    data_description = data_description,
    categorical_threshold = categorical_threshold,
    extra_instructions = extra_instructions,
    prompt_template = prompt_template,
    cleanup = cleanup
  )
}

#' @rdname querychat-convenience
#' @param bookmark_store The bookmarking storage method. Passed to
#'   [shiny::enableBookmarking()]. If `"url"` or `"server"`, the chat state
#'   (including current query) will be bookmarked. Default is `"url"`.
#' @return Invisibly returns the chat object after the app stops.
#'
#' @export
querychat_app <- function(
  data_source,
  table_name = missing_arg(),
  ...,
  id = NULL,
  greeting = NULL,
  client = NULL,
  tools = c("filter", "query"),
  data_description = NULL,
  categorical_threshold = 20,
  extra_instructions = NULL,
  prompt_template = NULL,
  cleanup = NA,
  bookmark_store = "url"
) {
  if (shiny::isRunning()) {
    cli::cli_abort(
      "{.fn querychat_app} cannot be called from within a Shiny app. Use {.fn querychat} instead."
    )
  }

  if (is_missing(table_name) && is.data.frame(data_source)) {
    table_name <- deparse1(substitute(data_source))
  }

  check_bool(cleanup, allow_na = TRUE)
  if (is.data.frame(data_source)) {
    cleanup <- TRUE
  } else if (is.na(cleanup)) {
    cleanup <- FALSE
  }

  qc <- QueryChat$new(
    data_source = data_source,
    table_name = table_name,
    ...,
    id = id,
    greeting = greeting,
    client = client,
    tools = tools,
    data_description = data_description,
    categorical_threshold = categorical_threshold,
    extra_instructions = extra_instructions,
    prompt_template = prompt_template,
    cleanup = cleanup
  )

  qc$app(bookmark_store = bookmark_store)
}

normalize_tools <- function(tools) {
  if (is.null(tools)) {
    return(NULL)
  }
  tools[tools == "filter"] <- "update"
  unique(tools)
}

normalize_data_source <- function(data_source, table_name) {
  if (is_data_source(data_source)) {
    return(data_source)
  }

  check_sql_table_name(table_name, call = caller_env())

  if (is.data.frame(data_source)) {
    return(DataFrameSource$new(data_source, table_name))
  }

  if (inherits(data_source, "tbl_sql")) {
    return(TblSqlSource$new(data_source, table_name))
  }

  if (inherits(data_source, "DBIConnection")) {
    return(DBISource$new(data_source, table_name))
  }

  cli::cli_abort(
    "{.arg data_source} must be a {.cls DataSource}, {.cls data.frame}, or {.cls DBIConnection}, not {.obj_type_friendly {data_source}}."
  )
}

namespaced_id <- function(id, session = shiny::getDefaultReactiveDomain()) {
  if (is.null(session)) {
    id
  } else {
    session$ns(id)
  }
}

describe("QueryChat$new()", {
  skip_if_no_dataframe_engine()

  it("automatically converts data.frame to DataFrameSource", {
    qc <- QueryChat$new(
      data_source = new_test_df(),
      table_name = "test_df",
      greeting = "Test greeting"
    )
    withr::defer(qc$cleanup())

    expect_s3_class(qc$data_source, "DataSource")
    expect_s3_class(qc$data_source, "DataFrameSource")
  })

  it("accepts DataFrameSource directly", {
    df_source <- local_data_frame_source(new_test_df(), "test_source")

    qc <- QueryChat$new(
      data_source = df_source,
      table_name = "test_source",
      greeting = "Test greeting"
    )

    expect_s3_class(qc$data_source, "DataFrameSource")
    expect_equal(qc$data_source$table_name, "test_source")
  })

  it("accepts DBISource", {
    db <- local_sqlite_connection(new_test_df())
    dbi_source <- DBISource$new(db$conn, "test_table")

    qc <- QueryChat$new(
      data_source = dbi_source,
      table_name = "test_table",
      greeting = "Test greeting"
    )

    expect_s3_class(qc$data_source, "DBISource")
    expect_equal(qc$data_source$table_name, "test_table")
  })

  it("infers table_name from data.frame variable name", {
    my_data <- new_test_df()
    qc <- QueryChat$new(my_data, greeting = "Test")
    withr::defer(qc$cleanup())

    expect_equal(qc$data_source$table_name, "my_data")
    expect_equal(qc$id, "querychat_my_data")
  })

  it("loads greeting from file if file exists", {
    withr::local_envvar(OPENAI_API_KEY = "boop")

    greeting_file <- withr::local_tempfile(fileext = ".md")
    greeting_text <- "# Welcome to the data!"
    writeLines(greeting_text, greeting_file)

    qc <- QueryChat$new(
      new_test_df(),
      table_name = "test_df",
      greeting = greeting_file
    )
    withr::defer(qc$cleanup())

    # File content should be loaded (exact format depends on read_utf8 implementation)
    expect_type(qc$greeting, "character")
    expect_match(qc$greeting, greeting_text, fixed = TRUE)
  })

  it("uses greeting string directly if file doesn't exist", {
    qc <- QueryChat$new(
      new_test_df(),
      table_name = "test_df",
      greeting = "Simple greeting"
    )
    withr::defer(qc$cleanup())
    expect_match(qc$greeting, "Simple greeting", fixed = TRUE)
  })

  it("errors with invalid argument types", {
    expect_snapshot(error = TRUE, {
      QueryChat$new(test_df, table_name = "test", id = 123)
    })

    expect_snapshot(error = TRUE, {
      QueryChat$new(test_df, table_name = "test", greeting = 123)
    })

    expect_snapshot(error = TRUE, {
      QueryChat$new(
        test_df,
        table_name = "test",
        categorical_threshold = "not_a_number"
      )
    })

    expect_snapshot(error = TRUE, {
      QueryChat$new(test_df, table_name = "test", cleanup = "not_logical")
    })
  })
})

describe("QueryChat deferred client", {
  it("accepts NULL data_source with table_name", {
    qc <- QueryChat$new(NULL, "users", greeting = "Test")
    expect_null(qc$data_source)
    expect_equal(qc$id, "querychat_users")
  })

  it("requires table_name when data_source is NULL", {
    expect_error(
      QueryChat$new(NULL),
      "table_name.*required"
    )
  })

  it("stores client spec without resolving it", {
    withr::local_envvar(OPENAI_API_KEY = NA)
    withr::local_options(querychat.client = NULL)
    qc <- QueryChat$new(NULL, "users", greeting = "Test")
    expect_null(qc$data_source)
  })

  it("stores explicit client string as spec", {
    withr::local_envvar(OPENAI_API_KEY = "boop")
    qc <- QueryChat$new(NULL, "users", greeting = "Test", client = "openai")
    expect_null(qc$data_source)
  })

  it("$client() errors when data_source is NULL", {
    qc <- QueryChat$new(NULL, "users", greeting = "Test")
    expect_error(qc$client(), "data_source.*must be set")
  })

  it("$console() errors when data_source is NULL", {
    qc <- QueryChat$new(NULL, "users", greeting = "Test")
    expect_error(qc$console(), "data_source.*must be set")
  })

  it("$generate_greeting() errors when data_source is NULL", {
    qc <- QueryChat$new(NULL, "users", greeting = "Test")
    expect_error(qc$generate_greeting(), "data_source.*must be set")
  })

  it("$system_prompt errors when data_source is NULL", {
    qc <- QueryChat$new(NULL, "users", greeting = "Test")
    expect_error(qc$system_prompt, "data_source.*must be set")
  })

  it("works after setting data_source later", {
    skip_if_no_dataframe_engine()
    qc <- QueryChat$new(NULL, "users", greeting = "Test")
    qc$data_source <- new_users_df()

    expect_s3_class(qc$data_source, "DataFrameSource")
    prompt <- qc$system_prompt
    expect_match(prompt, "users")
  })
})

describe("QueryChat integration with DBISource", {
  it("works with iris dataset queries", {
    skip_if_not_installed("RSQLite")

    library(DBI)
    library(RSQLite)

    temp_db <- withr::local_tempfile(fileext = ".db")
    conn <- dbConnect(RSQLite::SQLite(), temp_db)
    dbWriteTable(conn, "iris", iris, overwrite = TRUE)
    dbDisconnect(conn)

    db_conn <- dbConnect(RSQLite::SQLite(), temp_db)
    withr::defer(dbDisconnect(db_conn))

    iris_source <- DBISource$new(db_conn, "iris")

    withr::local_envvar(OPENAI_API_KEY = "boop")
    mock_client <- ellmer::chat_openai()

    qc <- QueryChat$new(
      data_source = iris_source,
      table_name = "iris",
      greeting = "Test greeting",
      client = mock_client
    )

    expect_s3_class(qc$data_source, "DBISource")
    expect_s3_class(qc$data_source, "DataSource")

    result_data <- qc$data_source$execute_query(NULL)
    expect_s3_class(result_data, "data.frame")
    expect_equal(nrow(result_data), 150)
    expect_equal(ncol(result_data), 5)

    query_result <- qc$data_source$execute_query(
      "SELECT \"Sepal.Length\", \"Sepal.Width\" FROM iris WHERE \"Species\" = 'setosa'"
    )
    expect_s3_class(query_result, "data.frame")
    expect_equal(nrow(query_result), 50)
    expect_equal(ncol(query_result), 2)
    expect_true(all(c("Sepal.Length", "Sepal.Width") %in% names(query_result)))
  })
})

describe("QueryChat$cleanup()", {
  it("cleans up data source resources", {
    test_df <- new_test_df()
    qc <- QueryChat$new(test_df, greeting = "Test")

    # Cleanup should not error
    expect_silent(qc$cleanup())

    # Should be idempotent
    expect_silent(qc$cleanup())
  })
})

describe("QueryChat$system_prompt", {
  it("returns the system prompt from the client", {
    test_df <- new_test_df()
    qc <- QueryChat$new(test_df, greeting = "Test")
    withr::defer(qc$cleanup())

    prompt <- qc$system_prompt

    expect_type(prompt, "character")
    expect_true(nchar(prompt) > 0)
    # Should contain table name
    expect_true(grepl("test_df", prompt, fixed = TRUE))
  })

  it("doesn't include update instructions if not enabled", {
    template <- "{{#has_tool_update}}update tool enabled!{{/has_tool_update}}"

    qc <- QueryChat$new(
      new_test_df(),
      "test_df",
      tools = "query",
      prompt_template = template
    )
    withr::defer(qc$cleanup())

    expect_equal(qc$system_prompt, "")
  })

  it("doesn't include query instructions if not enabled", {
    template <- "{{#has_tool_query}}query tool enabled!{{/has_tool_query}}"

    qc <- QueryChat$new(
      new_test_df(),
      "test_df",
      tools = "update",
      prompt_template = template
    )
    withr::defer(qc$cleanup())

    expect_equal(qc$system_prompt, "")
  })

  it("doesn't include update instructions if not enabled (full prompt)", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df",
      tools = "query"
    )
    withr::defer(qc$cleanup())

    tool_header <- "Filtering and Sorting Data"

    expect_no_match(qc$system_prompt, tool_header)
    expect_no_match(qc$client()$get_system_prompt(), tool_header)
    expect_match(qc$client(tools = "update")$get_system_prompt(), tool_header)
  })

  it("doesn't include query instructions if not enabled (full prompt)", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df",
      tools = "update"
    )
    withr::defer(qc$cleanup())

    tool_header <- "Answering Questions About Data"

    expect_no_match(qc$system_prompt, tool_header)
    expect_no_match(qc$client()$get_system_prompt(), tool_header)
    expect_match(qc$client(tools = "query")$get_system_prompt(), tool_header)
  })
})

describe("QueryChat$data_source", {
  skip_if_no_dataframe_engine()

  it("returns the data source object", {
    test_df <- new_test_df()
    qc <- QueryChat$new(test_df, greeting = "Test")
    withr::defer(qc$cleanup())

    ds <- qc$data_source

    expect_s3_class(ds, "DataSource")
    expect_s3_class(ds, "DataFrameSource")
    expect_equal(ds$table_name, "test_df")
  })
})

describe("QueryChat$client()", {
  it("uses default tools when tools = NA", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df",
      tools = c("update", "query")
    )
    withr::defer(qc$cleanup())

    client <- qc$client(tools = NA)

    # Should have both update and query tools
    tool_names <- sapply(client$get_tools(), function(t) t@name)
    expect_contains(tool_names, "querychat_update_dashboard")
    expect_contains(tool_names, "querychat_reset_dashboard")
    expect_contains(tool_names, "querychat_query")
  })

  it("overrides default tools when tools parameter is provided", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df",
      tools = c("update", "query")
    )
    withr::defer(qc$cleanup())

    client <- qc$client(tools = "query")

    # Should only have query tool
    tool_names <- sapply(client$get_tools(), function(t) t@name)
    expect_contains(tool_names, "querychat_query")
    expect_false("querychat_update_dashboard" %in% tool_names)
    expect_false("querychat_reset_dashboard" %in% tool_names)
  })

  it("registers only update tools when tools = 'update'", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df"
    )
    withr::defer(qc$cleanup())

    client <- qc$client(tools = "update")

    tool_names <- sapply(client$get_tools(), function(t) t@name)
    expect_contains(tool_names, "querychat_update_dashboard")
    expect_contains(tool_names, "querychat_reset_dashboard")
    expect_false("querychat_query" %in% tool_names)
  })

  it("treats 'filter' as an alias for 'update'", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df"
    )
    withr::defer(qc$cleanup())

    client <- qc$client(tools = "filter")

    tool_names <- sapply(client$get_tools(), function(t) t@name)
    expect_contains(tool_names, "querychat_update_dashboard")
    expect_contains(tool_names, "querychat_reset_dashboard")
    expect_false("querychat_query" %in% tool_names)
  })

  it("normalizes c('filter', 'query') to update + query tools", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df",
      tools = c("filter", "query")
    )
    withr::defer(qc$cleanup())

    client <- qc$client(tools = NA)

    tool_names <- sapply(client$get_tools(), function(t) t@name)
    expect_contains(tool_names, "querychat_update_dashboard")
    expect_contains(tool_names, "querychat_reset_dashboard")
    expect_contains(tool_names, "querychat_query")
  })

  it("deduplicates when both 'filter' and 'update' are provided", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df",
      tools = c("filter", "update", "query")
    )
    withr::defer(qc$cleanup())

    expect_equal(qc$tools, c("update", "query"))
  })

  it("registers only query tool when tools = 'query'", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df"
    )
    withr::defer(qc$cleanup())

    client <- qc$client(tools = "query")

    tool_names <- sapply(client$get_tools(), function(t) t@name)
    expect_contains(tool_names, "querychat_query")
    expect_false("querychat_update_dashboard" %in% tool_names)
    expect_false("querychat_reset_dashboard" %in% tool_names)
  })

  it("registers visualize tool when tools include 'visualize'", {
    skip_if_not_installed("ggsql")
    session <- structure(
      list(
        output = list(),
        ns = identity
      ),
      class = "MockShinySession"
    )

    qc <- QueryChat$new(
      new_test_df(),
      "test_df",
      tools = c("query", "visualize")
    )
    withr::defer(qc$cleanup())

    client <- qc$client(tools = c("query", "visualize"), session = session)

    tool_names <- sapply(client$get_tools(), function(t) t@name)
    expect_contains(tool_names, "querychat_query")
    expect_contains(tool_names, "querychat_visualize")
    expect_false("querychat_update_dashboard" %in% tool_names)
    expect_false("querychat_reset_dashboard" %in% tool_names)
  })

  it("registers visualize tool without a session", {
    skip_if_not_installed("ggsql")

    qc <- QueryChat$new(
      new_test_df(),
      "test_df"
    )
    withr::defer(qc$cleanup())

    client <- qc$client(tools = "visualize")

    tool_names <- sapply(client$get_tools(), function(t) t@name)
    expect_equal(unname(tool_names), "querychat_visualize")
  })

  it("registers only visualize tool when tools = 'visualize'", {
    skip_if_not_installed("ggsql")
    session <- structure(
      list(
        output = list(),
        ns = identity
      ),
      class = "MockShinySession"
    )

    qc <- QueryChat$new(
      new_test_df(),
      "test_df"
    )
    withr::defer(qc$cleanup())

    client <- qc$client(tools = "visualize", session = session)

    tool_names <- sapply(client$get_tools(), function(t) t@name)
    expect_equal(unname(tool_names), "querychat_visualize")
  })

  it("returns client with no tools when tools = NULL", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df"
    )
    withr::defer(qc$cleanup())

    client <- qc$client(tools = NULL)

    expect_length(client$get_tools(), 0)
  })

  it("sets system prompt based on tools parameter", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df"
    )
    withr::defer(qc$cleanup())

    client_query <- qc$client(tools = "query")
    client_update <- qc$client(tools = "update")

    prompt_query <- client_query$get_system_prompt()
    prompt_update <- client_update$get_system_prompt()

    # Query client should have query instructions but not update
    expect_match(prompt_query, "Answering Questions About Data")
    expect_no_match(prompt_query, "Filtering and Sorting Data")

    # Update client should have update instructions but not query
    expect_match(prompt_update, "Filtering and Sorting Data")
    expect_no_match(prompt_update, "Answering Questions About Data")
  })

  it("passes update_dashboard callback to tool", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df"
    )
    withr::defer(qc$cleanup())

    update_calls <- list()
    client <- qc$client(
      tools = "update",
      update_dashboard = function(query, title) {
        update_calls <<- list(query = query, title = title)
      }
    )

    # Find and call the update tool
    tools <- client$get_tools()
    update_tool <- tools[[
      which(
        sapply(tools, function(t) {
          t@name == "querychat_update_dashboard"
        })
      )
    ]]

    # Call the tool - it should execute the query and call the callback
    result <- update_tool(
      query = "SELECT * FROM test_df WHERE id = 1",
      title = "Test Filter"
    )

    expect_null(result@error)
    expect_equal(update_calls$query, "SELECT * FROM test_df WHERE id = 1")
    expect_equal(update_calls$title, "Test Filter")
  })

  it("passes reset_dashboard callback to tool", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df"
    )
    withr::defer(qc$cleanup())

    reset_called <- FALSE
    client <- qc$client(
      tools = "update",
      reset_dashboard = function() {
        reset_called <<- TRUE
      }
    )

    # Find and call the reset tool
    tools <- client$get_tools()
    reset_tool <- tools[[
      which(
        sapply(tools, function(t) {
          t@name == "querychat_reset_dashboard"
        })
      )
    ]]

    # Call the tool
    reset_tool()

    expect_true(reset_called)
  })

  it("returns independent client instances on each call", {
    qc <- QueryChat$new(
      new_test_df(),
      "test_df"
    )
    withr::defer(qc$cleanup())

    client1 <- qc$client()
    client2 <- qc$client()

    # Should be different objects
    expect_false(identical(client1, client2))

    # Modifying one shouldn't affect the other
    client1$set_turns(list(ellmer::Turn("user", "test message")))
    expect_length(client1$get_turns(), 1)
    expect_length(client2$get_turns(), 0)
  })

  it("returns independent instances when client spec is a Chat object", {
    withr::local_envvar(OPENAI_API_KEY = "boop")
    chat_spec <- ellmer::chat_openai()

    qc <- QueryChat$new(
      new_test_df(),
      "test_df",
      client = chat_spec
    )
    withr::defer(qc$cleanup())

    client1 <- qc$client()
    client2 <- qc$client()

    expect_false(identical(client1, client2))

    client1$set_turns(list(ellmer::Turn("user", "test message")))
    expect_length(client1$get_turns(), 1)
    expect_length(client2$get_turns(), 0)
  })

  it("respects QueryChat initialization tools by default", {
    qc_query_only <- QueryChat$new(
      new_test_df(),
      "test_df",
      tools = "query"
    )
    withr::defer(qc_query_only$cleanup())

    client <- qc_query_only$client()
    tool_names <- sapply(client$get_tools(), function(t) t@name)

    expect_contains(tool_names, "querychat_query")
    expect_false("querychat_update_dashboard" %in% tool_names)
  })
})

test_that("QueryChat$generate_greeting() generates a greeting using the LLM client", {
  client <- mock_ellmer_chat_client(
    public = list(
      chat = function(message, ...) {
        expect_equal(message, GREETING_PROMPT)
        "Welcome! This is a mock response for testing."
      }
    )
  )

  test_df <- new_test_df()

  # Create a mock client that returns a fixed greeting
  qc <- QueryChat$new(test_df, client = client)
  withr::defer(qc$cleanup())

  greeting <- qc$generate_greeting()
  expect_equal(greeting, "Welcome! This is a mock response for testing.")
})

test_that("QueryChat$server() errors when called outside Shiny context", {
  withr::local_envvar(OPENAI_API_KEY = "boop")

  test_df <- new_test_df()
  qc <- QueryChat$new(test_df, greeting = "Test")
  withr::defer(qc$cleanup())

  expect_snapshot(error = TRUE, {
    qc$server()
  })
})

describe("querychat()", {
  skip_if_no_dataframe_engine()
  withr::local_envvar(OPENAI_API_KEY = "boop")

  it("creates a QueryChat object", {
    test_df <- new_test_df()
    qc <- querychat(test_df, greeting = "Test greeting")
    withr::defer(qc$cleanup())

    expect_s3_class(qc, "QueryChat")
    expect_s3_class(qc$data_source, "DataFrameSource")
    expect_equal(qc$greeting, "Test greeting")
  })

  it("infers table_name from variable name", {
    my_test_data <- new_test_df()
    qc <- querychat(my_test_data, greeting = "Test")
    withr::defer(qc$cleanup())

    expect_equal(qc$data_source$table_name, "my_test_data")
  })

  it("passes all arguments to QueryChat$new()", {
    test_df <- new_test_df()

    qc <- querychat(
      test_df,
      table_name = "custom_name",
      id = "custom_id",
      greeting = "Custom greeting",
      categorical_threshold = 10,
      cleanup = FALSE
    )
    withr::defer(qc$cleanup())

    expect_equal(qc$id, "custom_id")
    expect_equal(qc$greeting, "Custom greeting")
    expect_equal(qc$data_source$table_name, "custom_name")
  })
})

describe("QueryChat$console()", {
  local_mocked_r6_class(
    QueryChat,
    public = list(
      get_client_console = function() {
        private$.client_console
      }
    )
  )

  it("defaults to query-only tools (privacy-focused)", {
    qc <- local_querychat()

    live_console_called <- FALSE
    local_mocked_bindings(
      live_console = function(chat) {
        live_console_called <<- TRUE
      },
      .package = "ellmer"
    )

    qc$console()
    expect_true(live_console_called)

    console_client <- qc$get_client_console()
    expect_s3_class(console_client, "Chat")

    tools <- console_client$get_tools()
    expect_equal(names(tools), "querychat_query")
  })

  it("persists console client across calls", {
    qc <- local_querychat()

    # Track live_console calls
    live_console_call_count <- 0
    local_mocked_bindings(
      live_console = function(chat) {
        live_console_call_count <<- live_console_call_count + 1
      },
      .package = "ellmer"
    )

    qc$console()
    first_client <- qc$get_client_console()

    qc$console()
    second_client <- qc$get_client_console()

    expect_identical(first_client, second_client)
    expect_equal(live_console_call_count, 2)
  })

  it("creates fresh client when `new = TRUE`", {
    qc <- local_querychat()
    local_mocked_bindings(live_console = identity, .package = "ellmer")

    qc$console()
    first_client <- qc$get_client_console()

    qc$console(new = TRUE)
    second_client <- qc$get_client_console()

    expect_false(identical(first_client, second_client))
  })

  it("allows overriding tools via `tools` parameter", {
    qc <- local_querychat()
    local_mocked_bindings(live_console = identity, .package = "ellmer")

    qc$console(tools = c("update", "query"))

    console_client <- qc$get_client_console()
    expect_s3_class(console_client, "Chat")

    tools <- console_client$get_tools()
    expect_setequal(
      names(tools),
      c(
        "querychat_query",
        "querychat_update_dashboard",
        "querychat_reset_dashboard"
      )
    )
  })
})

describe("normalize_data_source()", {
  skip_if_no_dataframe_engine()

  it("returns DataSource objects unchanged", {
    test_df <- new_test_df()
    df_source <- DataFrameSource$new(test_df, "test_df")
    withr::defer(df_source$cleanup())

    result <- normalize_data_source(df_source, "test_df")

    expect_identical(result, df_source)
  })

  it("converts data.frame to DataFrameSource", {
    test_df <- new_test_df()

    result <- normalize_data_source(test_df, "test_df")
    withr::defer(result$cleanup())

    expect_s3_class(result, "DataFrameSource")
    expect_equal(result$table_name, "test_df")
  })

  it("converts DBIConnection to DBISource", {
    test_df <- new_test_df()
    db <- local_sqlite_connection(test_df)

    result <- normalize_data_source(db$conn, "test_table")

    expect_s3_class(result, "DBISource")
    expect_equal(result$table_name, "test_table")
  })

  it("errors with invalid data source types", {
    expect_snapshot(error = TRUE, {
      normalize_data_source("not_a_data_source", "table_name")
    })

    expect_snapshot(error = TRUE, {
      normalize_data_source(list(a = 1, b = 2), "table_name")
    })

    expect_snapshot(error = TRUE, {
      normalize_data_source(NULL, "table_name")
    })
  })
})

test_that("querychat_app() only cleans up data frame sources on exit", {
  local_mocked_r6_class(
    QueryChat,
    public = list(
      initialize = function(..., cleanup) {
        # have to use an option because the code is evaluated in a far-away env
        options(.test_cleanup = cleanup)
      },
      app = function(...) {}
    )
  )
  withr::local_options(rlang_interactive = TRUE)

  withr::with_options(list(.test_cleanup = NULL), {
    test_df <- new_test_df()
    querychat_app(test_df)
    cleanup_result <- getOption(".test_cleanup")
    expect_true(cleanup_result)
  })

  withr::with_options(list(.test_cleanup = NULL), {
    test_ds <- local_data_frame_source(new_test_df())
    querychat_app(test_ds)
    cleanup_result <- getOption(".test_cleanup")
    expect_false(cleanup_result)
  })

  withr::with_options(list(.test_cleanup = NULL), {
    con <- local_sqlite_connection(new_test_df())
    test_ds <- DBISource$new(con$conn, "test_table")
    querychat_app(test_ds)
    cleanup_result <- getOption(".test_cleanup")
    expect_false(cleanup_result)
  })
})

describe("QueryChat$server() client override", {
  it("accepts a client parameter", {
    withr::local_envvar(OPENAI_API_KEY = "boop")
    test_df <- new_test_df()
    qc <- QueryChat$new(test_df, table_name = "test_df", greeting = "Test")
    withr::defer(qc$cleanup())

    expect_error(
      qc$server(client = "openai"),
      "must be called within a Shiny server function"
    )
  })
})

describe("QueryChat deferred client with $server()", {
  it("$server() errors when data_source is NULL", {
    qc <- QueryChat$new(NULL, "users", greeting = "Test")
    expect_error(
      qc$server(),
      "must be called within a Shiny server function"
    )
  })

  it("$server(data_source=...) sets the data_source", {
    skip_if_no_dataframe_engine()
    qc <- QueryChat$new(NULL, "users", greeting = "Test")
    expect_null(qc$data_source)

    expect_error(
      qc$server(data_source = new_users_df()),
      "must be called within a Shiny server function"
    )
  })
})

# Standalone file: do not edit by hand
# Source: https://github.com/r-lib/rlang/blob/HEAD/R/standalone-types-check.R
# Generated by: usethis::use_standalone("r-lib/rlang", "types-check")
# ----------------------------------------------------------------------
#
# ---
# repo: r-lib/rlang
# file: standalone-types-check.R
# last-updated: 2023-03-13
# license: https://unlicense.org
# dependencies: standalone-obj-type.R
# imports: rlang (>= 1.1.0)
# ---
#
# ## Changelog
#
# 2025-09-19:
# - `check_logical()` gains an `allow_na` argument (@jonthegeek, #1724)
#
# 2024-08-15:
# - `check_character()` gains an `allow_na` argument (@martaalcalde, #1724)
#
# 2023-03-13:
# - Improved error messages of number checkers (@teunbrand)
# - Added `allow_infinite` argument to `check_number_whole()` (@mgirlich).
# - Added `check_data_frame()` (@mgirlich).
#
# 2023-03-07:
# - Added dependency on rlang (>= 1.1.0).
#
# 2023-02-15:
# - Added `check_logical()`.
#
# - `check_bool()`, `check_number_whole()`, and
#   `check_number_decimal()` are now implemented in C.
#
# - For efficiency, `check_number_whole()` and
#   `check_number_decimal()` now take a `NULL` default for `min` and
#   `max`. This makes it possible to bypass unnecessary type-checking
#   and comparisons in the default case of no bounds checks.
#
# 2022-10-07:
# - `check_number_whole()` and `_decimal()` no longer treat
#   non-numeric types such as factors or dates as numbers.  Numeric
#   types are detected with `is.numeric()`.
#
# 2022-10-04:
# - Added `check_name()` that forbids the empty string.
#   `check_string()` allows the empty string by default.
#
# 2022-09-28:
# - Removed `what` arguments.
# - Added `allow_na` and `allow_null` arguments.
# - Added `allow_decimal` and `allow_infinite` arguments.
# - Improved errors with absent arguments.
#
#
# 2022-09-16:
# - Unprefixed usage of rlang functions with `rlang::` to
#   avoid onLoad issues when called from rlang (#1482).
#
# 2022-08-11:
# - Added changelog.
#
# nocov start

# Scalars -----------------------------------------------------------------

.standalone_types_check_dot_call <- .Call

check_bool <- function(
  x,
  ...,
  allow_na = FALSE,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (
    !missing(x) &&
      .standalone_types_check_dot_call(
        ffi_standalone_is_bool_1.0.7,
        x,
        allow_na,
        allow_null
      )
  ) {
    return(invisible(NULL))
  }

  stop_input_type(
    x,
    c("`TRUE`", "`FALSE`"),
    ...,
    allow_na = allow_na,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_string <- function(
  x,
  ...,
  allow_empty = TRUE,
  allow_na = FALSE,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    is_string <- .rlang_check_is_string(
      x,
      allow_empty = allow_empty,
      allow_na = allow_na,
      allow_null = allow_null
    )
    if (is_string) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a single string",
    ...,
    allow_na = allow_na,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

.rlang_check_is_string <- function(x, allow_empty, allow_na, allow_null) {
  if (is_string(x)) {
    if (allow_empty || !is_string(x, "")) {
      return(TRUE)
    }
  }

  if (allow_null && is_null(x)) {
    return(TRUE)
  }

  if (allow_na && (identical(x, NA) || identical(x, na_chr))) {
    return(TRUE)
  }

  FALSE
}

check_name <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    is_string <- .rlang_check_is_string(
      x,
      allow_empty = FALSE,
      allow_na = FALSE,
      allow_null = allow_null
    )
    if (is_string) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a valid name",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

IS_NUMBER_true <- 0
IS_NUMBER_false <- 1
IS_NUMBER_oob <- 2

check_number_decimal <- function(
  x,
  ...,
  min = NULL,
  max = NULL,
  allow_infinite = TRUE,
  allow_na = FALSE,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (missing(x)) {
    exit_code <- IS_NUMBER_false
  } else if (
    0 ==
      (exit_code <- .standalone_types_check_dot_call(
        ffi_standalone_check_number_1.0.7,
        x,
        allow_decimal = TRUE,
        min,
        max,
        allow_infinite,
        allow_na,
        allow_null
      ))
  ) {
    return(invisible(NULL))
  }

  .stop_not_number(
    x,
    ...,
    exit_code = exit_code,
    allow_decimal = TRUE,
    min = min,
    max = max,
    allow_na = allow_na,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_number_whole <- function(
  x,
  ...,
  min = NULL,
  max = NULL,
  allow_infinite = FALSE,
  allow_na = FALSE,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (missing(x)) {
    exit_code <- IS_NUMBER_false
  } else if (
    0 ==
      (exit_code <- .standalone_types_check_dot_call(
        ffi_standalone_check_number_1.0.7,
        x,
        allow_decimal = FALSE,
        min,
        max,
        allow_infinite,
        allow_na,
        allow_null
      ))
  ) {
    return(invisible(NULL))
  }

  .stop_not_number(
    x,
    ...,
    exit_code = exit_code,
    allow_decimal = FALSE,
    min = min,
    max = max,
    allow_na = allow_na,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

.stop_not_number <- function(
  x,
  ...,
  exit_code,
  allow_decimal,
  min,
  max,
  allow_na,
  allow_null,
  arg,
  call
) {
  if (allow_decimal) {
    what <- "a number"
  } else {
    what <- "a whole number"
  }

  if (exit_code == IS_NUMBER_oob) {
    min <- min %||% -Inf
    max <- max %||% Inf

    if (min > -Inf && max < Inf) {
      what <- sprintf("%s between %s and %s", what, min, max)
    } else if (x < min) {
      what <- sprintf("%s larger than or equal to %s", what, min)
    } else if (x > max) {
      what <- sprintf("%s smaller than or equal to %s", what, max)
    } else {
      abort("Unexpected state in OOB check", .internal = TRUE)
    }
  }

  stop_input_type(
    x,
    what,
    ...,
    allow_na = allow_na,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_symbol <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_symbol(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a symbol",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_arg <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_symbol(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "an argument name",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_call <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_call(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a defused call",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_environment <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_environment(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "an environment",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_function <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_function(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a function",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_closure <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_closure(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "an R function",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_formula <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_formula(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a formula",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}


# Vectors -----------------------------------------------------------------

# TODO: Figure out what to do with logical `NA` and `allow_na = TRUE`

check_character <- function(
  x,
  ...,
  allow_na = TRUE,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_character(x)) {
      if (!allow_na && any(is.na(x))) {
        abort(
          sprintf("`%s` can't contain NA values.", arg),
          arg = arg,
          call = call
        )
      }

      return(invisible(NULL))
    }

    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a character vector",
    ...,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_logical <- function(
  x,
  ...,
  allow_na = TRUE,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is_logical(x)) {
      if (!allow_na && any(is.na(x))) {
        abort(
          sprintf("`%s` can't contain NA values.", arg),
          arg = arg,
          call = call
        )
      }
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a logical vector",
    ...,
    allow_na = FALSE,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_data_frame <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (is.data.frame(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a data frame",
    ...,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

# nocov end

## ggsql Syntax Reference

### Quick Reference

```sql
[WITH cte AS (...), ...]
[SELECT columns FROM table WHERE conditions]
VISUALISE [mappings] [FROM source]
DRAW geom_type
    [MAPPING col AS aesthetic, ... FROM source]
    [REMAPPING stat AS aesthetic, ...]
    [SETTING param => value, ...]
    [FILTER sql_condition]
    [PARTITION BY col, ...]
    [ORDER BY col [ASC|DESC], ...]
[SCALE [TYPE] aesthetic [FROM ...] [TO ...] [VIA ...] [SETTING ...] [RENAMING ...]]
[PROJECT [aesthetics] TO coord_system [SETTING ...]]
[FACET var | row_var BY col_var [SETTING free => 'x'|'y'|('x','y'), ncol => N, nrow => N]]
[PLACE geom_type SETTING param => value, ...]
[LABEL x => '...', y => '...', ...]
```

### VISUALISE Clause

Entry point for visualization. Marks where SQL ends and visualization begins. Mappings in VISUALISE and MAPPING accept **column names only** — no SQL expressions, functions, or casts. All data transformations must happen in the SELECT clause.

```sql
-- After SELECT (most common)
SELECT date, revenue, region FROM sales
VISUALISE date AS x, revenue AS y, region AS color
DRAW line

-- Shorthand with FROM (auto-generates SELECT * FROM)
VISUALISE FROM sales
DRAW bar MAPPING region AS x, total AS y

-- FROM can also come first
FROM sales
VISUALISE date AS x, revenue AS y
DRAW line
```

### Mapping Styles

| Style | Syntax | Use When |
|-------|--------|----------|
| Explicit | `date AS x` | Column name differs from aesthetic |
| Implicit | `x` | Column name equals aesthetic name |
| Wildcard | `*` | Map all matching columns automatically |
| Literal | `'string' AS color` | Use a literal value (for legend labels in multi-layer plots) |
| Null | `null AS color` | Suppress an inherited global mapping for this layer |

### DRAW Clause (Layers)

Multiple DRAW clauses create layered visualizations.

```sql
DRAW geom_type
    [MAPPING col AS aesthetic, ... FROM source]
    [REMAPPING stat AS aesthetic, ...]
    [SETTING param => value, ...]
    [FILTER sql_condition]
    [PARTITION BY col, ...]
    [ORDER BY col [ASC|DESC], ...]
```

**Geom types:**

| Category | Types |
|----------|-------|
| Basic | `point`, `line`, `path`, `bar`, `area`, `tile`, `polygon`, `ribbon` |
| Statistical | `histogram`, `density`, `smooth`, `boxplot`, `violin` |
| Annotation | `text`, `label`, `segment`, `arrow`, `rule`, `rect`, `range` |

- `path` is like `line` but preserves data order instead of sorting by x.
- `tile` draws rectangles for heatmaps or range indicators. Map `x`/`y` for center (defaults to width/height of 1), or use `xmin`/`xmax`/`ymin`/`ymax` for explicit bounds.
- `smooth` fits a trendline to data. Settings: `method` (`'nw'` default for kernel regression, `'ols'` for linear, `'tls'` for total least squares), `bandwidth`, `adjust`, `kernel`.
- `text` (or `label`) renders text labels. Map `label` for the text content. Settings: `format` (template string for label formatting), `offset` (pixel offset as `(x, y)`). Labels containing `\n` are automatically split into multiple lines.
- `arrow` draws arrows between two points. Requires `x`, `y`, `xend`, `yend` aesthetics.
- `rule` draws full-span reference lines. Map a value to `y` for a horizontal line or `x` for a vertical line. Optionally map `slope` to create diagonal reference lines: `y = a + slope * x` (when `y` is mapped) or `x = a + slope * y` (when `x` is mapped).
- `rect` draws rectangles. Pick 2 per axis from center (`x`/`y`), min (`xmin`/`ymin`), max (`xmax`/`ymax`), `width`, `height`. Or just map center (defaults to width/height of 1).
- `range` displays interval marks. Requires `x`, `ymin`, `ymax` for vertical intervals, or `y`, `xmin`, `xmax` for horizontal intervals. Use it for confidence intervals, lollipops, and candlestick-style ranges. Setting `width => null` hides the hinges.
- `segment` draws arbitrary connections between two points and requires `x`, `y`, `xend`, and `yend`.
- `line` and `path` support continuously varying `linewidth`, `stroke`, and `opacity` aesthetics within groups.

**Aesthetics (MAPPING):**

| Category | Aesthetics |
|----------|------------|
| Position | `x`, `y`, `xmin`, `xmax`, `ymin`, `ymax`, `xend`, `yend` |
| Color | `color`/`colour`, `fill`, `stroke`, `opacity` |
| Size/Shape | `size`, `shape`, `linewidth`, `linetype`, `width`, `height` |
| Text | `label`, `typeface`, `fontweight`, `italic`, `fontsize`, `hjust`, `vjust`, `rotation` |
| Aggregation | `weight` (for histogram/bar/density/violin) |
| Rule | `slope` (for diagonal `rule` lines) |

**PARTITION BY** groups data without visual encoding (useful for separate lines per group without color):

```sql
DRAW line PARTITION BY category
```

**ORDER BY** controls row ordering within a layer:

```sql
DRAW line ORDER BY date ASC
```

### PLACE Clause (Annotations)

`PLACE` creates annotation layers with literal values only — no data mappings. Use it for reference lines, text labels, and other fixed annotations. All aesthetics are set via `SETTING` and bypass scaling.

```sql
PLACE geom_type SETTING param => value, ...
```

**Examples:**
```sql
-- Horizontal reference line
PLACE rule SETTING y => 100

-- Vertical reference line
PLACE rule SETTING x => '2024-06-01'

-- Multiple reference lines (array values)
PLACE rule SETTING y => (50, 75, 100)

-- Text annotation
PLACE text SETTING x => 10, y => 50, label => 'Threshold'

-- Diagonal reference line (y = -1 + 0.4 * x)
PLACE rule SETTING slope => 0.4, y => -1
```

`PLACE` supports any geom type but is most useful with `rule`, `text`, `segment`, and `tile`. Use `PLACE` for fixed annotation values known at query time; use `DRAW` with `MAPPING` when values come from data columns. Unlike `DRAW`, `PLACE` has no `MAPPING`, `FILTER`, `PARTITION BY`, or `ORDER BY` sub-clauses. Array values in PLACE SETTING are recycled into multiple rows only for supported aesthetics; geom parameters (like `offset` on `text`) are passed through as-is.

### Statistical Layers and REMAPPING

Some layers compute statistics. Use REMAPPING to access computed values:

| Layer | Computed Stats | Default Remapping |
|-------|---------------|-------------------|
| `bar` (y unmapped) | `count`, `proportion` | `count AS y` |
| `histogram` | `count`, `density` | `count AS y` |
| `density` | `density`, `intensity` | `density AS y` |
| `violin` | `density`, `intensity` | `density AS offset` |
| `smooth` | `intensity` | `intensity AS y` |
| `boxplot` | `value`, `type` | `value AS y` |

`boxplot` displays box-and-whisker plots. Settings: `outliers` (`true` default — show outlier points), `coef` (`1.5` default — whisker fence coefficient), `width` (`0.9` default — box width, 0–1).

`smooth` fits a trendline to data. Settings: `method` (`'nw'` or `'nadaraya-watson'` default kernel regression, `'ols'` for OLS linear, `'tls'` for total least squares). NW-only settings: `bandwidth` (numeric), `adjust` (multiplier, default 1), `kernel` (`'gaussian'` default, `'epanechnikov'`, `'triangular'`, `'rectangular'`, `'uniform'`, `'biweight'`, `'quartic'`, `'cosine'`).

`density` computes a KDE from a continuous `x`. Settings: `bandwidth` (numeric), `adjust` (multiplier, default 1), `kernel` (`'gaussian'` default, `'epanechnikov'`, `'triangular'`, `'rectangular'`, `'uniform'`, `'biweight'`, `'quartic'`, `'cosine'`). Use `REMAPPING intensity AS y` to show unnormalized density that reflects group size differences. Use `SETTING position => 'stack'` for stacked densities.

`violin` displays mirrored KDE curves for groups. Requires both `x` (categorical) and `y` (continuous). Accepts the same bandwidth/adjust/kernel settings as density. Use `REMAPPING intensity AS offset` to reflect group size differences. Additional settings: `side` (`'both'` default, `'left'`/`'bottom'`, `'right'`/`'top'` — for half-violin/ridgeline plots), `width` (any value > 0; values > 1 enable ridgeline-style overlapping).

**Examples:**

```sql
-- Density histogram (instead of count)
VISUALISE FROM products
DRAW histogram MAPPING price AS x REMAPPING density AS y

-- Bar showing proportion
VISUALISE FROM sales
DRAW bar MAPPING region AS x REMAPPING proportion AS y

-- Overlay histogram and density on the same scale
VISUALISE FROM measurements
DRAW histogram MAPPING value AS x SETTING opacity => 0.5
DRAW density MAPPING value AS x REMAPPING intensity AS y SETTING opacity => 0.5

-- Violin plot
SELECT department, salary FROM employees
VISUALISE department AS x, salary AS y
DRAW violin
```

### SCALE Clause

Configures how data maps to visual properties. All sub-clauses are optional; type and transform are auto-detected from data when omitted.

```sql
SCALE [TYPE] aesthetic [FROM range] [TO output] [VIA transform] [SETTING prop => value, ...] [RENAMING ...]
```

**Type identifiers** (optional — auto-detected if omitted):

| Type | Description |
|------|-------------|
| `CONTINUOUS` | Numeric data on a continuous axis |
| `DISCRETE` | Categorical/nominal data |
| `BINNED` | Pre-bucketed data |
| `ORDINAL` | Ordered categories with interpolated output |
| `IDENTITY` | Data values are already visual values (e.g., literal hex colors) |

**Important — integer columns used as categories:** When an integer column represents categories (e.g., a 0/1 `survived` column), ggsql will treat it as continuous by default. This causes errors when mapping to `fill`, `color`, `shape`, or using it in `FACET`. Two fixes:
- **Preferred:** Cast to string in the SELECT clause: `SELECT CAST(survived AS VARCHAR) AS survived ...`, then map the column by name in VISUALISE: `survived AS fill`
- **Alternative:** Declare the scale: `SCALE DISCRETE fill` or `SCALE fill VIA bool`

**FROM** — input domain:
```sql
SCALE x FROM (0, 100)            -- explicit min and max
SCALE x FROM (0, null)           -- explicit min, auto max
SCALE DISCRETE x FROM ('A', 'B', 'C')  -- explicit category order
```

**TO** — output range or palette:
```sql
SCALE color TO sequential         -- default continuous palette (derived from navia)
SCALE color TO viridis           -- other continuous: viridis, plasma, inferno, magma, cividis, navia, batlow
SCALE color TO vik               -- diverging: vik, rdbu, rdylbu, spectral, brbg, berlin, roma
SCALE DISCRETE color TO ggsql10  -- discrete (default: ggsql10): tableau10, category10, set1, set2, set3, dark2, paired, kelly
SCALE color TO ('red', 'blue')   -- explicit color array
SCALE size TO (1, 10)            -- numeric output range
```

**VIA** — transformation:
```sql
SCALE x VIA date                 -- date axis (auto-detected from Date columns)
SCALE x VIA datetime             -- datetime axis
SCALE y VIA log10                -- base-10 logarithm
SCALE y VIA sqrt                 -- square root
```

| Category | Transforms |
|----------|------------|
| Logarithmic | `log10`, `log2`, `log` (natural) |
| Power | `sqrt`, `square` |
| Exponential | `exp`, `exp2`, `exp10` |
| Other | `asinh`, `pseudo_log` |
| Temporal | `date`, `datetime`, `time` |
| Type coercion | `integer`, `string`, `bool` |

**SETTING** — additional properties:
```sql
SCALE x SETTING breaks => 5                -- number of tick marks
SCALE x SETTING breaks => '2 months'       -- interval-based breaks
SCALE x SETTING expand => 0.05             -- expand scale range by 5%
SCALE x SETTING reverse => true            -- reverse direction
SCALE y FROM (0, 100) SETTING oob => 'squish'  -- squish out-of-bounds values to range boundary
```

`oob` (out-of-bounds) controls data outside the scale range: `'keep'` (default for x/y), `'censor'` (remove, default for other aesthetics), `'squish'` (clamp to boundary).

**RENAMING** — custom axis/legend labels:
```sql
SCALE DISCRETE x RENAMING 'A' => 'Alpha', 'B' => 'Beta'
SCALE CONTINUOUS x RENAMING * => '{} units'         -- template for all labels
SCALE x VIA date RENAMING * => '{:time %b %Y}'     -- date label formatting
```

### Date/Time Axes

Temporal transforms are auto-detected from column data types, including after `DATE_TRUNC`.

**Break intervals:**
```sql
SCALE x SETTING breaks => 'month'        -- one break per month
SCALE x SETTING breaks => '2 weeks'      -- every 2 weeks
SCALE x SETTING breaks => '3 months'     -- quarterly
SCALE x SETTING breaks => 'year'         -- yearly
```

Valid units: `day`, `week`, `month`, `year` (for date); also `hour`, `minute`, `second` (for datetime/time).

**Date label formatting** (strftime syntax):
```sql
SCALE x VIA date RENAMING * => '{:time %b %Y}'       -- "Jan 2024"
SCALE x VIA date RENAMING * => '{:time %B %d, %Y}'   -- "January 15, 2024"
SCALE x VIA date RENAMING * => '{:time %b %d}'        -- "Jan 15"
```

### PROJECT Clause

Sets coordinate system. Use `PROJECT ... TO` to specify coordinates.

**Coordinate systems:** `cartesian` (default), `polar`.

**Polar aesthetics:** In polar coordinates, positional aesthetics use `angle` and `radius` (instead of `x` and `y`). Variants `anglemin`, `anglemax`, `angleend`, `radiusmin`, `radiusmax`, `radiusend` are also available. Typically you map to `x`/`y` and let `PROJECT TO polar` handle the conversion, but you can use `angle`/`radius` explicitly when needed.

```sql
PROJECT TO cartesian                        -- explicit default (usually omitted)
PROJECT y, x TO cartesian                   -- flip axes (maps y to horizontal, x to vertical)
PROJECT TO polar                            -- pie/radial charts
PROJECT TO polar SETTING start => 90        -- start at 3 o'clock
PROJECT TO polar SETTING inner => 0.5       -- donut chart (50% hole)
PROJECT TO polar SETTING start => -90, end => 90  -- half-circle gauge
```

**Cartesian settings:**
- `clip` — clip out-of-bounds data (default `true`)
- `ratio` — enforce aspect ratio between axes

**Polar settings:**
- `start` — starting angle in degrees (0 = 12 o'clock, 90 = 3 o'clock)
- `end` — ending angle in degrees (default: start + 360; use for partial arcs/gauges)
- `inner` — inner radius as proportion 0–1 (0 = full pie, 0.5 = donut with 50% hole)
- `clip` — clip out-of-bounds data (default `true`)

**Axis flipping:** To create horizontal bar charts or flip axes, use `PROJECT y, x TO cartesian`. This maps anything on `y` to the horizontal axis and `x` to the vertical axis.

### FACET Clause

Creates small multiples (subplots by category).

```sql
FACET category                               -- Single variable, wrapped layout
FACET row_var BY col_var                     -- Grid layout (rows x columns)
FACET category SETTING free => 'y'           -- Independent y-axes
FACET category SETTING free => ('x', 'y')   -- Independent both axes
FACET category SETTING ncol => 4             -- Control number of columns
FACET category SETTING nrow => 2             -- Control number of rows (mutually exclusive with ncol)
```

Custom strip labels via SCALE:
```sql
FACET region
SCALE panel RENAMING 'N' => 'North', 'S' => 'South'
```

Filter to specific panels via SCALE FROM:
```sql
FACET island
SCALE panel FROM ('Biscoe', 'Dream')
```

### LABEL Clause

Use LABEL for axis labels, subtitles, and captions. Do NOT use `LABEL title => ...` — the tool's `title` parameter handles chart titles. Set a label to `null` to suppress it.

Available labels: any aesthetic name (`x`, `y`, `fill`, `color`, etc.), `subtitle`, `caption`.

```sql
LABEL x => 'X Axis Label', y => 'Y Axis Label'
LABEL x => null                                   -- suppress x-axis label
LABEL subtitle => 'Q4 2024 data', caption => 'Source: internal database'
```

## Complete Examples

**Line chart with multiple series:**
```sql
SELECT date, revenue, region FROM sales WHERE year = 2024
VISUALISE date AS x, revenue AS y, region AS color
DRAW line
SCALE x VIA date
LABEL x => 'Date', y => 'Revenue ($)'
```

**Bar chart (auto-count):**
```sql
VISUALISE FROM products
DRAW bar MAPPING category AS x
```

**Horizontal bar chart:**
```sql
SELECT region, COUNT(*) as n FROM sales GROUP BY region
VISUALISE region AS y, n AS x
DRAW bar
PROJECT y, x TO cartesian
```

**Scatter plot with trend line:**
```sql
SELECT mpg, hp, cylinders FROM cars
VISUALISE mpg AS x, hp AS y
DRAW point MAPPING cylinders AS color
DRAW smooth
```

**Histogram with density overlay:**
```sql
VISUALISE FROM measurements
DRAW histogram MAPPING value AS x SETTING bins => 20, opacity => 0.5
DRAW density MAPPING value AS x REMAPPING intensity AS y SETTING opacity => 0.5
```

**Density plot with groups:**
```sql
VISUALISE FROM measurements
DRAW density MAPPING value AS x, category AS color SETTING opacity => 0.7
```

**Heatmap with tile:**
```sql
SELECT day, month, temperature FROM weather
VISUALISE day AS x, month AS y, temperature AS color
DRAW tile
```

**Threshold reference lines (using PLACE):**
```sql
SELECT date, temperature FROM sensor_data
VISUALISE date AS x, temperature AS y
DRAW line
PLACE rule SETTING y => 100, stroke => 'red', linetype => 'dashed'
LABEL y => 'Temperature (F)'
```

**Faceted chart:**
```sql
SELECT month, sales, region FROM data
VISUALISE month AS x, sales AS y
DRAW line
DRAW point
FACET region
SCALE x VIA date
```

**CTE with aggregation and date formatting:**
```sql
WITH monthly AS (
    SELECT DATE_TRUNC('month', order_date) as month, SUM(amount) as total
    FROM orders GROUP BY 1
)
VISUALISE month AS x, total AS y FROM monthly
DRAW line
DRAW point
SCALE x VIA date SETTING breaks => 'month' RENAMING * => '{:time %b %Y}'
LABEL y => 'Revenue ($)'
```

**Ribbon / confidence band:**
```sql
WITH daily AS (
    SELECT DATE_TRUNC('day', timestamp) as day,
           AVG(temperature) as avg_temp,
           MIN(temperature) as min_temp,
           MAX(temperature) as max_temp
    FROM sensor_data
    GROUP BY DATE_TRUNC('day', timestamp)
)
VISUALISE day AS x FROM daily
DRAW ribbon MAPPING min_temp AS ymin, max_temp AS ymax SETTING opacity => 0.3
DRAW line MAPPING avg_temp AS y
SCALE x VIA date
LABEL y => 'Temperature'
```

**Text labels on bars:**
```sql
SELECT region, COUNT(*) AS n FROM sales GROUP BY region
VISUALISE region AS x, n AS y
DRAW bar
DRAW text MAPPING n AS label SETTING offset => (0, -11), fill => 'white'
```

**Lollipop chart:**
```sql
SELECT ROUND(bill_dep) AS bill_dep, COUNT(*) AS n FROM penguins GROUP BY 1
VISUALISE bill_dep AS x, n AS y
DRAW range MAPPING 0 AS ymin, n AS ymax SETTING width => null
DRAW point
```

**Ridgeline / joy plot:**
```sql
VISUALISE temp AS x, month AS y FROM weather
DRAW violin SETTING width => 4, side => 'top'
SCALE ORDINAL y
```

**Donut chart:**
```sql
VISUALISE FROM products
DRAW bar MAPPING category AS fill
PROJECT TO polar SETTING inner => 0.5
```

## Important Notes

1. **Numeric columns as categories**: Integer columns representing categories (e.g., 0/1 `survived`) are treated as continuous by default, causing errors with `fill`, `color`, `shape`, and `FACET`. Fix by casting in SQL or declaring the scale:
    ```sql
    -- WRONG: integer fill without discrete scale — causes validation error
    SELECT sex, survived FROM titanic
    VISUALISE sex AS x, survived AS fill
    DRAW bar

    -- CORRECT: cast to string in SQL (preferred)
    SELECT sex, CAST(survived AS VARCHAR) AS survived FROM titanic
    VISUALISE sex AS x, survived AS fill
    DRAW bar

    -- ALSO CORRECT: declare the scale as discrete
    SELECT sex, survived FROM titanic
    VISUALISE sex AS x, survived AS fill
    DRAW bar
    SCALE DISCRETE fill
    ```
2. **Do not mix `VISUALISE FROM` with a preceding `SELECT`**: `VISUALISE FROM table` is shorthand that auto-generates `SELECT * FROM table`. If you already have a `SELECT`, use `SELECT ... VISUALISE` instead:
    ```sql
    -- WRONG: VISUALISE FROM after SELECT
    SELECT * FROM titanic
    VISUALISE FROM titanic
    DRAW bar MAPPING class AS x

    -- CORRECT: use VISUALISE (without FROM) after SELECT
    SELECT * FROM titanic
    VISUALISE class AS x
    DRAW bar

    -- ALSO CORRECT: use VISUALISE FROM without any SELECT
    VISUALISE FROM titanic
    DRAW bar MAPPING class AS x
    ```
3. **In querychat, all layers must come from the final SQL result**: Do not use layer-specific `FROM source` inside `DRAW ... MAPPING ...` clauses. If you need raw data and a summary in one chart, put both into one final relation and distinguish layers with a column such as `layer_type`:
    ```sql
    WITH raw AS (
      SELECT
        date,
        amount,
        region,
        'raw' AS layer_type
      FROM sales
    ),
    summary AS (
      SELECT
        date,
        AVG(amount) AS amount,
        region,
        'summary' AS layer_type
      FROM sales
      GROUP BY date, region
    ),
    combined AS (
      SELECT * FROM raw
      UNION ALL
      SELECT * FROM summary
    )
    SELECT * FROM combined
    VISUALISE date AS x, amount AS y
    DRAW point MAPPING region AS color FILTER layer_type = 'raw'
    DRAW line MAPPING region AS color FILTER layer_type = 'summary'
    ```
4. **String values use single quotes**: In SETTING, LABEL, and RENAMING clauses, always use single quotes for string values. Double quotes cause parse errors.
5. **Statistical layers**: When using `histogram`, `bar` (without y), `density`, `smooth`, `violin`, or `boxplot`, the layer computes statistics. Use REMAPPING to access `density`, `intensity`, `proportion`, etc.
6. **No trailing commas**: SETTING, LABEL, MAPPING, and RENAMING clauses must not end with a trailing comma. A comma after the last item causes a parse error.
   ```sql
   -- WRONG: trailing comma after the last label
   LABEL x => 'Gender', y => 'Count',

   -- CORRECT
   LABEL x => 'Gender', y => 'Count'
   ```
7. **Bar position adjustments**: Bars stack automatically when `fill` is mapped. Use `SETTING position => 'dodge'` for side-by-side bars, or `position => 'stack', total => 1` for proportional (100%) stacking:
   ```sql
   DRAW bar MAPPING category AS x, subcategory AS fill                                          -- stacked (default)
   DRAW bar MAPPING category AS x, subcategory AS fill SETTING position => 'dodge'              -- side-by-side
   DRAW bar MAPPING category AS x, subcategory AS fill SETTING position => 'stack', total => 1  -- proportional
   ```

Reset the dashboard to its original state

Resets the dashboard to use the original unfiltered dataset and clears any custom title.

If the user asks to reset the dashboard, simply call this tool with no other response. The reset action will be obvious to the user.

If the user asks to start over, call this tool and then provide a new set of suggestions for next steps. Include suggestions that encourage exploration of the data in new directions.

### SEMANTIC_VIEW() Query Syntax

#### Basic Syntax

```sql
SELECT * FROM SEMANTIC_VIEW(
    {view_name}
    METRICS {logical_table}.{metric_name}
    DIMENSIONS {logical_table}.{dimension_name}
    [WHERE {dimension} = 'value']  -- Optional: pre-aggregation filter
)
[WHERE {column} = 'value']  -- Optional: post-aggregation filter
```

#### Key Rules

1. **Use `SEMANTIC_VIEW()` function** - Not direct SELECT FROM the view
2. **No GROUP BY needed** - Semantic layer handles aggregation via DIMENSIONS
3. **No JOINs needed within model** - Relationships are pre-defined
4. **No aggregate functions needed** - Metrics are pre-aggregated
5. **Use DDL-defined names** - Metrics and dimensions must match the DDL exactly

#### WHERE Clause: Inside vs Outside

- **Inside** (pre-aggregation): Filters base data BEFORE metrics are computed
- **Outside** (post-aggregation): Filters results AFTER metrics are computed

```sql
-- Pre-aggregation: only include 'EXT' accounts in the calculation
SELECT * FROM SEMANTIC_VIEW(
    MODEL_NAME
    METRICS T_DATA.NET_REVENUE
    DIMENSIONS REF_ENTITIES.ACC_TYPE_CD
    WHERE REF_ENTITIES.ACC_TYPE_CD = 'EXT'
)

-- Post-aggregation: compute all, then filter results
SELECT * FROM SEMANTIC_VIEW(
    MODEL_NAME
    METRICS T_DATA.NET_REVENUE
    DIMENSIONS REF_ENTITIES.ACC_TYPE_CD
)
WHERE NET_REVENUE > 1000000
```

#### Common Patterns

**Single metric (total):**
```sql
SELECT * FROM SEMANTIC_VIEW(MODEL_NAME METRICS T_DATA.NET_REVENUE)
```

**Metric by dimension:**
```sql
SELECT * FROM SEMANTIC_VIEW(
    MODEL_NAME
    METRICS T_DATA.NET_REVENUE
    DIMENSIONS REF_ENTITIES.ACC_TYPE_CD
)
```

**Multiple metrics and dimensions:**
```sql
SELECT * FROM SEMANTIC_VIEW(
    MODEL_NAME
    METRICS T_DATA.NET_REVENUE, T_DATA.GROSS_REVENUE
    DIMENSIONS REF_ENTITIES.ACC_TYPE_CD, T_DATA.LOG_DT
)
ORDER BY LOG_DT ASC
```

**Time series:**
```sql
SELECT * FROM SEMANTIC_VIEW(
    MODEL_NAME
    METRICS T_DATA.NET_REVENUE
    DIMENSIONS T_DATA.LOG_DT
)
ORDER BY LOG_DT ASC
```

**Join results with other data:**
```sql
SELECT sv.*, lookup.category_name
FROM SEMANTIC_VIEW(
    MODEL_NAME
    METRICS T_DATA.NET_REVENUE
    DIMENSIONS REF_ENTITIES.ACC_TYPE_CD
) AS sv
JOIN category_lookup AS lookup ON sv.ACC_TYPE_CD = lookup.code
```

#### Troubleshooting

- **"Invalid identifier"**: Verify metric/dimension names match exactly what's in the DDL
- **Syntax error**: Use SEMANTIC_VIEW() function, GROUP BY isn't needed

Create a data visualization

Render a ggsql query (SQL with a VISUALISE clause) as an interactive chart displayed inline in the chat.

**When to use:** Call this tool when the user's question involves comparisons, distributions, or trends — even for small result sets, a chart is often clearer than a table.{{#has_tool_query}} For single-value answers (averages, counts, totals, specific lookups) or when the user needs exact values, use `querychat_query` instead.{{/has_tool_query}}

**Key constraints:**

- All data transformations must happen in the `SELECT` clause. `VISUALISE` and `MAPPING` accept column names only, not SQL expressions or functions.
- Do NOT include `LABEL title => ...` in the query — use the `title` parameter instead.
- If a visualization fails, read the error message carefully and retry with a corrected query. Common fixes: correcting column names, adding `SCALE DISCRETE` for integer categories, moving SQL expressions out of `VISUALISE` into the `SELECT` clause, and using `DRAW range` for interval-style marks instead of deprecated `errorbar`.{{#has_tool_query}} If the error persists, fall back to `querychat_query` for a tabular answer.{{/has_tool_query}}

Parameters
----------
ggsql :
    A full ggsql query. Must include a VISUALISE clause and at least one DRAW clause. The SELECT portion uses {{db_type}} SQL; VISUALISE and MAPPING accept column names only, not expressions. Do NOT include `LABEL title => ...` in the query — use the `title` parameter instead.
title :
    A brief, user-friendly title for this visualization. This is displayed as the card header above the chart.

Returns
-------
:
    If successful, a static image of the rendered plot. If not, an error message.

## Semantic Views

**IMPORTANT**: This database has Semantic Views available. Semantic Views provide a curated layer over raw data with pre-defined metrics, dimensions, and relationships. They encode business logic and calculation rules that ensure consistent, accurate results. When a Semantic View covers the data you need, prefer it over raw tables to benefit from these certified definitions (that is, use the `SEMANTIC_VIEW()` table function where appropriate when generating SQL).

**Real-world example**: A legacy ERP database had a revenue column (`X_AMT`) with hidden business rules—only status code 90 transactions count as realized revenue, and a discount factor (`ADJ_FCTR`) must be applied. Querying raw tables for "external customer revenue" returned **$184B**. The same query using the semantic model's certified `NET_REVENUE` metric returned **$84.5B**—the correct answer. The raw query was **2x+ too high** because it ignored discounts and included invalid transaction codes.

---
title: "Data Sources"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Data Sources}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

`querychat` supports several different data sources, including:

1. Data frames
2. DBI database connections (e.g., SQLite, PostgreSQL, MySQL, DuckDB)
3. Custom `DataSource` interfaces

The sections below describe how to use each type of data source with `querychat`.

## Data frames

You can use any data frame as a data source in `querychat`. Simply pass it to `querychat()`:

```{r}
library(querychat)
library(palmerpenguins)

qc <- querychat(mtcars)
qc$app()  # Launch the app
```

Behind the scenes, `querychat` creates an in-memory DuckDB database and registers your data frame as a table for SQL query execution.

## Database connections

You can also connect `querychat` directly to a table in any database supported by [DBI](https://dbi.r-dbi.org/). This includes popular databases like SQLite, DuckDB, PostgreSQL, MySQL, and many more.

Assuming you have a database set up and accessible, you can create a DBI connection and pass it to `querychat()`. Below are some examples for common databases.

### DuckDB

```{r}
library(DBI)
library(duckdb)
library(querychat)

# Connect to a DuckDB database file
con <- dbConnect(duckdb::duckdb(), dbdir = "my_database.duckdb")

qc <- querychat(con, "my_table")
qc$app()  # Launch the app

# Don't forget to disconnect when done
# dbDisconnect(con)
```

### SQLite

```{r}
library(DBI)
library(RSQLite)
library(querychat)

# Connect to a SQLite database file
con <- dbConnect(RSQLite::SQLite(), "my_database.db")

qc <- querychat(con, "my_table")
qc$app()  # Launch the app

# Don't forget to disconnect when done
# dbDisconnect(con)
```

### PostgreSQL

```{r}
library(DBI)
library(RPostgres)
library(querychat)

# Connect to PostgreSQL
con <- dbConnect(
  RPostgres::Postgres(),
  host = "localhost",
  port = 5432,
  dbname = "mydatabase",
  user = "myuser",
  password = "mypassword"
)

qc <- querychat(con, "my_table")
qc$app()  # Launch the app

# Don't forget to disconnect when done
# dbDisconnect(con)
```

### MySQL

```{r}
library(DBI)
library(RMariaDB)
library(querychat)

# Connect to MySQL
con <- dbConnect(
  RMariaDB::MariaDB(),
  host = "localhost",
  port = 3306,
  dbname = "mydatabase",
  user = "myuser",
  password = "mypassword"
)

qc <- querychat(con, "my_table")
qc$app()  # Launch the app

# Don't forget to disconnect when done
# dbDisconnect(con)
```

## Creating a database from a data frame

If you don't have a database set up, you can easily create a local DuckDB database from a data frame:

```{r}
library(DBI)
library(duckdb)

con <- dbConnect(duckdb::duckdb(), dbdir = "my_database.duckdb")

# Write a data frame to the database
dbWriteTable(con, "penguins", penguins)

# Or from CSV
duckdb::duckdb_read_csv(con, "my_table", "path/to/your/file.csv")
```

Then you can connect to this database using the DuckDB example above.

## Custom sources

If you have a custom data source that doesn't fit into the above categories, you can implement the `DataSource` interface.
See the [DataSource reference](../reference/DataSource.html) for more details on implementing this interface.

---
title: "Tools"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Tools}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

querychat combines [tool calling](https://ellmer.tidyverse.org/articles/tool-calling.html) with [reactivity](https://shiny.posit.co/r/articles/build/reactivity-overview.html) to not only execute SQL, but also reactively update dependent data views. Understanding how these tools work will help you better understand what querychat is capable of and how to customize/extend its behavior.

One important thing to understand generally about querychat's tools is they are R functions, and that execution happens on _your machine_, not on the LLM provider's side. In other words, the SQL queries generated by the LLM are executed locally in the R process running the app.

querychat provides the LLM access to three tool groups:

1. **Data updating** - Filter and sort data (without sending results to the LLM).
2. **Data analysis** - Calculate summaries and return results for interpretation by the LLM.
3. **Data visualization** - Create charts inline in the chat.

```{r}
library(querychat)
library(palmerpenguins)
```

## Data updating {#data-updating}

When a user asks to "Show me..." or "Filter to..." or "Sort by...", the LLM requests a call to the `update_dashboard` tool with an appropriate SQL query as input. An important constraint is that the query must return all original schema columns (typically using `SELECT *`). When called, querychat will both set a reactive value holding [the current SQL query](build.html#sql-query) and execute the query to get the result. The result of query then used to set a reactive value holding the [filtered/sorted data frame](build.html#filtered-data). Thanks to reactivity, this will automatically update any views depending on this data frame, such as the data table displayed in the UI.

This tool also takes a `title` parameter, which is a short description of the filter/sort operation (e.g., "Adelie penguins"). This, also, is made available through [a reactive value](build.html#title) for display somewhere in your app.

Here's a basic example of this tool in action with the `$app()` method. Notice how this pre-built app not only shows the data table, but also the SQL query and title generated by the LLM (for transparency):

```{r}
querychat_app(penguins)
```

![](../reference/figures/quickstart-filter.png){alt="Screenshot of the querychat's app with the penguins dataset filtered." class="shadow rounded"}

The other data updating tool is `reset_dashboard`, which clears any active filters and returns the data table to its original unfiltered state. The LLM typically uses this when users say "reset", "start over", or "clear filters".

## Data analysis

When a user asks analytical questions like "What is the average...?", "How many...?", or "Which item has the highest...?", the LLM generates a SQL query and requests a call to the `query` tool. Unlike the data updating tools, this tool will not update any reactive values. Instead, it will:

1. Execute the SQL query
2. Display both the SQL query and results in the UI
3. Return the results back to the LLM for interpretation

Here's an example of it in action:

```{r}
querychat_app(penguins)
```

![](../reference/figures/quickstart-summary.png){alt="Screenshot of the querychat's app with a summary statistic inlined in the chat." class="shadow rounded"}

## Data visualization

When a user asks for a chart or visualization, the LLM generates a [ggsql](https://ggsql.org/) query — standard SQL extended with a `VISUALISE` clause — and requests a call to the `visualize` tool.
This tool:

1. Executes the SQL portion of the query
2. Renders the `VISUALISE` clause as an interactive chart
3. Displays the chart inline in the chat

Unlike the data updating tools, visualization queries don't affect the dashboard filter.
They query the full dataset independently, and each call produces a new inline chart message in the chat.

The inline chart includes controls for fullscreen viewing, saving as PNG/SVG, and a "Show Query" toggle that reveals the underlying ggsql code.

To use the visualization tool, first install the ggsql package:

```r
install.packages("ggsql")
```

Then include `"visualize"` in the `tools` parameter (it is not enabled by default):

```{r}
library(palmerpenguins)

qc <- QueryChat$new(
  penguins,
  tools = c("query", "update", "visualize")
)
qc$app()
```

See [Visualizations](visualize.html) for more details.

## View the source

If you'd like to better understand how the tools work and how the LLM is prompted to use them, check out the following resources:

**Source code:**

- [`querychat_tools.R`](https://github.com/posit-dev/querychat/blob/main/pkg-r/R/querychat_tools.R)
- [`querychat_viz.R`](https://github.com/posit-dev/querychat/blob/main/pkg-r/R/querychat_viz.R)

**Prompts:**

- [`prompts/tool-update-dashboard.md`](https://github.com/posit-dev/querychat/blob/main/pkg-r/inst/prompts/tool-update-dashboard.md)
- [`prompts/tool-reset-dashboard.md`](https://github.com/posit-dev/querychat/blob/main/pkg-r/inst/prompts/tool-reset-dashboard.md)
- [`prompts/tool-query.md`](https://github.com/posit-dev/querychat/blob/main/pkg-r/inst/prompts/tool-query.md)
- [`prompts/tool-visualize.md`](https://github.com/posit-dev/querychat/blob/main/pkg-r/inst/prompts/tool-visualize.md)

You are a data dashboard chatbot that operates in a sidebar interface. Your role is to help users interact with their data through filtering, sorting, and answering questions.{{#has_tool_visualize}} You can also help them explore data visually.{{/has_tool_visualize}}

You have access to a {{db_type}} SQL database with the following schema:

<database_schema>
{{schema}}
</database_schema>

{{#data_description}}
Here is additional information about the data:

<data_description>
{{data_description}}
</data_description>
{{/data_description}}

For security reasons, you may only query this specific table.

{{#include_query_guidelines}}
## SQL Query Guidelines

When writing SQL queries to interact with the database, please adhere to the following guidelines to ensure compatibility and correctness.

### Structural Rules

**No trailing semicolons**
Never end your query with a semicolon (`;`). The parent query needs to continue after your subquery closes.

**Single statement only**
Return exactly one `SELECT` statement. Do not include multiple statements separated by semicolons.

**No procedural or meta statements**
Do not include:
- `EXPLAIN` / `EXPLAIN ANALYZE`
- `SET` statements
- Variable declarations
- Transaction controls (`BEGIN`, `COMMIT`, `ROLLBACK`)
- DDL statements (`CREATE`, `ALTER`, `DROP`)
- `INTO` clauses (e.g., `SELECT INTO`)
- Locking hints (`FOR UPDATE`, `FOR SHARE`)

### Column Naming Rules

**Alias all computed/derived columns**
Every expression that isn't a simple column reference must have an explicit alias.

**Ensure unique column names**
The result set must not have duplicate column names, even when selecting from multiple tables.

**Avoid `SELECT *` with JOINs**
Explicitly list columns to prevent duplicate column names and ensure a predictable output schema.

**Avoid reserved words as unquoted aliases**
If using reserved words as column aliases, quote them appropriately for your dialect.

{{/include_query_guidelines}}
{{#is_duck_db}}
### DuckDB SQL Tips

**Percentile functions:** In standard SQL, `percentile_cont` and `percentile_disc` are "ordered set" aggregate functions that use the `WITHIN GROUP (ORDER BY sort_expression)` syntax. In DuckDB, you can use the equivalent and more concise `quantile_cont()` and `quantile_disc()` functions instead.

**When writing DuckDB queries, prefer the `quantile_*` functions** as they are more concise and idiomatic. Both syntaxes are valid in DuckDB.

Example:
```sql
-- Standard SQL syntax (works but verbose)
percentile_cont(0.5) WITHIN GROUP (ORDER BY salary)

-- Preferred DuckDB syntax (more concise)
quantile_cont(salary, 0.5)
```

{{/is_duck_db}}
{{{semantic_views}}}
## Your Capabilities

You can handle these types of requests:

{{#has_tool_update}}
### Filtering and Sorting Data

When the user asks you to filter or sort the dashboard, e.g. "Show me..." or "Which ____ have the highest ____?" or "Filter to only include ____":

- Write a {{db_type}} SQL SELECT query
- Call `querychat_update_dashboard` with the query and a descriptive title
- The query MUST return all columns from the schema (you can use `SELECT *`)
- Use a single SQL query even if complex (subqueries and CTEs are fine)
- Optimize for **readability over efficiency**
- Include SQL comments to explain complex logic
- No confirmation messages are needed: the user will see your query in the dashboard.

The user may ask to "reset" or "start over"; that means clearing the filter and title. Do this by calling `querychat_reset_dashboard()`.

**Filtering Example:**
User: "Show only rows where sales are above average"
Tool Call: `querychat_update_dashboard({query: "SELECT * FROM table WHERE sales > (SELECT AVG(sales) FROM table)", title: "Above average sales"})`
Response: ""

No further response needed, the user will see the updated dashboard.

{{/has_tool_update}}
{{#has_tool_query}}
### Answering Questions About Data

When the user asks you a question about the data, e.g. "What is the average ____?" or "How many ____ are there?" or "Which ____ has the highest ____?":

- Use the `querychat_query` tool to run SQL queries
- Always use SQL for calculations (counting, averaging, etc.) - NEVER do manual calculations
- Always present key findings in your response text — do not assume the user can see the tool result (it may be collapsed)
- If you cannot complete the request using SQL, politely decline and explain why

**Question Example:**
User: "What's the average revenue?"
Tool Call: `querychat_query({query: "SELECT AVG(revenue) AS avg_revenue FROM table"})`
Response: "The average revenue is $X."

{{/has_tool_query}}
{{#has_tool_visualize}}
### Visualizing Data

You can create visualizations using the `querychat_visualize` tool, which uses ggsql — a SQL extension for declarative data visualization. Write a ggsql query (SQL with a VISUALISE clause), and the tool executes the SQL, renders the VISUALISE clause as an interactive chart, and displays it inline in the chat.

#### Visualization best practices

The database schema in this prompt includes column names, types, and summary statistics. {{#has_tool_query}}If that context isn't sufficient for a confident visualization — e.g., you're unsure about value distributions, need to check for NULLs, or want to gauge row counts before choosing a chart type — use the `querychat_query` tool to inspect the data before visualizing. Always pass `collapsed=true` for these preparatory queries so the chart remains the focal point of the response.{{/has_tool_query}}

Follow the principles below to produce clear, interpretable charts.

#### Axis labels must be readable

When the x-axis contains categorical labels (names, categories, long strings), prefer flipping axes with `PROJECT y, x TO cartesian` so labels read naturally left-to-right. Short numeric or date labels on the x-axis are fine horizontal — this applies specifically to text categories.

#### Always include axis labels with units

Charts should be interpretable without reading the surrounding prose. Always include axis labels that describe what is shown, including units when applicable (e.g., `LABEL y => 'Revenue ($M)'`, not just `LABEL y => 'Revenue'`).

#### Maximize data-ink ratio

Every visual element should serve a purpose:

- Don't map columns to aesthetics (color, size, shape) unless the distinction is meaningful to the user's question. A single-series bar chart doesn't need color.
- When using color for categories, keep to 7 or fewer distinct values. Beyond that, consider filtering to the most important categories or using facets instead.
- Avoid dual-encoding the same variable (e.g., mapping the same column to both x-position and color) unless it genuinely aids interpretation.

#### Avoid overplotting

When a dataset has many rows, plotting one mark per row creates clutter that obscures patterns. Before generating a query, consider the row count and data characteristics visible in the schema.

**For large datasets (hundreds+ rows):**

- **Aggregate first**: Use `GROUP BY` with `COUNT`, `AVG`, `SUM`, or other aggregates to reduce to meaningful summaries before visualizing.
- **Choose chart types that summarize naturally**: histograms for distributions, boxplots for group comparisons, line charts for trends over time.

**For two numeric variables with many rows:**

Bin in SQL and use `DRAW tile` to create a heatmap:

```sql
WITH binned AS (
    SELECT ROUND(x_col / 5) * 5 AS x_bin,
           ROUND(y_col / 5) * 5 AS y_bin,
           COUNT(*) AS n
    FROM large_table
    GROUP BY x_bin, y_bin
)
SELECT * FROM binned
VISUALISE x_bin AS x, y_bin AS y, n AS fill
DRAW tile
SCALE fill TO viridis
```

**If individual points matter** (e.g., outlier detection): use `SETTING opacity` to reveal density through overlap.

#### Choose chart types based on the data relationship

Match the chart type to what the user is trying to understand:

- **Comparison across categories**: bar chart (`DRAW bar`, with `PROJECT y, x TO cartesian` for long labels). Order bars by value, not alphabetically.
- **Trend over time**: line chart (`DRAW line`). Use `SCALE x VIA date` for date columns.
- **Distribution of a single variable**: histogram (`DRAW histogram`) or density (`DRAW density`).
- **Relationship between two numeric variables**: scatter plot (`DRAW point`), but prefer aggregation or heatmap if the dataset is large.
- **Part-of-whole**: stacked bar chart (map subcategory to `fill`). Avoid pie charts — position along a common scale is easier to decode than angle.

#### ggsql syntax reference

<ggsql-syntax-reference>
{{> ggsql-syntax}}
</ggsql-syntax-reference>
{{#has_tool_query}}

**Avoid redundant expanded results.** If you run a preparatory query before visualizing, or if both a table and chart would show the same data, always pass `collapsed=true` on the query so the user sees the chart prominently, not a duplicate table above it. The user can still expand the table if they want the exact values.
{{/has_tool_query}}
{{/has_tool_visualize}}
{{^has_tool_visualize}}
### Visualization Requests

You cannot create charts or visualizations. If users ask for a plot, chart, or visual representation of the data, explain that visualization is not currently enabled.{{#has_tool_query}} Offer to answer their question with a tabular query instead.{{/has_tool_query}} Suggest that the developer can enable visualization by installing the `ggsql` package and adding `"visualize"` to the `tools` parameter.

{{/has_tool_visualize}}
{{^has_tool_query}}
{{^has_tool_visualize}}
### Questions About Data

You cannot query or analyze the data. If users ask questions about data values, statistics, or calculations (e.g., "What is the average ____?" or "How many ____ are there?"), explain that you're not able to run queries on this data. Do not attempt to answer based on your own knowledge or assumptions about the data, even if the dataset seems familiar.

{{/has_tool_visualize}}
{{/has_tool_query}}
### Providing Suggestions for Next Steps

#### Suggestion Syntax

Use `<span class="suggestion">` tags to create clickable suggestion buttons in the UI. The text inside should be a complete, actionable suggestion that users can click to continue the conversation.

#### Syntax Examples

**List format (most common):**
```
<ul>
<li><span class="suggestion">Show me examples of …</span></li>
<li><span class="suggestion">What are the key differences between …</span></li>
<li><span class="suggestion">Explain how …</span></li>
</ul>
```

Use explicit HTML `<ul>`/`<li>` tags instead of markdown list markers (`*`, `-`). Markdown lists work when formatted correctly, but omitting the space after the marker (e.g., `-<span>` instead of `- <span>`) silently breaks the list parse, so HTML tags are more reliable.

**Grouped suggestions:**
```
{{#has_tool_query}}
##### Analyze the data
<ul>
<li><span class="suggestion">What's the average …?</span></li>
<li><span class="suggestion">How many …?</span></li>
</ul>

{{/has_tool_query}}
{{#has_tool_visualize}}
##### Visualize the data
<ul>
<li><span class="suggestion">Show a bar chart of …</span></li>
<li><span class="suggestion">Plot the trend of … over time</span></li>
</ul>

{{/has_tool_visualize}}
##### Filter and sort
<ul>
<li><span class="suggestion">Show records from the year …</span></li>
<li><span class="suggestion">Sort the ____ by ____ …</span></li>
</ul>
```

**Inline in prose (secondary format):**
Inline suggestions render as clickable text links within a sentence, not as cards.
```md
You might want to <span class="suggestion">explore the advanced features</span> or <span class="suggestion">show me a practical example</span>.
```

#### When to Include Suggestions

**Always provide suggestions:**
- At the start of a conversation
- When beginning a new line of exploration
- After completing a topic (to suggest new directions)

**Use best judgment for:**
- Mid-conversation responses (include when they add clear value)
- Follow-up answers (include if multiple paths forward exist)

**Avoid when:**
- The user has asked a very specific question requiring only a direct answer
- The conversation is clearly wrapping up

#### Suggestion Guidelines

- Suggestions can appear **anywhere** in your response—not just at the end
- Use list format at the end for 2-4 follow-up options (most common pattern)
- Never use nested lists for suggestions — group them under headings instead
- Use inline suggestions within prose when contextually appropriate
- Write suggestions as complete, natural prompts (not fragments)
- Only suggest actions you can perform with your tools and capabilities
- Never duplicate the suggestion text in your response
- Never use generic phrases like "If you'd like to..." or "Would you like to explore..." — instead, provide concrete suggestions
- Never refer to suggestions as "prompts" – call them "suggestions" or "ideas" or similar

## Important Guidelines

- **Ask for clarification** if any request is unclear or ambiguous
- **Be concise** due to the constrained interface
- **Only answer data questions using your tools** - never use prior knowledge or assumptions about the data, even if the dataset seems familiar
- **Be skeptical of your own interpretations** - when describing chart results or data patterns, encourage the user to verify findings rather than presenting analytical conclusions as fact
- **Use Markdown tables** when tabular data is the direct answer to the user's question. For simple results, reproduce the full table; for complex results, show only the most relevant subset

{{#extra_instructions}}
## Additional Instructions

{{extra_instructions}}
{{/extra_instructions}}

---
title: "Visualizations"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Visualizations}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

querychat can create charts inline in the chat.
When you ask a question that benefits from a visualization, the LLM writes a query using [ggsql](https://ggsql.org/) — a SQL-like visualization grammar — and renders an interactive chart directly in the conversation.

## Getting started

Visualization requires two steps:

1. **Install the ggsql package:**

    ```r
    install.packages("ggsql")
    ```

2. **Include `"visualize"` in the `tools` parameter:**

    ```r
    library(querychat)
    library(palmerpenguins)

    qc <- QueryChat$new(
      penguins,
      tools = c("query", "update", "visualize")
    )
    qc$app()
    ```

Ask something like "Show me body mass by species as a bar chart" and querychat will generate and display the chart inline:

![](../reference/figures/viz-bar-chart.png){alt="Bar chart showing average body mass by penguin species." class="shadow rounded mb-3"}

## Choosing tools

The `tools` parameter controls which capabilities the LLM has access to.
By default, only `"query"` and `"update"` are enabled — visualization must be opted into explicitly.

To enable only query and visualization (no dashboard filtering):

```r
qc <- QueryChat$new(
  penguins,
  tools = c("query", "visualize")
)
```

See [Tools](tools.html) for a full reference on available tools and what each one does.

## Custom apps

The example below shows a custom Shiny app with visualization enabled.

```r
library(shiny)
library(bslib)
library(querychat)
library(palmerpenguins)

qc <- QueryChat$new(
  penguins,
  tools = c("update", "query", "visualize"),
  data_description = paste(
    "The Palmer Penguins dataset contains measurements of bill",
    "dimensions, flipper length, body mass, sex, and species",
    "(Adelie, Chinstrap, and Gentoo) collected from three islands in",
    "the Palmer Archipelago, Antarctica."
  )
)

ui <- page_sidebar(
  title = "querychat viz demo",
  sidebar = qc$sidebar(width = 400),
  card(
    full_screen = TRUE,
    card_header("Data"),
    DT::DTOutput("dt")
  )
)

server <- function(input, output, session) {
  qc_vals <- qc$server()
  output$dt <- DT::renderDT(qc_vals$df(), fillContainer = TRUE)
}

shinyApp(ui, server)
```

## What you can ask for

querychat can generate a wide range of chart types.
Some example prompts:

- "Show me a bar chart of body mass by species"
- "Scatter plot of bill length vs flipper length, colored by species"
- "Line chart of average body mass over time"
- "Histogram of bill depths"
- "Facet flipper length by island and species"

The LLM chooses an appropriate chart type based on your question, but you can always be specific.
If you ask for a bar chart, you'll get a bar chart.

![](../reference/figures/viz-scatter.png){alt="Scatter plot of bill length vs flipper length colored by species." class="shadow rounded mb-3"}

::: {.alert .alert-info}
If you don't like the chart, ask the LLM to adjust it — for example, "make the dots bigger" or "use a log scale on the y-axis".
:::

## Chart controls

Each chart has controls in its footer:

**Fullscreen** — Click the expand icon to view the chart in fullscreen mode.

![](../reference/figures/viz-fullscreen.png){alt="A chart displayed in fullscreen mode." class="shadow rounded mb-3"}

**Save** — Download the chart as a PNG or SVG file.

**Show Query** — Expand the footer to see the ggsql query used to generate the chart.

![](../reference/figures/viz-show-query.png){alt="A chart with the Show Query footer expanded, showing the ggsql query." class="shadow rounded mb-3"}

## How it works

1. **The LLM generates a ggsql query** — a SQL-like grammar that describes both data transformation and visual encoding in a single statement.
2. **The SQL is executed** — querychat runs the data portion of the query against your data source locally.
3. **The VISUALISE clause is rendered** — the result is passed to a ggsql reader, which produces an interactive chart.
4. **The chart appears inline** — the chart is rendered in the conversation as an interactive widget.

Note that visualization queries are independent of any active dashboard filter set by the `update` tool.
They always run against the full dataset.

Learn more about the ggsql grammar at [ggsql.org](https://ggsql.org/).

## See also

- [Tools](tools.html) — Understand what querychat can do under the hood
- [Provide context](context.html) — Help the LLM understand your data better

---
title: "Models"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

Under the hood, `querychat` is powered by [ellmer](https://ellmer.tidyverse.org/), a library for building chat-based applications with large language models (LLMs). `ellmer` supports a wide range of LLM providers -- [see here](https://ellmer.tidyverse.org/reference/index.html#chat-constructors) for a full list.

```{r}
library(querychat)
library(palmerpenguins)
library(ellmer)
```

## Specify a model

To use a particular model, pass a `"{provider}/{model}"` string to the `client` parameter, which gets passed along to `ellmer::chat()`:

```{r}
qc <- querychat(penguins, client = "anthropic/claude-sonnet-4-5")
qc$app()  # Launch the app
```

And, if you'd like to effectively set a new default model, you can use the `querychat.client` R option or the `QUERYCHAT_CLIENT` environment variable.

```{r}
# In your .Rprofile
options(querychat.client = "anthropic/claude-sonnet-4-5")
```

Note that it can also be useful to pass a full `Chat` object to the `client` parameter for more advanced use cases (e.g., custom parameters, tools, etc):

```{r}
client <- chat_anthropic(model = "claude-sonnet-4-5")
qc <- querychat(penguins, client = client)
qc$app()  # Launch the app
```

## Credentials

Most models require an API key or some other form of authentication. See the reference page for the relevant model provider (e.g., [`chat_anthropic()`](https://ellmer.tidyverse.org/reference/chat_anthropic.html)) to learn more on how to set up credentials.

::: {.alert .alert-info}
**GitHub model marketplace**

If you are already setup with GitHub credentials, [GitHub model marketplace](https://github.com/marketplace/models) provides a free and easy way to get started. See [here](https://ellmer.tidyverse.org/reference/chat_github.html) for more details on how to get setup.

```{r}
library(ellmer)

# Just works if GITHUB_TOKEN is set in your environment
client <- chat_github(model = "gpt-4.1")
```
:::

In general, most providers will prefer credentials stored as environment variables. Common practice is to use an `.Renviron` file to manage these variables. For example, for `chat_openai()`, you might add to your `.Renviron` file:

```bash
OPENAI_API_KEY="your_api_key_here"
```

Then, you can edit your `.Renviron` file using:

```{r}
usethis::edit_r_environ()
```

## Recommended models

In theory, you could use any model that has tool calling support, but we currently recommend (as of November 2025):

- GPT-4.1 (the default)
- Claude 4.5 Sonnet
- Google Gemini 3.0

In our testing, we've found that those models strike a good balance between accuracy and latency. That said, smaller/faster models like GPT-4.1-mini or Claude Haiku 4.5 work well for most tables and are worth trying first—they're significantly cheaper and faster. You can always switch to a larger model if you find the results aren't meeting your needs. On the other end of the spectrum, reasoning models like o3-mini tend to slow down responses without providing meaningfully better results for this task.

We've also seen some decent results with frontier local models (e.g., `gpt-oss:20b`), but even if you have the compute to run the largest models, they still tend to lag behind the cloud-hosted options in terms of accuracy and speed.

::: {.alert .alert-info}
**Data privacy concerns?**

If you have data privacy concerns, consider that your org may provide access to private instances of these models with data residency guarantees. For example, Azure, AWS Bedrock, and Google Vertex AI all provide private instances of popular LLMs. You can interface with these enterprise providers by passing the right string (e.g., `"bedrock-anthropic"`) or `Chat` object (e.g., `chat_bedrock_anthropic()`) to the `client` parameter. See the [ellmer docs](https://ellmer.tidyverse.org/reference/index.html#chat-constructors) for more details.
:::

Filter and sort the dashboard data

This tool executes a {{db_type}} SQL SELECT `query` to filter or sort the data used in the dashboard.

**Returns:** A confirmation that the dashboard was updated successfully, or the error that occurred when running the SQL query. The results of the query will update the data shown in the dashboard.

**When to use:** Call this tool whenever the user requests filtering, sorting, or data manipulation on the dashboard with questions like "Show me..." or "Which records have...". This tool is appropriate for any request that involves showing a subset of the data or reordering it.

**When not to use:** Do NOT use this tool for general questions about the data that can be answered with a single value or summary statistic. For those questions, use the `querychat_query` tool instead.

**Important constraints:**

- All original schema columns must be present in the SELECT output
- Use a single SQL query. You can use CTEs but you cannot chain multiple queries
- For statistical filters (stddev, percentiles), use CTEs to calculate thresholds within the query
- Assume the user will only see the original columns in the dataset

---
title: "Greet Users"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Greet Users}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

```{r}
library(querychat)
library(palmerpenguins)
```

## Provide a greeting

When the querychat UI first appears, you will usually want it to greet the user with some basic instructions. By default, these instructions are auto-generated every time a user arrives. In a production setting with multiple users/visitors, this approach has some downsides: it's slower, uses more API tokens, and produces different results each time. Instead, you should create a greeting file and pass it when creating your `QueryChat` object:

```{r}
qc <- querychat(
  penguins,
  greeting = "greeting.md"
)
qc$app()  # Launch the app
```

You can provide suggestions to the user by using the `<span class="suggestion"> </span>` tag:

```markdown
##### Filter and sort the data
* <span class="suggestion">Show only Adelie penguins</span>
* <span class="suggestion">Filter to penguins with body mass over 4000g</span>
* <span class="suggestion">Sort by flipper length from longest to shortest</span>

##### Answer questions about the data
* <span class="suggestion">What is the average bill length by species?</span>
* <span class="suggestion">How many penguins are in each island?</span>
* <span class="suggestion">Which species has the largest average body mass?</span>
```

These suggestions appear in the greeting and automatically populate the chat text box when clicked.

## Generate a greeting

If you need help coming up with a greeting, you can use the `$generate_greeting()` method:

```{r}
library(querychat)

# Create QueryChat object with your dataset
qc <- querychat(penguins)

# Generate a greeting (this calls the LLM)
greeting_text <- qc$generate_greeting(echo = "text")
#> Hello! I'm here to help you explore and analyze the penguins dataset.
#> Here are some example prompts you can try:
#> ...

# Save it for reuse
writeLines(greeting_text, "penguins_greeting.md")
```

This approach generates a greeting once and saves it for reuse, avoiding the latency and cost of generating it for every user.

```{r}
# Then use the saved greeting in your app
querychat_app(
  penguins,
  greeting = "penguins_greeting.md"
)
```

Execute a SQL query and return the results

This tool executes a {{db_type}} SQL SELECT query against the database and returns the raw result data for analysis.

**When to use:** Call this tool whenever the user asks a question that requires data analysis, aggregation, or calculations. Use this for questions like:
- "What is the average...?"
- "How many records...?"
- "Which item has the highest/lowest...?"
- "What's the total sum of...?"
- "What percentage of ...?"

Always use SQL for counting, averaging, summing, and other calculations—NEVER attempt manual calculations on your own. Use this tool repeatedly if needed to avoid any kind of manual calculation.

**When not to use:** Do NOT use this tool for filtering or sorting the dashboard display. If the user wants to "Show me..." or "Filter to..." certain records in the dashboard, use the `querychat_update_dashboard` tool instead.

**Important guidelines:**

- Queries must be valid {{db_type}} SQL SELECT statements
- Optimize for readability over efficiency—use clear column aliases and SQL comments to explain complex logic
- Subqueries and CTEs are acceptable and encouraged for complex calculations
- After receiving results, always present the key findings in your response text — the tool result starts collapsed by default, so don't assume the user has seen the raw data
- When the result is a single value or small summary, state it directly in prose. When the result is a table that IS the answer, either use a Markdown table in your response or set `collapsed` to `false` — not both
- Do not reproduce large result sets in your response — summarize the key takeaways instead

Parameters
----------
query :
    A valid {{db_type}} SQL SELECT statement. Must follow the database schema provided in the system prompt. Use clear column aliases (e.g., 'AVG(price) AS avg_price') and include SQL comments for complex logic. Subqueries and CTEs are encouraged for readability.
collapsed :
    Optional (default: true). The result card starts collapsed by default; the user can expand it to see the query and results. Set to false when the query result table is the primary answer to the user's question and should be immediately visible without expanding.
_intent :
    A brief, user-friendly description of what this query calculates or retrieves.

Returns
-------
:
    The tabular data results from executing the SQL query. The query results will be visible to the user in the interface, so you must interpret and explain the data in natural language after receiving it.

---
title: "Provide Context"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Provide Context}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

querychat automatically gathers information about your table to help the LLM write accurate SQL queries. This includes column names and types, numerical ranges, and categorical value examples. (All of this information is provided to the LLM as part of the **system prompt** -- a string of text containing instructions and context for the LLM to consider when responding to user queries.)

Importantly, we are **not** sending your raw data to the LLM and asking it to do complicated math. The LLM only needs to understand the structure and schema of your data in order to write SQL queries.

You can get even better results by customizing the system prompt in three ways:

1. Add a [data description](#data-description) to provide more context about what the data represents
2. Add [custom instructions](#extra-instructions) to guide the LLM's behavior
3. Use a fully [custom prompt template](#custom-template) if you want complete control (useful if you want to be certain the model cannot see any literal values from your data)

```{r}
library(querychat)
library(palmerpenguins)
```

## Default prompt

For full visibility into the system prompt that querychat generates for the LLM, you can inspect the `system_prompt` field. This is useful for debugging and understanding exactly what context the LLM is using:

```{r}
qc <- querychat(penguins)
cat(qc$system_prompt)
```

By default, the system prompt contains the following components:

1. The basic set of behaviors and guidelines the LLM must follow in order for querychat to work properly, including how to use [tools](tools.html) to execute queries and update the app.
2. The SQL schema of the data frame you provided. This includes:
    - Column names
    - Data types (integer, real, boolean, date/datetime, text)
    - For text columns with less than 10 unique values, we assume they are categorical variables and include the list of values
    - For integer and real columns, we include the range
3. A [data description](#data-description) (if provided via `data_description`)
4. [Additional instructions](#additional-instructions) you want to use to guide querychat's behavior (if provided via `extra_instructions`).

## Data description {#data-description}

If your column names are descriptive, querychat may already work well without additional context. However, if your columns are named `x`, `V1`, `value`, etc., you should provide a data description. Use the `data_description` parameter for this:

```{r}
qc <- querychat(
  penguins,
  data_description = "data_description.md"
)

cat(qc$system_prompt)
```

querychat doesn't need this information in any particular format -- just provide what a human would find helpful:

```markdown
<!-- data_description.md -->

This dataset contains information about Palmer Archipelago penguins,
collected for studying penguin populations.

- species: Penguin species (Adelie, Chinstrap, Gentoo)
- island: Island where observed (Torgersen, Biscoe, Dream)
- bill_length_mm: Bill length in millimeters
- bill_depth_mm: Bill depth in millimeters
- flipper_length_mm: Flipper length in millimeters
- body_mass_g: Body mass in grams
- sex: Penguin sex (male, female)
- year: Year of observation
```

## Additional instructions {#extra-instructions}

You can add custom instructions to guide the LLM's behavior using the `extra_instructions` parameter:

```{r}
qc <- querychat(
  penguins,
  extra_instructions = "instructions.md"
)

cat(qc$system_prompt)
```

Or as a string:

```{r}
instructions <- "
- Use British spelling conventions
- Stay on topic and only discuss the data dashboard
- Refuse to answer unrelated questions
"

qc <- querychat(
  penguins,
  extra_instructions = instructions
)

cat(qc$system_prompt)
```

::: {.alert .alert-warning}
LLMs may not always follow your instructions perfectly. Test extensively when changing instructions or models.
:::

## Custom template {#custom-template}

If you want more control over the system prompt, you can provide a custom prompt template using the `prompt_template` parameter. This is for more advanced users who want to fully customize the LLM's behavior. See the [QueryChat reference](../reference/QueryChat.html) for details on the available template variables.

---
title: "Build an App"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Build an App}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

While `querychat_app()` provides a quick way to start exploring data, building bespoke Shiny apps with querychat unlocks the full power of integrating natural language data exploration with custom visualizations, layouts, and interactivity. This guide shows you how to integrate querychat into your own Shiny applications and leverage its reactive data outputs to create rich, interactive dashboards.

querychat is a particularly good fit for Shiny apps that have:

1. **A single data source** (or a set of related tables that can be joined)
2. **Multiple filters** that let users slice and explore the data in different ways
3. **Several visualizations and outputs** that all depend on the same filtered data

In these apps, querychat can replace or augment your filtering UI by allowing users to describe what they want to see in natural language. Instead of building complex filter controls, users can simply ask questions like "show me customers from California who spent over $1000 last quarter" and querychat will generate the appropriate SQL query.

This is especially valuable when:

- Your data has many columns and building a UI for all possible filters would be overwhelming
- Users want to explore ad-hoc combinations of filters that you didn't anticipate
- You want to make data exploration more accessible to users who aren't comfortable with traditional filtering UIs

If you have an existing app with a reactive data frame that flows through multiple outputs, querychat can be a natural addition to provide an alternative way to filter that data.


## Starter template

Integrating querychat into a Shiny app requires just three steps:

1. Initialize a `QueryChat` instance with your data
2. Add the UI component (either `$sidebar()` or `$ui()`)
3. Use reactive values like `$df()`, `$sql()`, and `$title()` to build outputs that respond to user queries

Here's a starter template demonstrating these steps:

```{r}
library(shiny)
library(bslib)
library(querychat)
library(DT)
library(palmerpenguins)

# Step 1: Initialize QueryChat
qc <- QueryChat$new(penguins)

# Step 2: Add UI component
ui <- page_sidebar(
  sidebar = qc$sidebar(),
  card(
    card_header("Data Table"),
    dataTableOutput("table")
  ),
  card(
    fill = FALSE,
    card_header("SQL Query"),
    verbatimTextOutput("sql")
  )
)

# Step 3: Use reactive values in server
server <- function(input, output, session) {
  qc_vals <- qc$server()
  
  output$table <- renderDataTable({
    datatable(qc_vals$df(), fillContainer = TRUE)
  })

  output$sql <- renderText({
    qc_vals$sql() %||% "SELECT * FROM penguins"
  })
}

shinyApp(ui, server)
```

::: {.alert .alert-info}
You'll need to call the `qc$server()` method within your server function to set up querychat's reactive behavior, and capture its return value to access reactive data.
:::

## Deferred data sources {#deferred-data-sources}

Some data sources, like database connections or reactive calculations, may need to be created within an active Shiny session. To help support this, `QueryChat` allows you to initialize without a data source and provide it later, like this:

```{r}
library(shiny)
library(bslib)
library(querychat)

# Global scope - create QueryChat without data source
qc <- QueryChat$new(NULL, "users")

ui <- page_sidebar(
  sidebar = qc$sidebar(),
  card(dataTableOutput("table"))
)

server <- function(input, output, session) {
  # Server scope - create connection with session credentials
  conn <- get_user_connection(session)
  qc_vals <- qc$server(data_source = conn)

  output$table <- renderDataTable({
    qc_vals$df()
  })
}

shinyApp(ui, server)
```

If your chat client also depends on session-scoped credentials, you can defer that too by passing it to `qc$server(client = ...)` alongside the `data_source`.

This is also a useful pattern when using something like [`{pool}`](https://github.com/rstudio/pool) to efficiently manage a pool of database connections (which we strongly recommend for production apps).

## Reactives

There are three main reactive values provided by querychat for use in your app:

### Filtered data {#filtered-data}

The `$df()` method returns the current filtered and/or sorted data frame. This updates whenever the user prompts a filtering or sorting operation through the chat interface (see [Data updating](tools.html#data-updating) for details).

```{r}
qc_vals <- qc$server()

output$table <- renderDataTable({
  qc_vals$df()  # Returns filtered/sorted data
})
```

You can use `$df()` to power any output in your app - visualizations, summary statistics, data tables, and more. When a user asks to "show only Adelie penguins" or "sort by body mass", `$df()` automatically updates, and any outputs that depend on it will re-render.

### SQL query {#sql-query}

The `$sql()` method returns the current SQL query as a string. This is useful for displaying the query to users for transparency and reproducibility:

```{r}
qc_vals <- qc$server()

output$current_query <- renderText({
  qc_vals$sql() %||% "SELECT * FROM penguins"
})
```

You can also use `$sql()` as a setter to programmatically update the query (see [Programmatic filtering](#programmatic-filtering) below).

### Title {#title}

The `$title()` method returns a short description of the current filter, provided by the LLM when it generates a query. For example, if a user asks to "show Adelie penguins", the title might be "Adelie penguins".

```{r}
qc_vals <- qc$server()

output$card_title <- renderText({
  qc_vals$title() %||% "All Data"
})
```

Returns `NULL` when no filter is active. You can also use `$title()` as a setter to update the title programmatically.

## Custom UI

In the starter template above, we used the `$sidebar()` method for a simple sidebar layout. In some cases, you might want to place the chat UI somewhere else in your app layout, or just more fully customize what goes in the sidebar. The `$ui()` method is designed for this -- it returns the chat component without additional layout wrappers.

For example, you might want to create some additional controls to [reset filters](#programmatic-filtering) alongside the chat UI:

```{r}
library(querychat)
library(palmerpenguins)

qc <- QueryChat$new(penguins)

ui <- page_sidebar(
  sidebar = sidebar(
    qc$ui(),  # Chat component
    actionButton("reset", "Reset Filters", class = "w-100"),
    fillable = TRUE,
    width = 300
  ),
  # Main content here
)
```

::: {.alert .alert-info}
**Customizing chat UIs**

See `{shinychat}`'s [docs](https://posit-dev.github.io/shinychat/r/index.html) to learn more about customizing the chat UI component returned by `qc$ui()`.
:::

## Data views

Thanks to Shiny's support for interactive visualizations with packages like [plotly](https://plotly.com/r/), it's straightforward to create rich data views that depend on QueryChat data. Here's an example of an app showing both the filtered data and a bar chart depending on that same data:

<details>
<summary> <code>app.R </code> </summary>

```{r}
library(shiny)
library(bslib)
library(querychat)
library(DT)
library(plotly)
library(palmerpenguins)

qc <- QueryChat$new(penguins, client = "claude/claude-sonnet-4-5")

ui <- page_sidebar(
  sidebar = qc$sidebar(),
  card(
    card_header("Data Table"),
    dataTableOutput("table")
  ),
  card(
    card_header("Body Mass by Species"),
    plotlyOutput("mass_plot")
  )
)

server <- function(input, output, session) {
  qc_vals <- qc$server()

  output$table <- renderDataTable({
    datatable(qc_vals$df(), fillContainer = TRUE)
  })

  output$mass_plot <- renderPlotly({
    ggplot(qc_vals$df(), aes(x = body_mass_g, fill = species)) +
      geom_density(alpha = 0.4) +
      theme_minimal()
  })
}

shinyApp(ui, server)
```

</details>

![](images/plotly-data-view.png){alt="Screenshot of a querychat app showing both a data table and a density plot of body mass by species" class="shadow rounded mb-3"}

A more useful, but slightly more involved example like the one below might incorporate other Shiny components like value boxes to summarize key statistics about the filtered data.

<details>
<summary> <code>app.R </code> </summary>

```{r}
library(shiny)
library(bslib)
library(DT)
library(plotly)
library(palmerpenguins)
library(dplyr)
library(bsicons)
library(querychat)


qc <- QueryChat$new(penguins)

ui <- page_sidebar(
  title = "Palmer Penguins Analysis",
  class = "bslib-page-dashboard",
  sidebar = qc$sidebar(),
  layout_column_wrap(
    width = 1 / 3,
    fill = FALSE,
    value_box(
      title = "Total Penguins",
      value = textOutput("count"),
      showcase = bs_icon("piggy-bank"),
      theme = "primary"
    ),
    value_box(
      title = "Species Count",
      value = textOutput("species_count"),
      showcase = bs_icon("bookmark-star"),
      theme = "success"
    ),
    value_box(
      title = "Avg Body Mass",
      value = textOutput("avg_mass"),
      showcase = bs_icon("speedometer"),
      theme = "info"
    )
  ),
  layout_columns(
    card(
      card_header(textOutput("table_title")),
      DT::dataTableOutput("data_table")
    ),
    card(
      card_header("Species Distribution"),
      plotlyOutput("species_plot")
    )
  ),
  layout_columns(
    card(
      card_header("Bill Length Distribution"),
      plotlyOutput("bill_length_dist")
    ),
    card(
      card_header("Body Mass by Species"),
      plotlyOutput("mass_by_species")
    )
  )
)

server <- function(input, output, session) {
  qc_vals <- qc$server()

  output$count <- renderText({
    nrow(qc_vals$df())
  })

  output$species_count <- renderText({
    length(unique(qc_vals$df()$species))
  })

  output$avg_mass <- renderText({
    avg <- mean(qc_vals$df()$body_mass_g, na.rm = TRUE)
    paste0(round(avg, 0), "g")
  })

  output$table_title <- renderText({
    qc_vals$title() %||% "All Penguins"
  })

  output$data_table <- DT::renderDataTable({
    DT::datatable(
      qc_vals$df(),
      fillContainer = TRUE,
      options = list(
        scrollX = TRUE,
        pageLength = 10,
        dom = "ti"
      )
    )
  })

  output$species_plot <- renderPlotly({
    plot_ly(
      count(qc_vals$df(), species),
      x = ~species,
      y = ~n,
      type = "bar",
      marker = list(color = c("#1f77b4", "#ff7f0e", "#2ca02c"))
    )
  })

  output$bill_length_dist <- renderPlotly({
    plot_ly(
      qc_vals$df(),
      x = ~bill_length_mm,
      type = "histogram",
      nbinsx = 30,
      marker = list(color = "#1f77b4", opacity = 0.7)
    )
  })

  output$mass_by_species <- renderPlotly({
    plot_ly(
      qc_vals$df(),
      x = ~species,
      y = ~body_mass_g,
      color = ~sex,
      type = "box",
      colors = c("#1f77b4", "#ff7f0e")
    )
  })
}

shinyApp(ui = ui, server = server)
```

</details>


## Programmatic updates {#programmatic-filtering}

querychat's reactive state can be updated programmatically. For example, you might want to add a "Reset Filters" button that clears any active filters and returns the data table to its original state. You can do this by setting both the SQL query and title to their default values. This way you don't have to rely on both the user and LLM to send the right prompt.

```{r}
ui <- page_sidebar(
  sidebar = sidebar(
    qc$ui(),
    hr(),
    actionButton("reset", "Reset Filters")
  ),
  # Main content
  card(dataTableOutput("table"))
)

server <- function(input, output, session) {
  qc_vals <- qc$server()

  output$table <- renderDataTable({
    qc_vals$df()
  })

  observeEvent(input$reset, {
    qc_vals$sql("")
    qc_vals$title(NULL)
  })
}

shinyApp(ui, server)
```

This is equivalent to the user asking the LLM to "reset" or "show all data".


## Multiple tables

Currently, you have two options for exploring multiple tables in querychat:

1. Join the tables into a single table before passing to querychat
2. Use multiple querychat instances in the same app

The first option makes it possible to chat with multiple tables inside a single chat interface, whereas the second option requires a separate chat interface for each table.

::: {.alert .alert-info}
### Multiple filtered tables

We do intend on supporting multiple filtered tables in a future release -- if you're interested in this feature, please upvote [the relevant issue](https://github.com/posit-dev/querychat/issues/6)
:::

<details>
<summary> <code>app.R </code> </summary>

```{r}
library(shiny)
library(bslib)
library(palmerpenguins)
library(titanic)
library(querychat)

qc_penguins <- QueryChat$new(penguins)
qc_titanic <- QueryChat$new(titanic_train)

ui <- page_navbar(
  title = "Multiple Datasets",
  sidebar = sidebar(
    id = "sidebar",
    conditionalPanel(
      "input.navbar == 'Penguins'",
      qc_penguins$ui()
    ),
    conditionalPanel(
      "input.navbar == 'Titanic'",
      qc_titanic$ui()
    )
  ),
  nav_panel(
    "Penguins",
    card(dataTableOutput("penguins_table"))
  ),
  nav_panel(
    "Titanic",
    card(dataTableOutput("titanic_table"))
  ),
  id = "navbar"
)

server <- function(input, output, session) {
  qc_penguins_vals <- qc_penguins$server()
  qc_titanic_vals <- qc_titanic$server()

  output$penguins_table <- renderDataTable({
    qc_penguins_vals$df()
  })

  output$titanic_table <- renderDataTable({
    qc_titanic_vals$df()
  })
}

shinyApp(ui, server)
```

</details>

## See also

- [Greet users](greet.html) - Create welcoming onboarding experiences
- [Provide context](context.html) - Help the LLM understand your data better
- [Tools](tools.html) - Understand what querychat can do under the hood

# tool_update_dashboard() checks inputs

    Code
      tool_update_dashboard("foo")
    Condition
      Error in `tool_update_dashboard()`:
      ! `data_source` must be a <DataSource> object, not a string.

---

    Code
      tool_update_dashboard(df_source, update_fn = NULL)
    Condition
      Error in `tool_update_dashboard()`:
      ! `update_fn` must be a function, not `NULL`.
    Code
      tool_update_dashboard(df_source, update_fn = function(query) { })
    Condition
      Error in `tool_update_dashboard()`:
      ! `update_fn` must accept at least two named arguments: "query" and "title".
      x "title" argument was missing.
    Code
      tool_update_dashboard(df_source, update_fn = function(title, extra) { })
    Condition
      Error in `tool_update_dashboard()`:
      ! `update_fn` must accept at least two named arguments: "query" and "title".
      x "query" argument was missing.

# tool_reset_dashboard() checks inputs

    Code
      tool_reset_dashboard("not_a_function")
    Condition
      Error in `tool_reset_dashboard()`:
      ! `reset_fn` must be a function, not the string "not_a_function".

# tool_query() checks inputs

    Code
      tool_query("invalid_source")
    Condition
      Error in `tool_query()`:
      ! `data_source` must be a <DataSource> object, not a string.

# DBISource$new() / errors with non-DBI connection

    Code
      DBISource$new(list(fake = "connection"), "test_table")
    Condition
      Error in `initialize()`:
      ! `conn` must be a <DBIConnection>, not a list

---

    Code
      DBISource$new(NULL, "test_table")
    Condition
      Error in `initialize()`:
      ! `conn` must be a <DBIConnection>, not NULL

---

    Code
      DBISource$new("not a connection", "test_table")
    Condition
      Error in `initialize()`:
      ! `conn` must be a <DBIConnection>, not a string

# DBISource$new() / errors with invalid table_name types

    Code
      DBISource$new(db$conn, 123)
    Condition
      Error in `initialize()`:
      ! `table_name` must be a single character string or a `DBI::Id()` object

---

    Code
      DBISource$new(db$conn, c("table1", "table2"))
    Condition
      Error in `initialize()`:
      ! `table_name` must be a single character string or a `DBI::Id()` object

---

    Code
      DBISource$new(db$conn, list(name = "table"))
    Condition
      Error in `initialize()`:
      ! `table_name` must be a single character string or a `DBI::Id()` object

# DBISource$new() / errors when table does not exist

    Code
      DBISource$new(db$conn, "non_existent_table")
    Condition
      Error in `initialize()`:
      ! Table "`non_existent_table`" not found in database
      i If you're using a table in a catalog or schema, pass a `DBI::Id()` object to `table_name`

# DataSource base class / throws not_implemented_error for all abstract methods

    Code
      base_source$get_db_type()
    Condition
      Error in `base_source$get_db_type()`:
      ! `get_db_type()` must be implemented by subclass

---

    Code
      base_source$get_schema()
    Condition
      Error in `base_source$get_schema()`:
      ! `get_schema()` must be implemented by subclass

---

    Code
      base_source$execute_query("SELECT * FROM test")
    Condition
      Error in `base_source$execute_query()`:
      ! `execute_query()` must be implemented by subclass

---

    Code
      base_source$test_query("SELECT * FROM test LIMIT 1")
    Condition
      Error in `base_source$test_query()`:
      ! `test_query()` must be implemented by subclass

---

    Code
      base_source$get_data()
    Condition
      Error in `base_source$get_data()`:
      ! `get_data()` must be implemented by subclass

---

    Code
      base_source$cleanup()
    Condition
      Error in `base_source$cleanup()`:
      ! `cleanup()` must be implemented by subclass

# test_query() column validation / provides helpful error message listing missing columns

    Code
      source$test_query("SELECT id FROM test_table", require_all_columns = TRUE)
    Condition
      Error in `source$test_query()`:
      ! Query result missing required columns: 'name', 'value'
      i The query must return all original table columns (in any order).

---

    Code
      source$test_query("SELECT id, name FROM test_table", require_all_columns = TRUE)
    Condition
      Error in `source$test_query()`:
      ! Query result missing required columns: 'value'
      i The query must return all original table columns (in any order).

# QueryChat$new() / errors with invalid argument types

    Code
      QueryChat$new(test_df, table_name = "test", id = 123)
    Condition
      Error in `initialize()`:
      ! `id` must be a single string or `NULL`, not the number 123.

---

    Code
      QueryChat$new(test_df, table_name = "test", greeting = 123)
    Condition
      Error in `initialize()`:
      ! `greeting` must be a single string or `NULL`, not the number 123.

---

    Code
      QueryChat$new(test_df, table_name = "test", categorical_threshold = "not_a_number")
    Condition
      Error in `initialize()`:
      ! `categorical_threshold` must be a whole number, not the string "not_a_number".

---

    Code
      QueryChat$new(test_df, table_name = "test", cleanup = "not_logical")
    Condition
      Error in `initialize()`:
      ! `cleanup` must be `TRUE`, `FALSE`, or `NA`, not the string "not_logical".

# QueryChat$server() errors when called outside Shiny context

    Code
      qc$server()
    Condition
      Error in `qc$server()`:
      ! `$server()` must be called within a Shiny server function

# normalize_data_source() / errors with invalid data source types

    Code
      normalize_data_source("not_a_data_source", "table_name")
    Condition
      Error in `normalize_data_source()`:
      ! `data_source` must be a <DataSource>, <data.frame>, or <DBIConnection>, not a string.

---

    Code
      normalize_data_source(list(a = 1, b = 2), "table_name")
    Condition
      Error in `normalize_data_source()`:
      ! `data_source` must be a <DataSource>, <data.frame>, or <DBIConnection>, not a list.

---

    Code
      normalize_data_source(NULL, "table_name")
    Condition
      Error in `normalize_data_source()`:
      ! `data_source` must be a <DataSource>, <data.frame>, or <DBIConnection>, not NULL.

# DataFrameSource$new() / errors with non-data.frame input

    Code
      DataFrameSource$new(list(a = 1, b = 2), "test_table")
    Condition
      Error in `initialize()`:
      ! `df` must be a data frame, not a list.

---

    Code
      DataFrameSource$new(c(1, 2, 3), "test_table")
    Condition
      Error in `initialize()`:
      ! `df` must be a data frame, not a double vector.

---

    Code
      DataFrameSource$new(NULL, "test_table")
    Condition
      Error in `initialize()`:
      ! `df` must be a data frame, not `NULL`.

# DataFrameSource$new() / errors with invalid table names

    Code
      DataFrameSource$new(test_df, "123_invalid")
    Condition
      Error in `initialize()`:
      ! `table_name` must be a valid SQL table name
      i Table names must begin with a letter and contain only letters, numbers, and underscores
      x You provided: "123_invalid"
    Code
      DataFrameSource$new(test_df, "table-name")
    Condition
      Error in `initialize()`:
      ! `table_name` must be a valid SQL table name
      i Table names must begin with a letter and contain only letters, numbers, and underscores
      x You provided: "table-name"
    Code
      DataFrameSource$new(test_df, "table name")
    Condition
      Error in `initialize()`:
      ! `table_name` must be a valid SQL table name
      i Table names must begin with a letter and contain only letters, numbers, and underscores
      x You provided: "table name"
    Code
      DataFrameSource$new(test_df, "")
    Condition
      Error in `initialize()`:
      ! `table_name` must be a valid SQL table name
      i Table names must begin with a letter and contain only letters, numbers, and underscores
      x You provided: ""
    Code
      DataFrameSource$new(test_df, NULL)
    Condition
      Error in `initialize()`:
      ! `table_name` must be a single string, not `NULL`.

# DataFrameSource engine parameter / engine parameter validation / errors on invalid engine name

    Code
      DataFrameSource$new(new_test_df(), "test_table", engine = "postgres")
    Condition
      Error in `initialize()`:
      ! `engine` must be one of "duckdb" or "sqlite", not "postgres".

---

    Code
      DataFrameSource$new(new_test_df(), "test_table", engine = "invalid")
    Condition
      Error in `initialize()`:
      ! `engine` must be one of "duckdb" or "sqlite", not "invalid".

---

    Code
      DataFrameSource$new(new_test_df(), "test_table", engine = "")
    Condition
      Error in `initialize()`:
      ! `engine` must be one of "duckdb" or "sqlite", not "".

# TblSqlSource$new() / errors with non-tbl_sql input

    Code
      TblSqlSource$new(data.frame(a = 1))
    Condition
      Error in `initialize()`:
      ! `tbl` must be a SQL tibble connected to a database, not a data frame

---

    Code
      TblSqlSource$new(list(a = 1, b = 2))
    Condition
      Error in `initialize()`:
      ! `tbl` must be a SQL tibble connected to a database, not a list

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DataSource.R
\name{DataSource}
\alias{DataSource}
\title{Data Source Base Class}
\description{
An abstract R6 class defining the interface that custom QueryChat data
sources must implement. This class should not be instantiated directly;
instead, use one of its concrete implementations like \link{DataFrameSource} or
\link{DBISource}.
}
\examples{
MyDataSource <- R6::R6Class(
  "MyDataSource",
  inherit = DataSource,
  public = list(
    initialize = function(table_name) {
      self$table_name <- table_name
    }
    # Implement abstract methods here...
  )
)

}
\section{Public fields}{
  \if{html}{\out{<div class="r6-fields">}}
  \describe{
    \item{\code{table_name}}{Name of the table to be used in SQL queries}
  }
  \if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
  \itemize{
    \item \href{#method-DataSource-get_db_type}{\code{DataSource$get_db_type()}}
    \item \href{#method-DataSource-get_schema}{\code{DataSource$get_schema()}}
    \item \href{#method-DataSource-execute_query}{\code{DataSource$execute_query()}}
    \item \href{#method-DataSource-test_query}{\code{DataSource$test_query()}}
    \item \href{#method-DataSource-get_data}{\code{DataSource$get_data()}}
    \item \href{#method-DataSource-cleanup}{\code{DataSource$cleanup()}}
    \item \href{#method-DataSource-clone}{\code{DataSource$clone()}}
  }
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataSource-get_db_type"></a>}}
\if{latex}{\out{\hypertarget{method-DataSource-get_db_type}{}}}
\subsection{\code{DataSource$get_db_type()}}{
  Get the database type
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DataSource$get_db_type()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A string describing the database type (e.g., "DuckDB", "SQLite")
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataSource-get_schema"></a>}}
\if{latex}{\out{\hypertarget{method-DataSource-get_schema}{}}}
\subsection{\code{DataSource$get_schema()}}{
  Get schema information about the table
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DataSource$get_schema(categorical_threshold = 20)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{categorical_threshold}}{Maximum number of unique values for a text
column to be considered categorical}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A string containing schema information formatted for LLM prompts
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataSource-execute_query"></a>}}
\if{latex}{\out{\hypertarget{method-DataSource-execute_query}{}}}
\subsection{\code{DataSource$execute_query()}}{
  Execute a SQL query and return results
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DataSource$execute_query(query)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{query}}{SQL query string to execute}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A data frame containing query results
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataSource-test_query"></a>}}
\if{latex}{\out{\hypertarget{method-DataSource-test_query}{}}}
\subsection{\code{DataSource$test_query()}}{
  Test a SQL query by fetching only one row
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DataSource$test_query(query, require_all_columns = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{query}}{SQL query string to test}
      \item{\code{require_all_columns}}{If \code{TRUE}, validates that the result includes
all original table columns (default: \code{FALSE})}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A data frame containing one row of results (or empty if no
matches)
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataSource-get_data"></a>}}
\if{latex}{\out{\hypertarget{method-DataSource-get_data}{}}}
\subsection{\code{DataSource$get_data()}}{
  Get the unfiltered data as a data frame
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DataSource$get_data()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A data frame containing all data from the table
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataSource-cleanup"></a>}}
\if{latex}{\out{\hypertarget{method-DataSource-cleanup}{}}}
\subsection{\code{DataSource$cleanup()}}{
  Clean up resources (close connections, etc.)
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DataSource$cleanup()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    NULL (invisibly)
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataSource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-DataSource-clone}{}}}
\subsection{\code{DataSource$clone()}}{
  The objects of this class are cloneable with this method.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DataSource$clone(deep = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{deep}}{Whether to make a deep clone.}
    }
    \if{html}{\out{</div>}}
  }
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/querychat-package.R
\docType{package}
\name{querychat-package}
\alias{querychat-package}
\title{querychat: Chat with Your Data Using Natural Language}
\description{
querychat provides an interactive chat interface for querying data using
natural language. It translates your questions into SQL queries, executes
them against your data, and displays the results. The package works with
both data frames and database connections.
}
\section{Quick Start}{

The easiest way to get started is with the \link{QueryChat} R6 class:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(querychat)

# Create a QueryChat object (table name inferred from variable)
qc <- QueryChat$new(mtcars)

# Option 1: Run a complete app with sensible defaults
qc$app()

# Option 2: Build a custom Shiny app
ui <- page_sidebar(
  qc$sidebar(),
  dataTableOutput("data")
)

server <- function(input, output, session) \{
  qc$server()
  output$data <- renderDataTable(qc$df())
\}

shinyApp(ui, server)
}\if{html}{\out{</div>}}
}

\section{Key Features}{

\itemize{
\item \strong{Natural language queries}: Ask questions in plain English
\item \strong{SQL transparency}: See the generated SQL queries
\item \strong{Multiple data sources}: Works with data frames and database connections
\item \strong{Customizable}: Add data descriptions, extra instructions, and custom greetings
\item \strong{LLM agnostic}: Works with OpenAI, Anthropic, Google, and other providers via ellmer
}
}

\section{Main Components}{

\itemize{
\item \link{QueryChat}: The main R6 class for creating chat interfaces
\item \link{DataSource}, \link{DataFrameSource}, \link{DBISource}: R6 classes for data sources
}
}

\section{Examples}{

To see examples included with the package, run:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{shiny::runExample(package = "querychat")
}\if{html}{\out{</div>}}

This provides a list of available examples. To run a specific example, like
'01-hello-app', use:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{shiny::runExample("01-hello-app", package = "querychat")
}\if{html}{\out{</div>}}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://posit-dev.github.io/querychat/r/}
  \item \url{https://posit-dev.github.io/querychat/}
  \item \url{https://github.com/posit-dev/querychat}
  \item Report bugs at \url{https://github.com/posit-dev/querychat/issues}
}

}
\author{
\strong{Maintainer}: Garrick Aden-Buie \email{garrick@posit.co} (\href{https://orcid.org/0000-0002-7111-0077}{ORCID})

Authors:
\itemize{
  \item Garrick Aden-Buie \email{garrick@posit.co} (\href{https://orcid.org/0000-0002-7111-0077}{ORCID})
  \item Joe Cheng \email{joe@posit.co} [conceptor]
  \item Carson Sievert \email{carson@posit.co} (\href{https://orcid.org/0000-0002-4958-2844}{ORCID})
}

Other contributors:
\itemize{
  \item Posit Software, PBC [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/querychat_viz.R
\name{has_layer_level_source}
\alias{has_layer_level_source}
\title{Detect whether a VISUALISE string has a layer-level FROM source}
\usage{
has_layer_level_source(visual)
}
\arguments{
\item{visual}{A ggsql VISUALISE string.}
}
\value{
\code{TRUE} if any DRAW clause contains a MAPPING ... FROM source.
}
\description{
Returns \code{TRUE} when a DRAW clause defines its own \verb{FROM <source>} via a
MAPPING sub-clause. Querychat replays VISUALISE against a single local
relation, so layer-specific sources cannot be preserved reliably.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/QueryChat.R
\name{QueryChat}
\alias{QueryChat}
\title{QueryChat: Interactive Data Querying with Natural Language}
\description{
\code{QueryChat} is an R6 class built on Shiny, shinychat, and ellmer to enable
interactive querying of data using natural language. It leverages large
language models (LLMs) to translate user questions into SQL queries, execute
them against a data source (data frame or database), and various ways of
accessing/displaying the results.

The \code{QueryChat} class takes your data (a data frame or database connection)
as input and provides methods to:
\itemize{
\item Generate a chat UI for natural language queries (e.g., \verb{$app()},
\verb{$sidebar()})
\item Initialize server logic that returns session-specific reactive values (via
\verb{$server()})
\item Access reactive data, SQL queries, and titles through the returned server
values
}
}
\section{Usage in Shiny Apps}{


\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(querychat)

# Create a QueryChat object
qc <- QueryChat$new(mtcars)

# Quick start: run a complete app
qc$app()

# Or build a custom Shiny app
ui <- page_sidebar(
  qc$sidebar(),
  verbatimTextOutput("sql"),
  dataTableOutput("data")
)

server <- function(input, output, session) \{
  qc_vals <- qc$server()

  output$sql <- renderText(qc_vals$sql())
  output$data <- renderDataTable(qc_vals$df())
\}

shinyApp(ui, server)
}\if{html}{\out{</div>}}
}

\examples{
\dontshow{if (rlang::is_installed("duckdb") || rlang::is_installed("RSQLite")) withAutoprint(\{ # examplesIf}
# Basic usage with a data frame
qc <- QueryChat$new(mtcars)
\dontrun{
app <- qc$app()
}

# With a custom greeting
greeting <- "Welcome! Ask me about the mtcars dataset."
qc <- QueryChat$new(mtcars, greeting = greeting)

# With a specific LLM provider
qc <- QueryChat$new(mtcars, client = "anthropic/claude-sonnet-4-5")

# Generate a greeting for reuse (requires internet/API access)
\dontrun{
qc <- QueryChat$new(mtcars)
greeting <- qc$generate_greeting(echo = "text")
# Save greeting for next time
writeLines(greeting, "mtcars_greeting.md")
}

# Or specify greeting and additional options at initialization
qc <- QueryChat$new(
  mtcars,
  greeting = "Welcome to the mtcars explorer!",
  client = "openai/gpt-4o",
  data_description = "Motor Trend car road tests dataset"
)
\dontshow{\}) # examplesIf}
\dontshow{if (rlang::is_installed("RSQLite")) withAutoprint(\{ # examplesIf}
# Create a QueryChat object from a database connection
# 1. Set up the database connection
con <- DBI::dbConnect(RSQLite::SQLite(), ":memory:")

# 2. (For this demo) Create a table in the database
DBI::dbWriteTable(con, "mtcars", mtcars)

# 3. Pass the connection and table name to `QueryChat`
qc <- QueryChat$new(con, "mtcars")
\dontshow{\}) # examplesIf}
}
\section{Public fields}{
  \if{html}{\out{<div class="r6-fields">}}
  \describe{
    \item{\code{greeting}}{The greeting message displayed to users.}

    \item{\code{id}}{ID for the QueryChat instance.}

    \item{\code{tools}}{The allowed tools for the chat client.}
  }
  \if{html}{\out{</div>}}
}
\section{Active bindings}{
  \if{html}{\out{<div class="r6-active-bindings">}}
  \describe{
    \item{\code{system_prompt}}{Get the system prompt.}

    \item{\code{data_source}}{Get or set the current data source. When setting,
the value is normalized and the system prompt is rebuilt.}
  }
  \if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
  \itemize{
    \item \href{#method-QueryChat-initialize}{\code{QueryChat$new()}}
    \item \href{#method-QueryChat-client}{\code{QueryChat$client()}}
    \item \href{#method-QueryChat-console}{\code{QueryChat$console()}}
    \item \href{#method-QueryChat-app}{\code{QueryChat$app()}}
    \item \href{#method-QueryChat-app_obj}{\code{QueryChat$app_obj()}}
    \item \href{#method-QueryChat-sidebar}{\code{QueryChat$sidebar()}}
    \item \href{#method-QueryChat-ui}{\code{QueryChat$ui()}}
    \item \href{#method-QueryChat-server}{\code{QueryChat$server()}}
    \item \href{#method-QueryChat-generate_greeting}{\code{QueryChat$generate_greeting()}}
    \item \href{#method-QueryChat-cleanup}{\code{QueryChat$cleanup()}}
    \item \href{#method-QueryChat-clone}{\code{QueryChat$clone()}}
  }
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-QueryChat-initialize"></a>}}
\if{latex}{\out{\hypertarget{method-QueryChat-initialize}{}}}
\subsection{\code{QueryChat$new()}}{
  Create a new QueryChat object.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{QueryChat$new(
  data_source,
  table_name = missing_arg(),
  ...,
  id = NULL,
  greeting = NULL,
  client = NULL,
  tools = c("filter", "query"),
  data_description = NULL,
  categorical_threshold = 20,
  extra_instructions = NULL,
  prompt_template = NULL,
  cleanup = NA
)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{data_source}}{Either a data.frame, a database connection (e.g., DBI
connection), or \code{NULL} to defer setting the data source until later.
When \code{NULL}, the data source must be set via the \verb{$data_source} property
or passed to \verb{$server()} before calling methods that require data access.}
      \item{\code{table_name}}{A string specifying the table name to use in SQL
queries. If \code{data_source} is a data.frame, this is the name to refer to
it by in queries (typically the variable name). If not provided, will
be inferred from the variable name for data.frame inputs. For database
connections or \code{NULL} data sources, this parameter is required.}
      \item{\code{...}}{Additional arguments (currently unused).}
      \item{\code{id}}{Optional module ID for the QueryChat instance. If not provided,
will be auto-generated from \code{table_name}. The ID is used to namespace
the Shiny module.}
      \item{\code{greeting}}{Optional initial message to display to users. Can be a
character string (in Markdown format) or a file path. If not provided,
a greeting will be generated at the start of each conversation using
the LLM, which adds latency and cost. Use \verb{$generate_greeting()} to
create a greeting to save and reuse.}
      \item{\code{client}}{Optional chat client. Can be:
\itemize{
\item An \link[ellmer:Chat]{ellmer::Chat} object
\item A string to pass to \code{\link[ellmer:chat]{ellmer::chat()}} (e.g., \code{"openai/gpt-4o"})
\item \code{NULL} (default): Uses the \code{querychat.client} option, the
\code{QUERYCHAT_CLIENT} environment variable, or defaults to
\code{\link[ellmer:chat_openai]{ellmer::chat_openai()}}
}}
      \item{\code{tools}}{Which querychat tools to include in the chat client, by
default. \code{"filter"} includes the tools for filtering and resetting the
dashboard and \code{"query"} includes the tool for executing SQL queries.
Use \code{tools = "filter"} when you only want the dashboard filtering tools,
or when you want to disable the querying tool entirely to prevent the
LLM from seeing any of the data in your dataset. The legacy name
\code{"update"} is still accepted as an alias for \code{"filter"}.}
      \item{\code{data_description}}{Optional description of the data in plain text or
Markdown. Can be a string or a file path. This provides context to the
LLM about what the data represents.}
      \item{\code{categorical_threshold}}{For text columns, the maximum number of
unique values to consider as a categorical variable. Default is 20.}
      \item{\code{extra_instructions}}{Optional additional instructions for the chat
model in plain text or Markdown. Can be a string or a file path.}
      \item{\code{prompt_template}}{Optional path to or string of a custom prompt
template file. If not provided, the default querychat template will be
used. See the package prompts directory for the default template
format.}
      \item{\code{cleanup}}{Whether or not to automatically run \verb{$cleanup()} when the
Shiny session/app stops. By default, cleanup only occurs if \code{QueryChat}
gets created within a Shiny session. Set to \code{TRUE} to always clean up,
or \code{FALSE} to never clean up automatically.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A new \code{QueryChat} object.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-QueryChat-client"></a>}}
\if{latex}{\out{\hypertarget{method-QueryChat-client}{}}}
\subsection{\code{QueryChat$client()}}{
  Create a chat client, complete with registered tools, for the current
data source.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{QueryChat$client(
  tools = NA,
  update_dashboard = function(query, title) {
 },
  reset_dashboard = function() {
 },
  visualize = function(data) {
 },
  session = NULL
)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{tools}}{Which querychat tools to include in the chat client.
\code{"filter"} includes the tools for filtering and resetting the dashboard
and \code{"query"} includes the tool for executing SQL queries. By default,
when \code{tools = NA}, the values provided at initialization are used.
The legacy name \code{"update"} is still accepted as an alias for \code{"filter"}.}
      \item{\code{update_dashboard}}{Optional function to call with the \code{query} and
\code{title} generated by the LLM for the \code{update_dashboard} tool.}
      \item{\code{reset_dashboard}}{Optional function to call when the
\code{reset_dashboard} tool is called.}
      \item{\code{visualize}}{Optional function to call with a list containing
\code{ggsql}, \code{title}, and \code{widget_id} when a visualization succeeds.}
      \item{\code{session}}{A Shiny session object. Required when \code{"visualize"} is
in \code{tools} and you want interactive chart rendering. When \code{NULL}
(the default), visualizations still execute but are not rendered
as Shiny outputs.}
    }
    \if{html}{\out{</div>}}
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-QueryChat-console"></a>}}
\if{latex}{\out{\hypertarget{method-QueryChat-console}{}}}
\subsection{\code{QueryChat$console()}}{
  Launch a console-based chat interface with the data source.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{QueryChat$console(new = FALSE, ..., tools = "query")}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{new}}{Whether to create a new chat client instance or continue the
conversation from the last console chat session (the default).}
      \item{\code{...}}{Additional arguments passed to the \verb{$client()} method.}
      \item{\code{tools}}{Which querychat tools to include in the chat client. See
\verb{$client()} for details. Ignored when not creating a new chat client.
By default, only the \code{"query"} tool is included, regardless of the
\code{tools} set at initialization.}
    }
    \if{html}{\out{</div>}}
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-QueryChat-app"></a>}}
\if{latex}{\out{\hypertarget{method-QueryChat-app}{}}}
\subsection{\code{QueryChat$app()}}{
  Create and run a Shiny gadget for chatting with data

Runs a Shiny gadget (designed for interactive use) that provides a
complete interface for chatting with your data using natural language. If
you're looking to deploy this app or run it through some other means, see
\verb{$app_obj()}.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(querychat)

qc <- QueryChat$new(mtcars)
qc$app()
}\if{html}{\out{</div>}}
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{QueryChat$app(..., bookmark_store = "url")}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{...}}{Arguments passed to \verb{$app_obj()}.}
      \item{\code{bookmark_store}}{The bookmarking storage method. Passed to
\code{\link[shiny:enableBookmarking]{shiny::enableBookmarking()}}. If \code{"url"} or \code{"server"}, the chat state
(including current query) will be bookmarked. Default is \code{"url"}.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    Invisibly returns a list of session-specific values:
\itemize{
\item \code{df}: The final filtered data frame
\item \code{sql}: The final SQL query string
\item \code{title}: The final title
\item \code{client}: The session-specific chat client instance
}
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-QueryChat-app_obj"></a>}}
\if{latex}{\out{\hypertarget{method-QueryChat-app_obj}{}}}
\subsection{\code{QueryChat$app_obj()}}{
  A streamlined Shiny app for chatting with data

Creates a Shiny app designed for chatting with data, with:
\itemize{
\item A sidebar containing the chat interface
\item A card displaying the current SQL query
\item A card displaying the filtered data table
\item A reset button to clear the query
}

\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(querychat)

qc <- QueryChat$new(mtcars)
app <- qc$app_obj()
shiny::runApp(app)
}\if{html}{\out{</div>}}
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{QueryChat$app_obj(..., bookmark_store = "url")}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{...}}{Additional arguments (currently unused).}
      \item{\code{bookmark_store}}{The bookmarking storage method. Passed to
\code{\link[shiny:enableBookmarking]{shiny::enableBookmarking()}}. If \code{"url"} or \code{"server"}, the chat state
(including current query) will be bookmarked. Default is \code{"url"}.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A Shiny app object that can be run with \code{shiny::runApp()}.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-QueryChat-sidebar"></a>}}
\if{latex}{\out{\hypertarget{method-QueryChat-sidebar}{}}}
\subsection{\code{QueryChat$sidebar()}}{
  Create a sidebar containing the querychat UI.

This method generates a \code{\link[bslib:sidebar]{bslib::sidebar()}} component containing the chat
interface, suitable for use with \code{\link[bslib:page_sidebar]{bslib::page_sidebar()}} or similar
layouts.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{qc <- QueryChat$new(mtcars)

ui <- page_sidebar(
  qc$sidebar(),
  # Main content here
)
}\if{html}{\out{</div>}}
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{QueryChat$sidebar(
  ...,
  width = 400,
  height = "100\%",
  fillable = TRUE,
  id = NULL
)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{...}}{Additional arguments passed to \code{\link[bslib:sidebar]{bslib::sidebar()}}.}
      \item{\code{width}}{Width of the sidebar in pixels. Default is 400.}
      \item{\code{height}}{Height of the sidebar. Default is "100\%".}
      \item{\code{fillable}}{Whether the sidebar should be fillable. Default is
\code{TRUE}.}
      \item{\code{id}}{Optional ID for the QueryChat instance. If not provided, will
use the ID provided at initialization. If using \verb{$sidebar()} in a Shiny
module, you'll need to provide \code{id = ns("your_id")} where \code{ns} is the
namespacing function from \code{\link[shiny:NS]{shiny::NS()}}.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A \code{\link[bslib:sidebar]{bslib::sidebar()}} UI component.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-QueryChat-ui"></a>}}
\if{latex}{\out{\hypertarget{method-QueryChat-ui}{}}}
\subsection{\code{QueryChat$ui()}}{
  Create the UI for the querychat chat interface.

This method generates the chat UI component. Typically you'll use
\verb{$sidebar()} instead, which wraps this in a sidebar layout.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{qc <- QueryChat$new(mtcars)

ui <- fluidPage(
  qc$ui()
)
}\if{html}{\out{</div>}}
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{QueryChat$ui(..., id = NULL)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{...}}{Additional arguments passed to \code{\link[shinychat:chat_ui]{shinychat::chat_ui()}}.}
      \item{\code{id}}{Optional ID for the QueryChat instance. If not provided,
will use the ID provided at initialization. If using \verb{$ui()} in a Shiny
module, you'll need to provide \code{id = ns("your_id")} where \code{ns} is the
namespacing function from \code{\link[shiny:NS]{shiny::NS()}}.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A UI component containing the chat interface.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-QueryChat-server"></a>}}
\if{latex}{\out{\hypertarget{method-QueryChat-server}{}}}
\subsection{\code{QueryChat$server()}}{
  Initialize the querychat server logic.

This method must be called within a Shiny server function. It sets up the
reactive logic for the chat interface and returns session-specific
reactive values.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{qc <- QueryChat$new(mtcars)

server <- function(input, output, session) \{
  qc_vals <- qc$server(enable_bookmarking = TRUE)

  output$data <- renderDataTable(qc_vals$df())
  output$query <- renderText(qc_vals$sql())
  output$title <- renderText(qc_vals$title() \%||\% "No Query")
\}
}\if{html}{\out{</div>}}
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{QueryChat$server(
  data_source = NULL,
  client = NULL,
  enable_bookmarking = FALSE,
  ...,
  id = NULL,
  session = shiny::getDefaultReactiveDomain()
)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{data_source}}{Optional data source to use. If provided, sets the
data_source property before initializing server logic. This is useful
for the deferred pattern where data_source is not known at
initialization time (e.g., when the data source depends on session-
specific authentication).}
      \item{\code{client}}{Optional chat client override for this session. Can be an
\link[ellmer:Chat]{ellmer::Chat} object or a string (e.g., \code{"openai/gpt-4o"}). If provided,
overrides the client set at initialization for this session only —
other sessions are unaffected. This is useful when the client must be
created within a session scope (e.g., Posit Connect managed credentials).}
      \item{\code{enable_bookmarking}}{Whether to enable bookmarking for the chat
state. Default is \code{FALSE}. When enabled, the chat state (including
current query, title, and chat history) will be saved and restored
with Shiny bookmarks. This requires that the Shiny app has bookmarking
enabled via \code{shiny::enableBookmarking()} or the \code{enableBookmarking}
parameter of \code{shiny::shinyApp()}.}
      \item{\code{...}}{Ignored.}
      \item{\code{id}}{Optional module ID for the QueryChat instance. If not provided,
will use the ID provided at initialization. When used in Shiny modules,
this \code{id} should match the \code{id} used in the corresponding UI function
(i.e., \code{qc$ui(id = ns("your_id"))} pairs with \code{qc$server(id = "your_id")}).}
      \item{\code{session}}{The Shiny session object.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A list containing session-specific reactive values and the chat
client with the following elements:
\itemize{
\item \code{df}: Reactive expression returning the current filtered data frame
\item \code{sql}: Reactive value for the current SQL query string
\item \code{title}: Reactive value for the current title
\item \code{client}: The session-specific chat client instance
}
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-QueryChat-generate_greeting"></a>}}
\if{latex}{\out{\hypertarget{method-QueryChat-generate_greeting}{}}}
\subsection{\code{QueryChat$generate_greeting()}}{
  Generate a welcome greeting for the chat.

By default, \code{QueryChat$new()} generates a greeting at the start of every
new conversation, which is convenient for getting started and
development, but also might add unnecessary latency and cost. Use this
method to generate a greeting once and save it for reuse.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# Create QueryChat object
qc <- QueryChat$new(mtcars)

# Generate a greeting and save it
greeting <- qc$generate_greeting()
writeLines(greeting, "mtcars_greeting.md")

# Later, use the saved greeting
qc2 <- QueryChat$new(mtcars, greeting = "mtcars_greeting.md")
}\if{html}{\out{</div>}}
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{QueryChat$generate_greeting(echo = c("none", "output"))}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{echo}}{Whether to print the greeting to the console. Options are
\code{"none"} (default, no output) or \code{"output"} (print to console).}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    The greeting string in Markdown format.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-QueryChat-cleanup"></a>}}
\if{latex}{\out{\hypertarget{method-QueryChat-cleanup}{}}}
\subsection{\code{QueryChat$cleanup()}}{
  Clean up resources associated with the data source.

This method releases any resources (e.g., database connections)
associated with the data source. Call this when you are done using the
QueryChat object to avoid resource leaks.

Note: If \code{auto_cleanup} was set to \code{TRUE} in the constructor, this will
be called automatically when the Shiny app stops.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{QueryChat$cleanup()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    Invisibly returns \code{NULL}. Resources are cleaned up internally.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-QueryChat-clone"></a>}}
\if{latex}{\out{\hypertarget{method-QueryChat-clone}{}}}
\subsection{\code{QueryChat$clone()}}{
  The objects of this class are cloneable with this method.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{QueryChat$clone(deep = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{deep}}{Whether to make a deep clone.}
    }
    \if{html}{\out{</div>}}
  }
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DBISource.R
\name{DBISource}
\alias{DBISource}
\title{DBI Source}
\description{
A DataSource implementation for DBI database connections (SQLite, PostgreSQL,
MySQL, etc.). This class wraps a DBI connection and provides SQL query
execution against a single table in the database.
}
\examples{
\dontshow{if (rlang::is_installed("RSQLite")) withAutoprint(\{ # examplesIf}
# Connect to a database
con <- DBI::dbConnect(RSQLite::SQLite(), ":memory:")
DBI::dbWriteTable(con, "mtcars", mtcars)

# Create a DBI source
db_source <- DBISource$new(con, "mtcars")

# Get database type
db_source$get_db_type()  # Returns "SQLite"

# Execute a query
result <- db_source$execute_query("SELECT * FROM mtcars WHERE mpg > 25")

# Note: cleanup() will disconnect the connection
# If you want to keep the connection open, don't call cleanup()
db_source$cleanup()
\dontshow{\}) # examplesIf}
}
\section{Super class}{
\code{\link[querychat:DataSource]{DataSource}} -> \code{DBISource}
}
\section{Methods}{
\subsection{Public methods}{
  \itemize{
    \item \href{#method-DBISource-initialize}{\code{DBISource$new()}}
    \item \href{#method-DBISource-get_db_type}{\code{DBISource$get_db_type()}}
    \item \href{#method-DBISource-get_schema}{\code{DBISource$get_schema()}}
    \item \href{#method-DBISource-get_semantic_views_description}{\code{DBISource$get_semantic_views_description()}}
    \item \href{#method-DBISource-execute_query}{\code{DBISource$execute_query()}}
    \item \href{#method-DBISource-test_query}{\code{DBISource$test_query()}}
    \item \href{#method-DBISource-get_data}{\code{DBISource$get_data()}}
    \item \href{#method-DBISource-cleanup}{\code{DBISource$cleanup()}}
    \item \href{#method-DBISource-clone}{\code{DBISource$clone()}}
  }
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DBISource-initialize"></a>}}
\if{latex}{\out{\hypertarget{method-DBISource-initialize}{}}}
\subsection{\code{DBISource$new()}}{
  Create a new DBISource
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DBISource$new(conn, table_name)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{conn}}{A DBI connection object}
      \item{\code{table_name}}{Name of the table in the database. Can be a character
string or a \code{\link[DBI:Id]{DBI::Id()}} object for tables in catalogs/schemas}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A new DBISource object
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DBISource-get_db_type"></a>}}
\if{latex}{\out{\hypertarget{method-DBISource-get_db_type}{}}}
\subsection{\code{DBISource$get_db_type()}}{
  Get the database type
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DBISource$get_db_type()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A string identifying the database type
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DBISource-get_schema"></a>}}
\if{latex}{\out{\hypertarget{method-DBISource-get_schema}{}}}
\subsection{\code{DBISource$get_schema()}}{
  Get schema information for the database table
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DBISource$get_schema(categorical_threshold = 20)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{categorical_threshold}}{Maximum number of unique values for a text
column to be considered categorical (default: 20)}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A string describing the schema
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DBISource-get_semantic_views_description"></a>}}
\if{latex}{\out{\hypertarget{method-DBISource-get_semantic_views_description}{}}}
\subsection{\code{DBISource$get_semantic_views_description()}}{
  Get information about semantic views (if any) for the system prompt.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DBISource$get_semantic_views_description()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A string with semantic view information, or empty string if none
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DBISource-execute_query"></a>}}
\if{latex}{\out{\hypertarget{method-DBISource-execute_query}{}}}
\subsection{\code{DBISource$execute_query()}}{
  Execute a SQL query
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DBISource$execute_query(query)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{query}}{SQL query string. If NULL or empty, returns all data}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A data frame with query results
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DBISource-test_query"></a>}}
\if{latex}{\out{\hypertarget{method-DBISource-test_query}{}}}
\subsection{\code{DBISource$test_query()}}{
  Test a SQL query by fetching only one row
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DBISource$test_query(query, require_all_columns = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{query}}{SQL query string}
      \item{\code{require_all_columns}}{If \code{TRUE}, validates that the result includes
all original table columns (default: \code{FALSE})}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A data frame with one row of results
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DBISource-get_data"></a>}}
\if{latex}{\out{\hypertarget{method-DBISource-get_data}{}}}
\subsection{\code{DBISource$get_data()}}{
  Get all data from the table
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DBISource$get_data()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A data frame containing all data
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DBISource-cleanup"></a>}}
\if{latex}{\out{\hypertarget{method-DBISource-cleanup}{}}}
\subsection{\code{DBISource$cleanup()}}{
  Disconnect from the database
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DBISource$cleanup()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    NULL (invisibly)
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DBISource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-DBISource-clone}{}}}
\subsection{\code{DBISource$clone()}}{
  The objects of this class are cloneable with this method.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DBISource$clone(deep = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{deep}}{Whether to make a deep clone.}
    }
    \if{html}{\out{</div>}}
  }
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TblSqlSource.R
\name{TblSqlSource}
\alias{TblSqlSource}
\title{Data Source: SQL Tibble}
\description{
A DataSource implementation for lazy SQL tibbles connected to databases via
\code{\link[dbplyr:tbl_sql]{dbplyr::tbl_sql()}} or \code{\link[dplyr:sql]{dplyr::sql()}}.
}
\examples{
\dontshow{if (rlang::is_installed("dbplyr") && rlang::is_installed("dplyr") && rlang::is_installed("duckdb")) withAutoprint(\{ # examplesIf}
con <- DBI::dbConnect(duckdb::duckdb())
DBI::dbWriteTable(con, "mtcars", mtcars)

mtcars_source <- TblSqlSource$new(dplyr::tbl(con, "mtcars"))
mtcars_source$get_db_type()  # "DuckDB"

result <- mtcars_source$execute_query("SELECT * FROM mtcars WHERE cyl > 4")

# Note, the result is not the *full* data frame, but a lazy SQL tibble
result

# You can chain this result into a dplyr pipeline
dplyr::count(result, cyl, gear)

# Or collect the entire data frame into local memory
dplyr::collect(result)

# Finally, clean up when done with the database (closes the DB connection)
mtcars_source$cleanup()
\dontshow{\}) # examplesIf}
}
\section{Super classes}{
\code{\link[querychat:DataSource]{DataSource}} -> \code{\link[querychat:DBISource]{DBISource}} -> \code{TblSqlSource}
}
\section{Public fields}{
  \if{html}{\out{<div class="r6-fields">}}
  \describe{
    \item{\code{table_name}}{Name of the table to be used in SQL queries}
  }
  \if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
  \itemize{
    \item \href{#method-TblSqlSource-initialize}{\code{TblSqlSource$new()}}
    \item \href{#method-TblSqlSource-get_db_type}{\code{TblSqlSource$get_db_type()}}
    \item \href{#method-TblSqlSource-get_schema}{\code{TblSqlSource$get_schema()}}
    \item \href{#method-TblSqlSource-execute_query}{\code{TblSqlSource$execute_query()}}
    \item \href{#method-TblSqlSource-test_query}{\code{TblSqlSource$test_query()}}
    \item \href{#method-TblSqlSource-prep_query}{\code{TblSqlSource$prep_query()}}
    \item \href{#method-TblSqlSource-get_data}{\code{TblSqlSource$get_data()}}
    \item \href{#method-TblSqlSource-cleanup}{\code{TblSqlSource$cleanup()}}
    \item \href{#method-TblSqlSource-clone}{\code{TblSqlSource$clone()}}
  }
}
\if{html}{\out{<details open><summary>Inherited methods</summary>
<ul>
  <li><span class="pkg-link" data-pkg="querychat" data-topic="DBISource" data-id="get_semantic_views_description"><a href='../../querychat/html/DBISource.html#method-DBISource-get_semantic_views_description'><code>DBISource$get_semantic_views_description()</code></a></span></li>
</ul>
</details>}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TblSqlSource-initialize"></a>}}
\if{latex}{\out{\hypertarget{method-TblSqlSource-initialize}{}}}
\subsection{\code{TblSqlSource$new()}}{
  Create a new TblSqlSource
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{TblSqlSource$new(tbl, table_name = missing_arg())}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{tbl}}{A \code{\link[dbplyr:tbl_sql]{dbplyr::tbl_sql()}} (or SQL tibble via \code{\link[dplyr:tbl]{dplyr::tbl()}}).}
      \item{\code{table_name}}{Name of the table in the database. Can be a character
string, or will be inferred from the \code{tbl} argument, if possible.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A new TblSqlSource object
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TblSqlSource-get_db_type"></a>}}
\if{latex}{\out{\hypertarget{method-TblSqlSource-get_db_type}{}}}
\subsection{\code{TblSqlSource$get_db_type()}}{
  Get the database type
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{TblSqlSource$get_db_type()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A string describing the database type (e.g., "DuckDB", "SQLite")
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TblSqlSource-get_schema"></a>}}
\if{latex}{\out{\hypertarget{method-TblSqlSource-get_schema}{}}}
\subsection{\code{TblSqlSource$get_schema()}}{
  Get schema information about the table
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{TblSqlSource$get_schema(categorical_threshold = 20)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{categorical_threshold}}{Maximum number of unique values for a text
column to be considered categorical}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A string containing schema information formatted for LLM prompts
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TblSqlSource-execute_query"></a>}}
\if{latex}{\out{\hypertarget{method-TblSqlSource-execute_query}{}}}
\subsection{\code{TblSqlSource$execute_query()}}{
  Execute a SQL query and return results
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{TblSqlSource$execute_query(query)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{query}}{SQL query string to execute}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A data frame containing query results
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TblSqlSource-test_query"></a>}}
\if{latex}{\out{\hypertarget{method-TblSqlSource-test_query}{}}}
\subsection{\code{TblSqlSource$test_query()}}{
  Test a SQL query by fetching only one row
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{TblSqlSource$test_query(query, require_all_columns = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{query}}{SQL query string to test}
      \item{\code{require_all_columns}}{If \code{TRUE}, validates that the result includes
all original table columns (default: \code{FALSE})}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A data frame containing one row of results (or empty if no matches)
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TblSqlSource-prep_query"></a>}}
\if{latex}{\out{\hypertarget{method-TblSqlSource-prep_query}{}}}
\subsection{\code{TblSqlSource$prep_query()}}{
  Prepare a generic \verb{SELECT * FROM ____} query to work with the SQL tibble
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{TblSqlSource$prep_query(query)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{query}}{SQL query as a string}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A complete SQL query string
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TblSqlSource-get_data"></a>}}
\if{latex}{\out{\hypertarget{method-TblSqlSource-get_data}{}}}
\subsection{\code{TblSqlSource$get_data()}}{
  Get the unfiltered data as a SQL tibble
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{TblSqlSource$get_data()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A \code{\link[dbplyr:tbl_sql]{dbplyr::tbl_sql()}} containing the original, unfiltered data
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TblSqlSource-cleanup"></a>}}
\if{latex}{\out{\hypertarget{method-TblSqlSource-cleanup}{}}}
\subsection{\code{TblSqlSource$cleanup()}}{
  Clean up resources (close connections, etc.)
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{TblSqlSource$cleanup()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    NULL (invisibly)
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TblSqlSource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-TblSqlSource-clone}{}}}
\subsection{\code{TblSqlSource$clone()}}{
  The objects of this class are cloneable with this method.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{TblSqlSource$clone(deep = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{deep}}{Whether to make a deep clone.}
    }
    \if{html}{\out{</div>}}
  }
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/querychat_viz.R
\name{extract_visualise_table}
\alias{extract_visualise_table}
\title{Extract the table name from a VISUALISE clause's FROM, if present}
\usage{
extract_visualise_table(visual)
}
\arguments{
\item{visual}{A ggsql VISUALISE string.}
}
\value{
The table name string (possibly quoted), or \code{NULL} if not present.
}
\description{
Looks only in the portion of the visual string before the first DRAW keyword,
so FROM clauses inside DRAW (e.g., MAPPING x FROM other) are ignored.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DataFrameSource.R
\name{DataFrameSource}
\alias{DataFrameSource}
\title{Data Frame Source}
\description{
A DataSource implementation that wraps a data frame using DuckDB or SQLite
for SQL query execution.
}
\details{
This class creates an in-memory database connection and registers the
provided data frame as a table. All SQL queries are executed against this
database table. See \link{DBISource} for the full description of available
methods.

By default, DataFrameSource uses the first available engine from duckdb
(checked first) or RSQLite. You can explicitly set the \code{engine} parameter to
choose between \code{"duckdb"} or \code{"sqlite"}, or set the global option
\code{querychat.DataFrameSource.engine} to choose the default engine for all
DataFrameSource instances. At least one of these packages must be installed.
}
\examples{
\dontshow{if (rlang::is_installed("duckdb") || rlang::is_installed("RSQLite")) withAutoprint(\{ # examplesIf}
# Create a data frame source (uses first available: duckdb or sqlite)
df_source <- DataFrameSource$new(mtcars, "mtcars")

# Get database type
df_source$get_db_type()  # Returns "DuckDB" or "SQLite"

# Execute a query
result <- df_source$execute_query("SELECT * FROM mtcars WHERE mpg > 25")

# Explicitly choose an engine
df_sqlite <- DataFrameSource$new(mtcars, "mtcars", engine = "sqlite")

# Clean up when done
df_source$cleanup()
df_sqlite$cleanup()
\dontshow{\}) # examplesIf}
}
\section{Super classes}{
\code{\link[querychat:DataSource]{DataSource}} -> \code{\link[querychat:DBISource]{DBISource}} -> \code{DataFrameSource}
}
\section{Methods}{
\subsection{Public methods}{
  \itemize{
    \item \href{#method-DataFrameSource-initialize}{\code{DataFrameSource$new()}}
    \item \href{#method-DataFrameSource-clone}{\code{DataFrameSource$clone()}}
  }
}
\if{html}{\out{<details><summary>Inherited methods</summary>
<ul>
  <li><span class="pkg-link" data-pkg="querychat" data-topic="DBISource" data-id="cleanup"><a href='../../querychat/html/DBISource.html#method-DBISource-cleanup'><code>DBISource$cleanup()</code></a></span></li>
  <li><span class="pkg-link" data-pkg="querychat" data-topic="DBISource" data-id="execute_query"><a href='../../querychat/html/DBISource.html#method-DBISource-execute_query'><code>DBISource$execute_query()</code></a></span></li>
  <li><span class="pkg-link" data-pkg="querychat" data-topic="DBISource" data-id="get_data"><a href='../../querychat/html/DBISource.html#method-DBISource-get_data'><code>DBISource$get_data()</code></a></span></li>
  <li><span class="pkg-link" data-pkg="querychat" data-topic="DBISource" data-id="get_db_type"><a href='../../querychat/html/DBISource.html#method-DBISource-get_db_type'><code>DBISource$get_db_type()</code></a></span></li>
  <li><span class="pkg-link" data-pkg="querychat" data-topic="DBISource" data-id="get_schema"><a href='../../querychat/html/DBISource.html#method-DBISource-get_schema'><code>DBISource$get_schema()</code></a></span></li>
  <li><span class="pkg-link" data-pkg="querychat" data-topic="DBISource" data-id="get_semantic_views_description"><a href='../../querychat/html/DBISource.html#method-DBISource-get_semantic_views_description'><code>DBISource$get_semantic_views_description()</code></a></span></li>
  <li><span class="pkg-link" data-pkg="querychat" data-topic="DBISource" data-id="test_query"><a href='../../querychat/html/DBISource.html#method-DBISource-test_query'><code>DBISource$test_query()</code></a></span></li>
</ul>
</details>}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataFrameSource-initialize"></a>}}
\if{latex}{\out{\hypertarget{method-DataFrameSource-initialize}{}}}
\subsection{\code{DataFrameSource$new()}}{
  Create a new DataFrameSource
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DataFrameSource$new(
  df,
  table_name,
  engine = getOption("querychat.DataFrameSource.engine", NULL)
)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{df}}{A data frame.}
      \item{\code{table_name}}{Name to use for the table in SQL queries. Must be a
valid table name (start with letter, contain only letters, numbers,
and underscores)}
      \item{\code{engine}}{Database engine to use: "duckdb" or "sqlite". Set the
global option \code{querychat.DataFrameSource.engine} to specify the default
engine for all instances. If NULL (default), uses the first available
engine from duckdb or RSQLite (in that order).}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A new DataFrameSource object
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataFrameSource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-DataFrameSource-clone}{}}}
\subsection{\code{DataFrameSource$clone()}}{
  The objects of this class are cloneable with this method.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DataFrameSource$clone(deep = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{deep}}{Whether to make a deep clone.}
    }
    \if{html}{\out{</div>}}
  }
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/QueryChat.R
\name{querychat}
\alias{querychat}
\alias{querychat_app}
\title{QueryChat convenience functions}
\usage{
querychat(
  data_source,
  table_name = missing_arg(),
  ...,
  id = NULL,
  greeting = NULL,
  client = NULL,
  tools = c("filter", "query"),
  data_description = NULL,
  categorical_threshold = 20,
  extra_instructions = NULL,
  prompt_template = NULL,
  cleanup = NA
)

querychat_app(
  data_source,
  table_name = missing_arg(),
  ...,
  id = NULL,
  greeting = NULL,
  client = NULL,
  tools = c("filter", "query"),
  data_description = NULL,
  categorical_threshold = 20,
  extra_instructions = NULL,
  prompt_template = NULL,
  cleanup = NA,
  bookmark_store = "url"
)
}
\arguments{
\item{data_source}{Either a data.frame or a database connection (e.g., DBI
connection).}

\item{table_name}{A string specifying the table name to use in SQL queries.
If \code{data_source} is a data.frame, this is the name to refer to it by in
queries (typically the variable name). If not provided, will be inferred
from the variable name for data.frame inputs. For database connections,
this parameter is required.}

\item{...}{Additional arguments (currently unused).}

\item{id}{Optional module ID for the QueryChat instance. If not provided,
will be auto-generated from \code{table_name}. The ID is used to namespace
the Shiny module.}

\item{greeting}{Optional initial message to display to users. Can be a
character string (in Markdown format) or a file path. If not provided,
a greeting will be generated at the start of each conversation using the
LLM, which adds latency and cost. Use \verb{$generate_greeting()} to create
a greeting to save and reuse.}

\item{client}{Optional chat client. Can be:
\itemize{
\item An \link[ellmer:Chat]{ellmer::Chat} object
\item A string to pass to \code{\link[ellmer:chat]{ellmer::chat()}} (e.g., \code{"openai/gpt-4o"})
\item \code{NULL} (default): Uses the \code{querychat.client} option, the
\code{QUERYCHAT_CLIENT} environment variable, or defaults to
\code{\link[ellmer:chat_openai]{ellmer::chat_openai()}}
}}

\item{tools}{Which querychat tools to include in the chat client, by
default. \code{"filter"} includes the tools for filtering and resetting the
dashboard and \code{"query"} includes the tool for executing SQL queries.
Use \code{tools = "filter"} when you only want the dashboard filtering tools,
or when you want to disable the querying tool entirely to prevent the
LLM from seeing any of the data in your dataset. The legacy name
\code{"update"} is still accepted as an alias for \code{"filter"}.}

\item{data_description}{Optional description of the data in plain text or
Markdown. Can be a string or a file path. This provides context to the
LLM about what the data represents.}

\item{categorical_threshold}{For text columns, the maximum number of unique
values to consider as a categorical variable. Default is 20.}

\item{extra_instructions}{Optional additional instructions for the chat
model in plain text or Markdown. Can be a string or a file path.}

\item{prompt_template}{Optional path to or string of a custom prompt
template file. If not provided, the default querychat template will be
used. See the package prompts directory for the default template format.}

\item{cleanup}{Whether or not to automatically run \verb{$cleanup()} when the
Shiny session/app stops. By default, cleanup only occurs if \code{QueryChat}
is created within a Shiny app. Set to \code{TRUE} to always clean up, or
\code{FALSE} to never clean up automatically.

In \code{querychat_app()}, in-memory databases created for data frames are
always cleaned up.}

\item{bookmark_store}{The bookmarking storage method. Passed to
\code{\link[shiny:enableBookmarking]{shiny::enableBookmarking()}}. If \code{"url"} or \code{"server"}, the chat state
(including current query) will be bookmarked. Default is \code{"url"}.}
}
\value{
A \code{QueryChat} object. See \link{QueryChat} for available methods.

Invisibly returns the chat object after the app stops.
}
\description{
Convenience functions for wrapping \link{QueryChat} creation (i.e., \code{querychat()})
and app launching (i.e., \code{querychat_app()}).
}
\examples{
\dontshow{if (rlang::is_interactive() && rlang::is_installed("RSQLite")) withAutoprint(\{ # examplesIf}
# Quick start - chat with mtcars dataset in one line
querychat_app(mtcars)

# Add options
querychat_app(
  mtcars,
  greeting = "Welcome to the mtcars explorer!",
  client = "openai/gpt-4o"
)

# Chat with a database table (table_name required)
con <- DBI::dbConnect(RSQLite::SQLite(), ":memory:")
DBI::dbWriteTable(con, "mtcars", mtcars)
querychat_app(con, "mtcars")

# Create QueryChat class object
qc <- querychat(mtcars, greeting = "Welcome to the mtcars explorer!")

# Run the app later
qc$app()
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/querychat_viz.R
\name{execute_ggsql}
\alias{execute_ggsql}
\title{Execute a pre-validated ggsql query against a DataSource}
\usage{
execute_ggsql(data_source, validated)
}
\arguments{
\item{data_source}{A querychat DataSource R6 object.}

\item{validated}{A pre-validated ggsql query (from \code{ggsql::ggsql_validate()}).
Must be a list with \verb{$sql} and \verb{$visual} fields.}
}
\value{
A \code{ggsql::Spec} R6 object (the writer-independent plot specification).
}
\description{
Executes the SQL portion through a DataSource (preserving database pushdown),
then feeds the result into a ggsql DuckDB reader to produce a Spec.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{deprecated}
\alias{deprecated}
\alias{querychat_init}
\alias{querychat_sidebar}
\alias{querychat_ui}
\alias{querychat_server}
\alias{querychat_greeting}
\alias{querychat_data_source}
\title{Deprecated functions}
\usage{
querychat_init(...)

querychat_sidebar(...)

querychat_ui(...)

querychat_server(...)

querychat_greeting(...)

querychat_data_source(...)
}
\value{
Please see the updated function details for return values.
}
\description{
These functions have been replaced by the new \code{QueryChat} R6 class API.
Please update your code to use the new class-based approach.
}
\section{Functions}{
\itemize{
\item \code{querychat_init()}: was replaced with the \verb{$new()} method of \link{QueryChat}.

\item \code{querychat_sidebar()}: was replaced with the \verb{$sidebar()} method of
\link{QueryChat}.

\item \code{querychat_ui()}: was replaced with the \verb{$ui()} method of \link{QueryChat}.

\item \code{querychat_server()}: was replaced with the \verb{$server()} method of
\link{QueryChat}.

\item \code{querychat_greeting()}: was replaced with the \verb{$generate_greeting()} method of
\link{QueryChat}.

\item \code{querychat_data_source()}: was replaced with the \verb{$new()} method of \link{QueryChat}.

}}
\keyword{internal}
